@@ -99,10 +99,10 @@ def drawfunction(xstart, xend, ybottom, ytop, samples, width, height, left, bott
     x0 =   xstart
     y0 = f(xstart)
     if polar :
-      xp0 = y0 * cos( x0 )
-      yp0 = y0 * sin( x0 )
-      x0 = xp0
-      y0 = yp0
+        xp0 = y0 * cos( x0 )
+        yp0 = y0 * sin( x0 )
+        x0 = xp0
+        y0 = yp0
     if fponum or polar: # numerical derivative, using 0.001*step as the small differential
         x1 = xstart + ds # Second point AFTER first point (Good for first point)
         y1 = f(x1)
@@ -114,7 +114,7 @@ def drawfunction(xstart, xend, ybottom, ytop, samples, width, height, left, bott
         dx0 = (x1 - x0)/ds 
         dy0 = (y1 - y0)/ds
     else: # derivative given by the user
-        dx0 = 0 # Only works for rectangular coordinates
+        dx0 = 1 # Only works for rectangular coordinates
         dy0 = fp(xstart)
 
     # Start curve
@@ -138,7 +138,7 @@ def drawfunction(xstart, xend, ybottom, ytop, samples, width, height, left, bott
             dx1 = (x1 - x2)/ds
             dy1 = (y1 - y2)/ds
         else: # derivative given by the user
-            dx1 = 0 # Only works for rectangular coordinates
+            dx1 = 1 # Only works for rectangular coordinates
             dy1 = fp(x1)
         # create curve
         a.append([' C ',
@@ -269,5 +269,9 @@ class FuncPlot(inkex.Effect):
                 if self.options.remove:
                     node.getparent().remove(node)
                 
-e = FuncPlot()
-e.affect()
+if __name__ == '__main__':
+    e = FuncPlot()
+    e.affect()
+
+
+# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 encoding=utf-8 textwidth=99
