@@ -12,31 +12,39 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include <string>
+#include <cstring>
+
 #include "desktop.h"
 #include "color-rgba.h"
 #include "svg/css-ostringstream.h"
 #include "svg/svg.h"
 #include "svg/svg-color.h"
 #include "selection.h"
-#include "sp-tspan.h"
-#include "sp-textpath.h"
 #include "inkscape.h"
 #include "style.h"
-#include "prefs-utils.h"
+#include "preferences.h"
 #include "sp-use.h"
+#include "filters/blend.h"
 #include "sp-filter.h"
+#include "sp-filter-reference.h"
 #include "sp-gaussian-blur.h"
 #include "sp-flowtext.h"
 #include "sp-flowregion.h"
 #include "sp-flowdiv.h"
 #include "sp-linear-gradient.h"
-#include "sp-radial-gradient.h"
 #include "sp-pattern.h"
+#include "sp-radial-gradient.h"
+#include "sp-textpath.h"
+#include "sp-tref.h"
+#include "sp-tspan.h"
 #include "xml/repr.h"
 #include "libnrtype/font-style-to-pos.h"
-
+#include "sp-path.h"
 
 #include "desktop-style.h"
+#include "svg/svg-icc-color.h"
+#include "box3d-side.h"
 
 /**
  * Set color on selection on desktop.
@@ -52,7 +60,7 @@ sp_desktop_set_color(SPDesktop *desktop, ColorRGBA const &color, bool is_relativ
 
     guint32 rgba = SP_RGBA32_F_COMPOSE(color[0], color[1], color[2], color[3]);
     gchar b[64];
-    sp_svg_write_color(b, 64, rgba);
+    sp_svg_write_color(b, sizeof(b), rgba);
     SPCSSAttr *css = sp_repr_css_attr_new();
     if (fill) {
         sp_repr_css_set_property(css, "fill", b);
@@ -112,8 +120,8 @@ sp_desktop_apply_css_recursive(SPObject *o, SPCSSAttr *css, bool skip_lines)
 
         // Scale the style by the inverse of the accumulated parent transform in the paste context.
         {
-            NR::Matrix const local(sp_item_i2doc_affine(SP_ITEM(o)));
-            double const ex(NR::expansion(local));
+            Geom::Matrix const local(sp_item_i2doc_affine(SP_ITEM(o)));
+            double const ex(local.descrim());
             if ( ( ex != 0. )
                  && ( ex != 1. ) ) {
                 sp_css_attr_scale(css_set, 1/ex);
@@ -151,14 +159,25 @@ void
 sp_desktop_set_style(SPDesktop *desktop, SPCSSAttr *css, bool change, bool write_current)
 {
     if (write_current) {
-// 1. Set internal value
+        Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+        // 1. Set internal value
         sp_repr_css_merge(desktop->current, css);
 
-// 1a. Write to prefs; make a copy and unset any URIs first
+        // 1a. Write to prefs; make a copy and unset any URIs first
         SPCSSAttr *css_write = sp_repr_css_attr_new();
         sp_repr_css_merge(css_write, css);
         sp_css_attr_unset_uris(css_write);
-        sp_repr_css_change(inkscape_get_repr(INKSCAPE, "desktop"), css_write, "style");
+        prefs->mergeStyle("/desktop/style", css_write);
+
+        for (const GSList *i = desktop->selection->itemList(); i != NULL; i = i->next) {
+            /* last used styles for 3D box faces are stored separately */
+            if (SP_IS_BOX3D_SIDE (i->data)) {
+                const char * descr  = box3d_side_axes_string(SP_BOX3D_SIDE(i->data));
+                if (descr != NULL) {
+                    prefs->mergeStyle(Glib::ustring("/desktop/") + descr + "/style", css_write);
+                }
+            }
+        }
         sp_repr_css_attr_unref(css_write);
     }
 
@@ -224,25 +243,28 @@ sp_desktop_get_color(SPDesktop *desktop, bool is_fill)
 }
 
 double
-sp_desktop_get_master_opacity_tool(SPDesktop *desktop, char const *tool)
+sp_desktop_get_master_opacity_tool(SPDesktop *desktop, Glib::ustring const &tool, bool *has_opacity)
 {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
     SPCSSAttr *css = NULL;
     gfloat value = 1.0; // default if nothing else found
-    if (prefs_get_double_attribute(tool, "usecurrent", 0) != 0) {
+    if (has_opacity)
+        *has_opacity = false;
+    if (prefs->getBool(tool + "/usecurrent")) {
         css = sp_desktop_get_style(desktop, true);
-    } else { 
-        Inkscape::XML::Node *tool_repr = inkscape_get_repr(INKSCAPE, tool);
-        if (tool_repr) {
-            css = sp_repr_css_attr_inherited(tool_repr, "style");
-        }
+    } else {
+        css = prefs->getStyle(tool + "/style");
     }
-   
+
     if (css) {
         gchar const *property = css ? sp_repr_css_property(css, "opacity", "1.000") : 0;
-           
+
         if (desktop->current && property) { // if there is style and the property in it,
             if ( !sp_svg_number_read_f(property, &value) ) {
                 value = 1.0; // things failed. set back to the default
+            } else {
+                if (has_opacity)
+                   *has_opacity = false;
             }
         }
 
@@ -252,22 +274,20 @@ sp_desktop_get_master_opacity_tool(SPDesktop *desktop, char const *tool)
     return value;
 }
 double
-sp_desktop_get_opacity_tool(SPDesktop *desktop, char const *tool, bool is_fill)
+sp_desktop_get_opacity_tool(SPDesktop *desktop, Glib::ustring const &tool, bool is_fill)
 {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
     SPCSSAttr *css = NULL;
     gfloat value = 1.0; // default if nothing else found
-    if (prefs_get_double_attribute(tool, "usecurrent", 0) != 0) {
+    if (prefs->getBool(tool + "/usecurrent")) {
         css = sp_desktop_get_style(desktop, true);
-    } else { 
-        Inkscape::XML::Node *tool_repr = inkscape_get_repr(INKSCAPE, tool);
-        if (tool_repr) {
-            css = sp_repr_css_attr_inherited(tool_repr, "style");
-        }
+    } else {
+        css = prefs->getStyle(tool + "/style");
     }
-   
+
     if (css) {
         gchar const *property = css ? sp_repr_css_property(css, is_fill ? "fill-opacity": "stroke-opacity", "1.000") : 0;
-           
+
         if (desktop->current && property) { // if there is style and the property in it,
             if ( !sp_svg_number_read_f(property, &value) ) {
                 value = 1.0; // things failed. set back to the default
@@ -279,27 +299,30 @@ sp_desktop_get_opacity_tool(SPDesktop *desktop, char const *tool, bool is_fill)
 
     return value;
 }
+
 guint32
-sp_desktop_get_color_tool(SPDesktop *desktop, char const *tool, bool is_fill)
+sp_desktop_get_color_tool(SPDesktop *desktop, Glib::ustring const &tool, bool is_fill, bool *has_color)
 {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
     SPCSSAttr *css = NULL;
     guint32 r = 0; // if there's no color, return black
-    if (prefs_get_int_attribute(tool, "usecurrent", 0) != 0) {
+    if (has_color)
+        *has_color = false;
+    if (prefs->getBool(tool + "/usecurrent")) {
         css = sp_desktop_get_style(desktop, true);
     } else {
-        Inkscape::XML::Node *tool_repr = inkscape_get_repr(INKSCAPE, tool);
-        if (tool_repr) {
-            css = sp_repr_css_attr_inherited(tool_repr, "style");
-        }
+        css = prefs->getStyle(tool + "/style");
     }
-   
+
     if (css) {
         gchar const *property = sp_repr_css_property(css, is_fill ? "fill" : "stroke", "#000");
-           
+
         if (desktop->current && property) { // if there is style and the property in it,
-            if (strncmp(property, "url", 3)) { // and if it's not url,
+            if (strncmp(property, "url", 3) && strncmp(property, "none", 4)) { // and if it's not url or none,
                 // read it
                 r = sp_svg_read_color(property, r);
+                if (has_color)
+                    *has_color = true;
             }
         }
 
@@ -308,22 +331,22 @@ sp_desktop_get_color_tool(SPDesktop *desktop, char const *tool, bool is_fill)
 
     return r | 0xff;
 }
+
 /**
  * Apply the desktop's current style or the tool style to repr.
  */
 void
-sp_desktop_apply_style_tool(SPDesktop *desktop, Inkscape::XML::Node *repr, char const *tool, bool with_text)
+sp_desktop_apply_style_tool(SPDesktop *desktop, Inkscape::XML::Node *repr, Glib::ustring const &tool_path, bool with_text)
 {
     SPCSSAttr *css_current = sp_desktop_get_style(desktop, with_text);
-    if ((prefs_get_int_attribute(tool, "usecurrent", 0) != 0) && css_current) {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+
+    if (prefs->getBool(tool_path + "/usecurrent") && css_current) {
         sp_repr_css_set(repr, css_current, "style");
     } else {
-        Inkscape::XML::Node *tool_repr = inkscape_get_repr(INKSCAPE, tool);
-        if (tool_repr) {
-            SPCSSAttr *css = sp_repr_css_attr_inherited(tool_repr, "style");
-            sp_repr_css_set(repr, css, "style");
-            sp_repr_css_attr_unref(css);
-        }
+        SPCSSAttr *css = prefs->getInheritedStyle(tool_path + "/style");
+        sp_repr_css_set(repr, css, "style");
+        sp_repr_css_attr_unref(css);
     }
     if (css_current) {
         sp_repr_css_attr_unref(css_current);
@@ -337,21 +360,20 @@ sp_desktop_apply_style_tool(SPDesktop *desktop, Inkscape::XML::Node *repr, char
 double
 sp_desktop_get_font_size_tool(SPDesktop *desktop)
 {
-    gchar const *desktop_style = inkscape_get_repr(INKSCAPE, "desktop")->attribute("style");
-    gchar const *style_str = NULL;
-    if ((prefs_get_int_attribute("tools.text", "usecurrent", 0) != 0) && desktop_style) {
+    (void)desktop; // TODO cleanup
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+    Glib::ustring desktop_style = prefs->getString("/desktop/style");
+    Glib::ustring style_str;
+    if ((prefs->getBool("/tools/text/usecurrent")) && !desktop_style.empty()) {
         style_str = desktop_style;
     } else {
-        Inkscape::XML::Node *tool_repr = inkscape_get_repr(INKSCAPE, "tools.text");
-        if (tool_repr) {
-            style_str = tool_repr->attribute("style");
-        }
+        style_str = prefs->getString("/tools/text/style");
     }
 
     double ret = 12;
-    if (style_str) {
-        SPStyle *style = sp_style_new();
-        sp_style_merge_from_style_string(style, style_str);
+    if (!style_str.empty()) {
+        SPStyle *style = sp_style_new(SP_ACTIVE_DOCUMENT);
+        sp_style_merge_from_style_string(style, style_str.data());
         ret = style->font_size.computed;
         sp_style_unref(style);
     }
@@ -374,18 +396,18 @@ stroke_average_width (GSList const *objects)
         if (!SP_IS_ITEM (l->data))
             continue;
 
-        NR::Matrix i2d = sp_item_i2d_affine (SP_ITEM(l->data));
+        Geom::Matrix i2d = sp_item_i2d_affine (SP_ITEM(l->data));
 
         SPObject *object = SP_OBJECT(l->data);
 
-        if ( object->style->stroke.type == SP_PAINT_TYPE_NONE ) {
+        if ( object->style->stroke.isNone() ) {
             ++n_notstroked;   // do not count nonstroked objects
             continue;
         } else {
             notstroked = false;
         }
 
-        avgwidth += SP_OBJECT_STYLE (object)->stroke_width.computed * i2d.expansion();
+        avgwidth += SP_OBJECT_STYLE (object)->stroke_width.computed * i2d.descrim();
     }
 
     if (notstroked)
@@ -406,15 +428,17 @@ objects_query_fillstroke (GSList *objects, SPStyle *style_res, bool const isfill
     }
 
     SPIPaint *paint_res = isfill? &style_res->fill : &style_res->stroke;
-    paint_res->type = SP_PAINT_TYPE_IMPOSSIBLE;
+    bool paintImpossible = true;
     paint_res->set = TRUE;
 
+    SVGICCColor* iccColor = 0;
+    bool iccSeen = false;
     gfloat c[4];
     c[0] = c[1] = c[2] = c[3] = 0.0;
     gint num = 0;
 
-    gfloat prev[4];
-    prev[0] = prev[1] = prev[2] = prev[3] = 0.0;
+    gfloat prev[3];
+    prev[0] = prev[1] = prev[2] = 0.0;
     bool same_color = true;
 
     for (GSList const *i = objects; i != NULL; i = i->next) {
@@ -426,15 +450,18 @@ objects_query_fillstroke (GSList *objects, SPStyle *style_res, bool const isfill
 
         // We consider paint "effectively set" for anything within text hierarchy
         SPObject *parent = SP_OBJECT_PARENT (obj);
-        bool paint_effectively_set = paint->set || (SP_IS_TEXT(parent) || SP_IS_TEXTPATH(parent) || SP_IS_TSPAN(parent) || SP_IS_FLOWTEXT(parent) || SP_IS_FLOWDIV(parent) || SP_IS_FLOWPARA(parent) || SP_IS_FLOWTSPAN(parent) || SP_IS_FLOWLINE(parent));
+        bool paint_effectively_set =
+            paint->set || (SP_IS_TEXT(parent) || SP_IS_TEXTPATH(parent) || SP_IS_TSPAN(parent)
+            || SP_IS_FLOWTEXT(parent) || SP_IS_FLOWDIV(parent) || SP_IS_FLOWPARA(parent)
+            || SP_IS_FLOWTSPAN(parent) || SP_IS_FLOWLINE(parent));
 
         // 1. Bail out with QUERY_STYLE_MULTIPLE_DIFFERENT if necessary
 
-        if ((paint_res->type != SP_PAINT_TYPE_IMPOSSIBLE) && (paint->type != paint_res->type || (paint_res->set != paint_effectively_set))) {
+        if ((!paintImpossible) && (!paint->isSameType(*paint_res) || (paint_res->set != paint_effectively_set))) {
             return QUERY_STYLE_MULTIPLE_DIFFERENT;  // different types of paint
         }
 
-        if (paint_res->set && paint->set && paint_res->type == SP_PAINT_TYPE_PAINTSERVER) {
+        if (paint_res->set && paint->set && paint_res->isPaintserver()) {
             // both previous paint and this paint were a server, see if the servers are compatible
 
             SPPaintServer *server_res = isfill? SP_STYLE_FILL_SERVER (style_res) : SP_STYLE_STROKE_SERVER (style_res);
@@ -474,20 +501,27 @@ objects_query_fillstroke (GSList *objects, SPStyle *style_res, bool const isfill
 
         // 2. Sum color, copy server from paint to paint_res
 
-        if (paint_res->set && paint_effectively_set && paint->type == SP_PAINT_TYPE_COLOR) {
-
+        if (paint_res->set && paint_effectively_set && paint->isColor()) {
             gfloat d[3];
             sp_color_get_rgb_floatv (&paint->value.color, d);
 
             // Check if this color is the same as previous
-            if (paint_res->type == SP_PAINT_TYPE_IMPOSSIBLE) {
+            if (paintImpossible) {
                 prev[0] = d[0];
                 prev[1] = d[1];
                 prev[2] = d[2];
-                prev[3] = d[3];
+                paint_res->setColor(d[0], d[1], d[2]);
+                iccColor = paint->value.color.icc;
+                iccSeen = true;
             } else {
-                if (same_color && (prev[0] != d[0] || prev[1] != d[1] || prev[2] != d[2] || prev[3] != d[3]))
+                if (same_color && (prev[0] != d[0] || prev[1] != d[1] || prev[2] != d[2]))
                     same_color = false;
+                if ( iccSeen ) {
+                    if(paint->value.color.icc) {
+                        // TODO fix this
+                        iccColor = 0;
+                    }
+                }
             }
 
             // average color
@@ -498,32 +532,42 @@ objects_query_fillstroke (GSList *objects, SPStyle *style_res, bool const isfill
             num ++;
         }
 
-       if (paint_res->set && paint_effectively_set && paint->type == SP_PAINT_TYPE_PAINTSERVER) { // copy the server
+       paintImpossible = false;
+       paint_res->colorSet = paint->colorSet;
+       paint_res->currentcolor = paint->currentcolor;
+       if (paint_res->set && paint_effectively_set && paint->isPaintserver()) { // copy the server
            if (isfill) {
-               SP_STYLE_FILL_SERVER (style_res) = SP_STYLE_FILL_SERVER (style);
+               sp_style_set_to_uri_string (style_res, true, style->getFillURI());
            } else {
-               SP_STYLE_STROKE_SERVER (style_res) = SP_STYLE_STROKE_SERVER (style);
+               sp_style_set_to_uri_string (style_res, false, style->getStrokeURI());
            }
        }
-       paint_res->type = paint->type;
        paint_res->set = paint_effectively_set;
        style_res->fill_rule.computed = style->fill_rule.computed; // no averaging on this, just use the last one
     }
 
     // After all objects processed, divide the color if necessary and return
-    if (paint_res->set && paint_res->type == SP_PAINT_TYPE_COLOR) { // set the color
+    if (paint_res->set && paint_res->isColor()) { // set the color
         g_assert (num >= 1);
 
         c[0] /= num;
         c[1] /= num;
         c[2] /= num;
         c[3] /= num;
-        sp_color_set_rgb_float(&paint_res->value.color, c[0], c[1], c[2]);
+        paint_res->setColor(c[0], c[1], c[2]);
         if (isfill) {
             style_res->fill_opacity.value = SP_SCALE24_FROM_FLOAT (c[3]);
         } else {
             style_res->stroke_opacity.value = SP_SCALE24_FROM_FLOAT (c[3]);
         }
+
+
+        if ( iccSeen && iccColor ) {
+            // TODO check for existing
+            SVGICCColor* tmp = new SVGICCColor(*iccColor);
+            paint_res->value.color.icc = tmp;
+        }
+
         if (num > 1) {
             if (same_color)
                 return QUERY_STYLE_MULTIPLE_SAME;
@@ -603,6 +647,7 @@ objects_query_strokewidth (GSList *objects, SPStyle *style_res)
 
     gdouble prev_sw = -1;
     bool same_sw = true;
+    bool noneSet = true; // is stroke set to none?
 
     int n_stroked = 0;
 
@@ -612,14 +657,21 @@ objects_query_strokewidth (GSList *objects, SPStyle *style_res)
         SPStyle *style = SP_OBJECT_STYLE (obj);
         if (!style) continue;
 
-        if ( style->stroke.type == SP_PAINT_TYPE_NONE ) {
+        if ( style->stroke.isNone() && !(
+				style->marker[SP_MARKER_LOC].set || // stroke width affects markers, so if there's no stroke but only markers then we should
+				style->marker[SP_MARKER_LOC_START].set || // still calculate the stroke width
+				style->marker[SP_MARKER_LOC_MID].set ||
+				style->marker[SP_MARKER_LOC_END].set))
+		{
             continue;
         }
 
         n_stroked ++;
 
-        NR::Matrix i2d = sp_item_i2d_affine (SP_ITEM(obj));
-        double sw = style->stroke_width.computed * i2d.expansion();
+        noneSet &= style->stroke.isNone();
+
+        Geom::Matrix i2d = sp_item_i2d_affine (SP_ITEM(obj));
+        double sw = style->stroke_width.computed * i2d.descrim();
 
         if (prev_sw != -1 && fabs(sw - prev_sw) > 1e-3)
             same_sw = false;
@@ -633,6 +685,7 @@ objects_query_strokewidth (GSList *objects, SPStyle *style_res)
 
     style_res->stroke_width.computed = avgwidth;
     style_res->stroke_width.set = true;
+    style_res->stroke.noneSet = noneSet; // Will only be true if none of the selected objects has it's stroke set.
 
     if (n_stroked == 0) {
         return QUERY_STYLE_NOTHING;
@@ -669,7 +722,7 @@ objects_query_miterlimit (GSList *objects, SPStyle *style_res)
         SPStyle *style = SP_OBJECT_STYLE (obj);
         if (!style) continue;
 
-        if ( style->stroke.type == SP_PAINT_TYPE_NONE ) {
+        if ( style->stroke.isNone() ) {
             continue;
         }
 
@@ -722,7 +775,7 @@ objects_query_strokecap (GSList *objects, SPStyle *style_res)
         SPStyle *style = SP_OBJECT_STYLE (obj);
         if (!style) continue;
 
-        if ( style->stroke.type == SP_PAINT_TYPE_NONE ) {
+        if ( style->stroke.isNone() ) {
             continue;
         }
 
@@ -772,7 +825,7 @@ objects_query_strokejoin (GSList *objects, SPStyle *style_res)
         SPStyle *style = SP_OBJECT_STYLE (obj);
         if (!style) continue;
 
-        if ( style->stroke.type == SP_PAINT_TYPE_NONE ) {
+        if ( style->stroke.isNone() ) {
             continue;
         }
 
@@ -826,7 +879,7 @@ objects_query_fontnumbers (GSList *objects, SPStyle *style_res)
         SPObject *obj = SP_OBJECT (i->data);
 
         if (!SP_IS_TEXT(obj) && !SP_IS_FLOWTEXT(obj)
-            && !SP_IS_TSPAN(obj) && !SP_IS_TEXTPATH(obj)
+            && !SP_IS_TSPAN(obj) && !SP_IS_TREF(obj) && !SP_IS_TEXTPATH(obj)
             && !SP_IS_FLOWDIV(obj) && !SP_IS_FLOWPARA(obj) && !SP_IS_FLOWTSPAN(obj))
             continue;
 
@@ -834,7 +887,7 @@ objects_query_fontnumbers (GSList *objects, SPStyle *style_res)
         if (!style) continue;
 
         texts ++;
-        size += style->font_size.computed * NR::expansion(sp_item_i2d_affine(SP_ITEM(obj))); /// \todo FIXME: we assume non-% units here
+        size += style->font_size.computed * Geom::Matrix(sp_item_i2d_affine(SP_ITEM(obj))).descrim(); /// \todo FIXME: we assume non-% units here
 
         if (style->letter_spacing.normal) {
             if (!different && (letterspacing_prev == 0 || letterspacing_prev == letterspacing))
@@ -919,7 +972,7 @@ objects_query_fontstyle (GSList *objects, SPStyle *style_res)
         SPObject *obj = SP_OBJECT (i->data);
 
         if (!SP_IS_TEXT(obj) && !SP_IS_FLOWTEXT(obj)
-            && !SP_IS_TSPAN(obj) && !SP_IS_TEXTPATH(obj)
+            && !SP_IS_TSPAN(obj) && !SP_IS_TREF(obj) && !SP_IS_TEXTPATH(obj)
             && !SP_IS_FLOWDIV(obj) && !SP_IS_FLOWPARA(obj) && !SP_IS_FLOWTSPAN(obj))
             continue;
 
@@ -974,7 +1027,7 @@ objects_query_fontfamily (GSList *objects, SPStyle *style_res)
         SPObject *obj = SP_OBJECT (i->data);
 
         if (!SP_IS_TEXT(obj) && !SP_IS_FLOWTEXT(obj)
-            && !SP_IS_TSPAN(obj) && !SP_IS_TEXTPATH(obj)
+            && !SP_IS_TSPAN(obj) && !SP_IS_TREF(obj) && !SP_IS_TEXTPATH(obj)
             && !SP_IS_FLOWDIV(obj) && !SP_IS_FLOWPARA(obj) && !SP_IS_FLOWTSPAN(obj))
             continue;
 
@@ -1011,6 +1064,141 @@ objects_query_fontfamily (GSList *objects, SPStyle *style_res)
     }
 }
 
+int
+objects_query_fontspecification (GSList *objects, SPStyle *style_res)
+{
+    bool different = false;
+    int texts = 0;
+
+    if (style_res->text->font_specification.value) {
+        g_free(style_res->text->font_specification.value);
+        style_res->text->font_specification.value = NULL;
+    }
+    style_res->text->font_specification.set = FALSE;
+
+    for (GSList const *i = objects; i != NULL; i = i->next) {
+        SPObject *obj = SP_OBJECT (i->data);
+
+        if (!SP_IS_TEXT(obj) && !SP_IS_FLOWTEXT(obj)
+            && !SP_IS_TSPAN(obj) && !SP_IS_TREF(obj) && !SP_IS_TEXTPATH(obj)
+            && !SP_IS_FLOWDIV(obj) && !SP_IS_FLOWPARA(obj) && !SP_IS_FLOWTSPAN(obj))
+            continue;
+
+        SPStyle *style = SP_OBJECT_STYLE (obj);
+        if (!style) continue;
+
+        texts ++;
+
+        if (style_res->text->font_specification.value && style_res->text->font_specification.set &&
+            style->text->font_specification.value && style->text->font_specification.set &&
+            strcmp (style_res->text->font_specification.value, style->text->font_specification.value)) {
+            different = true;  // different fonts
+        }
+
+        if (style->text->font_specification.set) {
+
+            if (style_res->text->font_specification.value) {
+                g_free(style_res->text->font_specification.value);
+                style_res->text->font_specification.value = NULL;
+            }
+
+            style_res->text->font_specification.set = TRUE;
+            style_res->text->font_specification.value = g_strdup(style->text->font_specification.value);
+        }
+    }
+
+    if (texts == 0)
+        return QUERY_STYLE_NOTHING;
+
+    if (texts > 1) {
+        if (different) {
+            return QUERY_STYLE_MULTIPLE_DIFFERENT;
+        } else {
+            return QUERY_STYLE_MULTIPLE_SAME;
+        }
+    } else {
+        return QUERY_STYLE_SINGLE;
+    }
+}
+
+int
+objects_query_blend (GSList *objects, SPStyle *style_res)
+{
+    const int empty_prev = -2;
+    const int complex_filter = 5;
+    int blend = 0;
+    float blend_prev = empty_prev;
+    bool same_blend = true;
+    guint items = 0;
+
+    for (GSList const *i = objects; i != NULL; i = i->next) {
+        SPObject *obj = SP_OBJECT (i->data);
+        SPStyle *style = SP_OBJECT_STYLE (obj);
+        if(!style || !SP_IS_ITEM(obj)) continue;
+
+        items++;
+
+        //if object has a filter
+        if (style->filter.set && style->getFilter()) {
+            int blurcount = 0;
+            int blendcount = 0;
+
+            // determine whether filter is simple (blend and/or blur) or complex
+            for(SPObject *primitive_obj = style->getFilter()->children;
+                primitive_obj && SP_IS_FILTER_PRIMITIVE(primitive_obj);
+                primitive_obj = primitive_obj->next) {
+                SPFilterPrimitive *primitive = SP_FILTER_PRIMITIVE(primitive_obj);
+                if(SP_IS_FEBLEND(primitive))
+                    ++blendcount;
+                else if(SP_IS_GAUSSIANBLUR(primitive))
+                    ++blurcount;
+                else {
+                    blurcount = complex_filter;
+                    break;
+                }
+            }
+
+            // simple filter
+            if(blurcount == 1 || blendcount == 1) {
+                for(SPObject *primitive_obj = style->getFilter()->children;
+                    primitive_obj && SP_IS_FILTER_PRIMITIVE(primitive_obj);
+                    primitive_obj = primitive_obj->next) {
+                    if(SP_IS_FEBLEND(primitive_obj)) {
+                        SPFeBlend *spblend = SP_FEBLEND(primitive_obj);
+                        blend = spblend->blend_mode;
+                    }
+                }
+            }
+            else {
+                blend = complex_filter;
+            }
+        }
+        // defaults to blend mode = "normal"
+        else {
+            blend = 0;
+        }
+
+        if(blend_prev != empty_prev && blend_prev != blend)
+            same_blend = false;
+        blend_prev = blend;
+    }
+
+    if (items > 0) {
+        style_res->filter_blend_mode.value = blend;
+    }
+
+    if (items == 0) {
+        return QUERY_STYLE_NOTHING;
+    } else if (items == 1) {
+        return QUERY_STYLE_SINGLE;
+    } else {
+        if(same_blend)
+            return QUERY_STYLE_MULTIPLE_SAME;
+        else
+            return QUERY_STYLE_MULTIPLE_DIFFERENT;
+    }
+}
+
 /**
  * Write to style_res the average blurring of a list of objects.
  */
@@ -1027,37 +1215,40 @@ objects_query_blur (GSList *objects, SPStyle *style_res)
     bool same_blur = true;
     guint blur_items = 0;
     guint items = 0;
-    
+
     for (GSList const *i = objects; i != NULL; i = i->next) {
         SPObject *obj = SP_OBJECT (i->data);
         SPStyle *style = SP_OBJECT_STYLE (obj);
         if (!style) continue;
         if (!SP_IS_ITEM(obj)) continue;
 
-        NR::Matrix i2d = sp_item_i2d_affine (SP_ITEM(obj));
+        Geom::Matrix i2d = sp_item_i2d_affine (SP_ITEM(obj));
 
         items ++;
 
         //if object has a filter
-        if (style->filter.set && style->filter.filter) {
+        if (style->filter.set && style->getFilter()) {
             //cycle through filter primitives
-            for(int i=0; i<style->filter.filter->_primitive_count; i++)
-            {
-                SPFilterPrimitive *primitive = style->filter.filter->_primitives[i];
-                //if primitive is gaussianblur
-                if(SP_IS_GAUSSIANBLUR(primitive)) {
-                    SPGaussianBlur * spblur = SP_GAUSSIANBLUR(primitive);
-                    float num = spblur->stdDeviation.getNumber();
-                    blur_sum += num * NR::expansion(i2d);
-                    if (blur_prev != -1 && fabs (num - blur_prev) > 1e-2) // rather low tolerance because difference in blur radii is much harder to notice than e.g. difference in sizes
-                        same_blur = false;
-                    blur_prev = num;
-                    //TODO: deal with opt number, for the moment it's not necessary to the ui.
-                    blur_items ++;
+            SPObject *primitive_obj = style->getFilter()->children;
+            while (primitive_obj) {
+                if (SP_IS_FILTER_PRIMITIVE(primitive_obj)) {
+                    SPFilterPrimitive *primitive = SP_FILTER_PRIMITIVE(primitive_obj);
+
+                    //if primitive is gaussianblur
+                    if(SP_IS_GAUSSIANBLUR(primitive)) {
+                        SPGaussianBlur * spblur = SP_GAUSSIANBLUR(primitive);
+                        float num = spblur->stdDeviation.getNumber();
+                        blur_sum += num * i2d.descrim();
+                        if (blur_prev != -1 && fabs (num - blur_prev) > 1e-2) // rather low tolerance because difference in blur radii is much harder to notice than e.g. difference in sizes
+                            same_blur = false;
+                        blur_prev = num;
+                        //TODO: deal with opt number, for the moment it's not necessary to the ui.
+                        blur_items ++;
+                    }
                 }
+                primitive_obj = primitive_obj->next;
             }
         }
-
     }
 
     if (items > 0) {
@@ -1102,6 +1293,8 @@ sp_desktop_query_style_from_list (GSList *list, SPStyle *style, int property)
     } else if (property == QUERY_STYLE_PROPERTY_MASTEROPACITY) {
         return objects_query_opacity (list, style);
 
+    } else if (property == QUERY_STYLE_PROPERTY_FONT_SPECIFICATION) {
+        return objects_query_fontspecification (list, style);
     } else if (property == QUERY_STYLE_PROPERTY_FONTFAMILY) {
         return objects_query_fontfamily (list, style);
     } else if (property == QUERY_STYLE_PROPERTY_FONTSTYLE) {
@@ -1109,6 +1302,8 @@ sp_desktop_query_style_from_list (GSList *list, SPStyle *style, int property)
     } else if (property == QUERY_STYLE_PROPERTY_FONTNUMBERS) {
         return objects_query_fontnumbers (list, style);
 
+    } else if (property == QUERY_STYLE_PROPERTY_BLEND) {
+        return objects_query_blend (list, style);
     } else if (property == QUERY_STYLE_PROPERTY_BLUR) {
         return objects_query_blur (list, style);
     }
@@ -1129,12 +1324,16 @@ sp_desktop_query_style(SPDesktop *desktop, SPStyle *style, int property)
         return ret; // subselection returned a style, pass it on
 
     // otherwise, do querying and averaging over selection
-    return sp_desktop_query_style_from_list ((GSList *) desktop->selection->itemList(), style, property);
+    if (desktop->selection != NULL) {
+    	return sp_desktop_query_style_from_list ((GSList *) desktop->selection->itemList(), style, property);
+    }
+
+    return QUERY_STYLE_NOTHING;
 }
 
 /**
- * Do the same as sp_desktop_query_style for all (defined) style properties, return true if none of
- * the properties returned QUERY_STYLE_NOTHING.
+ * Do the same as sp_desktop_query_style for all (defined) style properties, return true if at
+ * least one of the properties did not return QUERY_STYLE_NOTHING.
  */
 bool
 sp_desktop_query_style_all (SPDesktop *desktop, SPStyle *query)
@@ -1150,8 +1349,18 @@ sp_desktop_query_style_all (SPDesktop *desktop, SPStyle *query)
         int result_strokejoin = sp_desktop_query_style (desktop, query, QUERY_STYLE_PROPERTY_STROKEJOIN);
         int result_opacity = sp_desktop_query_style (desktop, query, QUERY_STYLE_PROPERTY_MASTEROPACITY);
         int result_blur = sp_desktop_query_style (desktop, query, QUERY_STYLE_PROPERTY_BLUR);
-        
-        return (result_family != QUERY_STYLE_NOTHING && result_fstyle != QUERY_STYLE_NOTHING && result_fnumbers != QUERY_STYLE_NOTHING && result_fill != QUERY_STYLE_NOTHING && result_stroke != QUERY_STYLE_NOTHING && result_opacity != QUERY_STYLE_NOTHING && result_strokewidth != QUERY_STYLE_NOTHING && result_strokemiterlimit != QUERY_STYLE_NOTHING && result_strokecap != QUERY_STYLE_NOTHING && result_strokejoin != QUERY_STYLE_NOTHING && result_blur != QUERY_STYLE_NOTHING);
+
+        return (result_family != QUERY_STYLE_NOTHING ||
+                result_fstyle != QUERY_STYLE_NOTHING ||
+                result_fnumbers != QUERY_STYLE_NOTHING ||
+                result_fill != QUERY_STYLE_NOTHING ||
+                result_stroke != QUERY_STYLE_NOTHING ||
+                result_opacity != QUERY_STYLE_NOTHING ||
+                result_strokewidth != QUERY_STYLE_NOTHING ||
+                result_strokemiterlimit != QUERY_STYLE_NOTHING ||
+                result_strokecap != QUERY_STYLE_NOTHING ||
+                result_strokejoin != QUERY_STYLE_NOTHING ||
+                result_blur != QUERY_STYLE_NOTHING);
 }
 
 
