@@ -29,12 +29,15 @@
 #include <gtk/gtktypeutils.h>
 #include <sigc++/sigc++.h>
 
-#include "libnr/nr-matrix.h"
-#include "libnr/nr-matrix-fns.h"
-#include "libnr/nr-rect.h"
+#include <2geom/matrix.h>
+#include <2geom/rect.h>
+
 #include "ui/view/view.h"
 #include "ui/view/edit-widget-interface.h"
 
+#include "display/rendermode.h"
+#include "display/snap-indicator.h"
+
 class NRRect;
 class SPCSSAttr;
 struct _GtkWidget;
@@ -72,6 +75,11 @@ namespace Inkscape {
   namespace Whiteboard {
       class SessionManager;
   }
+  namespace Display {
+      class TemporaryItemList;
+      class TemporaryItem;
+      //class SnapIndicator;
+  }
 }
 
 /**
@@ -90,6 +98,9 @@ struct SPDesktop : public Inkscape::UI::View::View
     Inkscape::LayerManager    *layer_manager;
     Inkscape::EventLog        *event_log;
 
+    Inkscape::Display::TemporaryItemList *temporary_item_list;
+    Inkscape::Display::SnapIndicator *snapindicator;
+
     SPCanvasItem  *acetate;
     SPCanvasGroup *main;
     SPCanvasGroup *gridgroup;
@@ -97,13 +108,17 @@ struct SPDesktop : public Inkscape::UI::View::View
     SPCanvasItem  *drawing;
     SPCanvasGroup *sketch;
     SPCanvasGroup *controls;
+    SPCanvasGroup *tempgroup;   ///< contains temporary canvas items
     SPCanvasItem  *table;       ///< outside-of-page background
     SPCanvasItem  *page;        ///< page background
     SPCanvasItem  *page_border; ///< page border
     SPCSSAttr     *current;     ///< current style
+    bool           _focusMode;  ///< Whether we're focused working or general working
 
     GList *zooms_past;
     GList *zooms_future;
+    bool _quick_zoom_enabled; ///< Signifies that currently we're in quick zoom mode
+    Geom::Rect _quick_zoom_stored_area;  ///< The area of the screen before quick zoom
     unsigned int dkey;
     unsigned int number;
     guint window_state;
@@ -165,7 +180,7 @@ struct SPDesktop : public Inkscape::UI::View::View
 #endif
 
     SPDesktop();
-    void init (SPNamedView* nv, SPCanvas* canvas);
+    void init (SPNamedView* nv, SPCanvas* canvas, Inkscape::UI::View::EditWidgetInterface *widget);
     virtual ~SPDesktop();
     void destroy();
 
@@ -173,11 +188,22 @@ struct SPDesktop : public Inkscape::UI::View::View
         return _guides_message_context;
     }
 
-    void setDisplayModeNormal();
-    void setDisplayModeOutline();
+    Inkscape::Display::TemporaryItem * add_temporary_canvasitem (SPCanvasItem *item, guint lifetime, bool move_to_bottom = true);
+    void remove_temporary_canvasitem (Inkscape::Display::TemporaryItem * tempitem);
+
+    void _setDisplayMode(Inkscape::RenderMode mode);
+    void setDisplayModeNormal() {
+        _setDisplayMode(Inkscape::RENDERMODE_NORMAL);
+    }
+    void setDisplayModeNoFilters() {
+        _setDisplayMode(Inkscape::RENDERMODE_NO_FILTERS);
+    }
+    void setDisplayModeOutline() {
+        _setDisplayMode(Inkscape::RENDERMODE_OUTLINE);
+    }
     void displayModeToggle();
-    int displayMode;
-    int getMode() const { return displayMode; }
+    Inkscape::RenderMode _display_mode;
+    Inkscape::RenderMode getMode() const { return _display_mode; }
 
     Inkscape::UI::Widget::Dock* getDock() { return _widget->getDock(); }
 
@@ -185,6 +211,7 @@ struct SPDesktop : public Inkscape::UI::View::View
     SPObject *currentRoot() const;
     SPObject *currentLayer() const;
     void setCurrentLayer(SPObject *object);
+    void toggleLayerSolo(SPObject *object);
     SPObject *layerForObject(SPObject *object);
     bool isLayer(SPObject *object) const;
     bool isWithinViewport(SPItem *item) const;
@@ -196,24 +223,22 @@ struct SPDesktop : public Inkscape::UI::View::View
     void set_event_context (GtkType type, const gchar *config);
     void push_event_context (GtkType type, const gchar *config, unsigned int key);
 
-    void set_coordinate_status (NR::Point p);
-    SPItem *item_from_list_at_point_bottom (const GSList *list, NR::Point const p) const;
-    SPItem *item_at_point (NR::Point const p, bool into_groups, SPItem *upto = NULL) const;
-    SPItem *group_at_point (NR::Point const p) const;
-    NR::Point point() const;
+    void set_coordinate_status (Geom::Point p);
+    SPItem *item_from_list_at_point_bottom (const GSList *list, Geom::Point const p) const;
+    SPItem *item_at_point (Geom::Point const p, bool into_groups, SPItem *upto = NULL) const;
+    SPItem *group_at_point (Geom::Point const p) const;
+    Geom::Point point() const;
 
-    NR::Rect get_display_area() const;
+    Geom::Rect get_display_area() const;
     void set_display_area (double x0, double y0, double x1, double y1, double border, bool log = true);
-    void set_display_area(NR::Rect const &a, NR::Coord border, bool log = true);
+    void set_display_area(Geom::Rect const &a, Geom::Coord border, bool log = true);
     void zoom_absolute (double cx, double cy, double zoom);
     void zoom_relative (double cx, double cy, double zoom);
     void zoom_absolute_keep_point (double cx, double cy, double px, double py, double zoom);
     void zoom_relative_keep_point (double cx, double cy, double zoom);
-    void zoom_relative_keep_point (NR::Point const &c, double const zoom)
+    void zoom_relative_keep_point (Geom::Point const &c, double const zoom)
     {
-            using NR::X;
-            using NR::Y;
-            zoom_relative_keep_point (c[X], c[Y], zoom);
+            zoom_relative_keep_point (c[Geom::X], c[Geom::Y], zoom);
     }
 
     void zoom_page();
@@ -221,21 +246,23 @@ struct SPDesktop : public Inkscape::UI::View::View
     void zoom_drawing();
     void zoom_selection();
     void zoom_grab_focus();
-    double current_zoom() const  { return _d2w.expansion(); }
+    double current_zoom() const  { return _d2w.descrim(); }
     void prev_zoom();
     void next_zoom();
+    void zoom_quick(bool enable = true);
+	/** \brief  Returns whether the desktop is in quick zoom mode or not */
+	bool quick_zoomed(void) { return _quick_zoom_enabled; }
 
-    bool scroll_to_point (NR::Point const *s_dt, gdouble autoscrollspeed = 0);
+    bool scroll_to_point (Geom::Point const &s_dt, gdouble autoscrollspeed = 0);
     void scroll_world (double dx, double dy, bool is_scrolling = false);
-    void scroll_world (NR::Point const scroll, bool is_scrolling = false)
+    void scroll_world (Geom::Point const scroll, bool is_scrolling = false)
     {
-        using NR::X;
-        using NR::Y;
-        scroll_world(scroll[X], scroll[Y], is_scrolling);
+        scroll_world(scroll[Geom::X], scroll[Geom::Y], is_scrolling);
     }
+    void scroll_world_in_svg_coords (double dx, double dy, bool is_scrolling = false);
 
     void getWindowGeometry (gint &x, gint &y, gint &w, gint &h);
-    void setWindowPosition (NR::Point p);
+    void setWindowPosition (Geom::Point p);
     void setWindowSize (gint w, gint h);
     void setWindowTransient (void* p, int transient_policy=1);
     Gtk::Window* getToplevel();
@@ -257,31 +284,32 @@ struct SPDesktop : public Inkscape::UI::View::View
 
     void setWaitingCursor();
     void clearWaitingCursor();
+    bool isWaitingCursor() const { return waiting_cursor; };
 
     void toggleColorProfAdjust();
 
     void toggleGrids();
-    void toggleSnapping();
-    bool gridsEnabled() { return grids_visible; }
+    void toggleSnapGlobal();
+    bool gridsEnabled() const { return grids_visible; };
     void showGrids(bool show, bool dirty_document = true);
 
     bool is_iconified();
     bool is_maximized();
     bool is_fullscreen();
+    bool is_focusMode();
 
     void iconify();
     void maximize();
     void fullscreen();
+    void focusMode(bool mode = true);
 
-    void registerEditWidget (Inkscape::UI::View::EditWidgetInterface *widget)
-    { _widget = widget; }
-
-    NR::Matrix w2d() const; //transformation from window to desktop coordinates (used for zooming)
-    NR::Point w2d(NR::Point const &p) const;
-    NR::Point d2w(NR::Point const &p) const;
-    NR::Matrix doc2dt() const;
-    NR::Point doc2dt(NR::Point const &p) const;
-    NR::Point dt2doc(NR::Point const &p) const;
+    Geom::Matrix w2d() const; //transformation from window to desktop coordinates (used for zooming)
+    Geom::Point w2d(Geom::Point const &p) const;
+    Geom::Point d2w(Geom::Point const &p) const;
+    Geom::Matrix doc2dt() const;
+    Geom::Matrix dt2doc() const;
+    Geom::Point doc2dt(Geom::Point const &p) const;
+    Geom::Point dt2doc(Geom::Point const &p) const;
 
     virtual void setDocument (SPDocument* doc);
     virtual bool shutdown();
@@ -296,9 +324,9 @@ private:
     Inkscape::Application     *_inkscape;
     Inkscape::MessageContext  *_guides_message_context;
     bool _active;
-    NR::Matrix _w2d;
-    NR::Matrix _d2w;
-    NR::Matrix _doc2dt;
+    Geom::Matrix _w2d;
+    Geom::Matrix _d2w;
+    Geom::Matrix _doc2dt;
 
     bool grids_visible; /* don't set this variable directly, use the method below */
     void set_grids_visible(bool visible);
