@@ -1,9 +1,7 @@
-#define __SP_ITEM_PROPERTIES_C__
-
-/*
- * Object properties dialog
- *
- * Authors:
+/** @file
+ * @brief Object properties dialog
+ */
+/* Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   bulia byak <buliabyak@users.sf.net>
  *   Johan Engelen <goejendaagh@zonnet.nl>
@@ -21,26 +19,27 @@
 #include <gtk/gtkhbox.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkspinbutton.h>
+#include <gtk/gtk.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkframe.h>
+#include <gtk/gtkexpander.h>
 #include <gtk/gtktextview.h>
 #include <gtk/gtktooltips.h>
 
-#include <glibmm/i18n.h>
-#include "helper/window.h"
-#include "../widgets/sp-widget.h"
-#include "../inkscape.h"
-#include "../document.h"
 #include "../desktop-handles.h"
+#include "dialog-events.h"
+#include "../document.h"
+#include <glibmm/i18n.h>
+#include "../helper/window.h"
+#include "../inkscape.h"
+#include "../interface.h"
+#include "../macros.h"
+#include "../preferences.h"
 #include "../selection.h"
 #include "../sp-item.h"
-#include "../macros.h"
 #include "../verbs.h"
-#include "../interface.h"
-
-#include "dialog-events.h"
-#include "../prefs-utils.h"
+#include "../widgets/sp-attribute-widget.h"
+#include "../widgets/sp-widget.h"
 
 #define MIN_ONSCREEN_DISTANCE 50
 
@@ -49,7 +48,7 @@ static win_data wd;
 
 // impossible original values to make sure they are read from prefs
 static gint x = -1000, y = -1000, w = 0, h = 0;
-static gchar *prefs_path = "dialogs.object";
+static Glib::ustring const prefs_path = "/dialogs/object/";
 
 static void sp_item_widget_modify_selection (SPWidget *spw, Inkscape::Selection *selection, guint flags, GtkWidget *itemw);
 static void sp_item_widget_change_selection (SPWidget *spw, Inkscape::Selection *selection, GtkWidget *itemw);
@@ -75,10 +74,11 @@ sp_item_dialog_delete( GtkObject */*object*/, GdkEvent */*event*/, gpointer /*da
     if (x<0) x=0;
     if (y<0) y=0;
 
-    prefs_set_int_attribute (prefs_path, "x", x);
-    prefs_set_int_attribute (prefs_path, "y", y);
-    prefs_set_int_attribute (prefs_path, "w", w);
-    prefs_set_int_attribute (prefs_path, "h", h);
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+    prefs->setInt(prefs_path + "x", x);
+    prefs->setInt(prefs_path + "y", y);
+    prefs->setInt(prefs_path + "w", w);
+    prefs->setInt(prefs_path + "h", h);
 
     return FALSE; // which means, go ahead and destroy it
 
@@ -92,7 +92,7 @@ GtkWidget *
 sp_item_widget_new (void)
 {
 
-    GtkWidget *spw, *vb, *t, *cb, *l, *f, *tf, *pb;
+    GtkWidget *spw, *vb, *t, *cb, *l, *f, *tf, *pb, *int_expander, *int_label;
     GtkTextBuffer *desc_buffer;
 
     GtkTooltips *tt = gtk_tooltips_new();
@@ -170,7 +170,7 @@ sp_item_widget_new (void)
     g_signal_connect ( G_OBJECT (tf), "activate", G_CALLBACK (sp_item_widget_label_changed), spw);
 
     /* Create the label for the object title */
-    l = gtk_label_new (_("Title"));
+    l = gtk_label_new_with_mnemonic (_("_Title"));
     gtk_misc_set_alignment (GTK_MISC (l), 1, 0.5);
     gtk_table_attach ( GTK_TABLE (t), l, 0, 1, 2, 3,
                        (GtkAttachOptions)( GTK_SHRINK | GTK_FILL ),
@@ -185,13 +185,15 @@ sp_item_widget_new (void)
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ),
                        (GtkAttachOptions)0, 0, 0 );
     gtk_object_set_data (GTK_OBJECT (spw), "title", tf);
+    gtk_label_set_mnemonic_widget (GTK_LABEL(l), tf);
 
     /* Create the frame for the object description */
-    f = gtk_frame_new (_("Description"));
+    l = gtk_label_new_with_mnemonic (_("_Description"));
+    f = gtk_frame_new (NULL);
+    gtk_frame_set_label_widget (GTK_FRAME (f),l);
     gtk_table_attach ( GTK_TABLE (t), f, 0, 3, 3, 4,
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ),
                        (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ), 0, 0 );
-    gtk_object_set_data (GTK_OBJECT (spw), "desc_frame", l);
 
     /* Create the text view box for the object description */
     GtkWidget *textframe = gtk_frame_new(NULL);
@@ -199,12 +201,15 @@ sp_item_widget_new (void)
     gtk_widget_set_sensitive (GTK_WIDGET (textframe), FALSE);
     gtk_container_add (GTK_CONTAINER (f), textframe);
     gtk_frame_set_shadow_type (GTK_FRAME (textframe), GTK_SHADOW_IN);
+    gtk_object_set_data(GTK_OBJECT(spw), "desc_frame", textframe);
 
     tf = gtk_text_view_new();
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(tf), GTK_WRAP_WORD);
     desc_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(tf));
     gtk_text_buffer_set_text(desc_buffer, "", -1);
     gtk_container_add (GTK_CONTAINER (textframe), tf);
     gtk_object_set_data (GTK_OBJECT (spw), "desc", tf);
+    gtk_label_set_mnemonic_widget (GTK_LABEL (gtk_frame_get_label_widget (GTK_FRAME (f))), tf);
 
     /* Check boxes */
     GtkWidget *hb_cb = gtk_hbox_new (FALSE, 0);
@@ -234,6 +239,14 @@ sp_item_widget_new (void)
                          spw );
     gtk_object_set_data (GTK_OBJECT (spw), "sensitive", cb);
 
+    /* Create the frame for interactivity options */
+    int_label = gtk_label_new_with_mnemonic (_("_Interactivity"));
+    int_expander = gtk_expander_new (NULL);
+    gtk_expander_set_label_widget (GTK_EXPANDER (int_expander),int_label);
+    gtk_object_set_data (GTK_OBJECT (spw), "interactivity", int_expander);
+
+    gtk_box_pack_start (GTK_BOX (vb), int_expander, FALSE, FALSE, 0);
+
     gtk_widget_show_all (spw);
 
     sp_item_widget_setup (SP_WIDGET (spw), sp_desktop_selection (SP_ACTIVE_DESKTOP));
@@ -324,7 +337,45 @@ sp_item_widget_setup ( SPWidget *spw, Inkscape::Selection *selection )
         w = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (spw), "label"));
         gtk_entry_set_text (GTK_ENTRY (w), obj->defaultLabel());
         gtk_widget_set_sensitive (w, TRUE);
-        w = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (spw), "label_label"));
+
+        /* Title */
+        w = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(spw), "title"));
+        gchar *title = obj->title();
+        if (title) {
+            gtk_entry_set_text(GTK_ENTRY(w), title);
+            g_free(title);
+        }
+        else gtk_entry_set_text(GTK_ENTRY(w), "");
+        gtk_widget_set_sensitive(w, TRUE);
+
+        /* Description */
+        w = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(spw), "desc"));
+        GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(w));
+        gchar *desc = obj->desc();
+        if (desc) {
+            gtk_text_buffer_set_text(buf, desc, -1);
+            g_free(desc);
+        } else {
+            gtk_text_buffer_set_text(buf, "", 0);
+        }
+        w = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(spw), "desc_frame"));
+        gtk_widget_set_sensitive(w, TRUE);
+
+        w = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(spw), "interactivity"));
+
+        GtkWidget* int_table = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(spw), "interactivity_table"));
+        if (int_table){
+            gtk_container_remove(GTK_CONTAINER(w), int_table);
+        }
+
+        const gchar* int_labels[10] = {"onclick", "onmouseover", "onmouseout", "onmousedown", "onmouseup", "onmousemove","onfocusin", "onfocusout", "onactivate", "onload"};
+
+        int_table = sp_attribute_table_new (obj, 10, int_labels, int_labels);
+        gtk_widget_show_all (int_table);
+        gtk_object_set_data(GTK_OBJECT(spw), "interactivity_table", int_table);
+
+        gtk_container_add (GTK_CONTAINER (w), int_table);
+
     }
 
     gtk_object_set_data (GTK_OBJECT (spw), "blocked", GUINT_TO_POINTER (FALSE));
@@ -419,21 +470,22 @@ sp_item_widget_label_changed( GtkWidget */*widget*/, SPWidget *spw )
     }
 
     /* Retrieve the title */
-    GtkWidget *w = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (spw), "title"));
-    gchar *title = (gchar *)gtk_entry_get_text (GTK_ENTRY (w));
-    if (title != NULL) {
-        obj->setTitle(title);
-        sp_document_done (SP_ACTIVE_DOCUMENT, SP_VERB_DIALOG_ITEM,
-                                _("Set object title"));
-    }
+    GtkWidget *w = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(spw), "title"));
+    gchar *title = (gchar *)gtk_entry_get_text(GTK_ENTRY (w));
+    if (obj->setTitle(title))
+        sp_document_done(SP_ACTIVE_DOCUMENT, SP_VERB_DIALOG_ITEM,
+                         _("Set object title"));
 
     /* Retrieve the description */
-    gchar *desc = NULL; /* TODO:  get text from text buffer */
-    if (desc != NULL) {
-        obj->setDesc(desc);
-        sp_document_done (SP_ACTIVE_DOCUMENT, SP_VERB_DIALOG_ITEM,
-                                _("Set object description"));
-    }
+    GtkTextView *tv = GTK_TEXT_VIEW(gtk_object_get_data(GTK_OBJECT(spw), "desc"));
+    GtkTextBuffer *buf = gtk_text_view_get_buffer(tv);
+    GtkTextIter start, end;
+    gtk_text_buffer_get_bounds(buf, &start, &end);
+    gchar *desc = gtk_text_buffer_get_text(buf, &start, &end, TRUE);
+    if (obj->setDesc(desc))
+        sp_document_done(SP_ACTIVE_DOCUMENT, SP_VERB_DIALOG_ITEM,
+                         _("Set object description"));
+    g_free(desc);
 
     gtk_object_set_data (GTK_OBJECT (spw), "blocked", GUINT_TO_POINTER (FALSE));
 
@@ -451,16 +503,16 @@ sp_item_dialog (void)
 
         gchar title[500];
         sp_ui_dialog_title_string (Inkscape::Verb::get(SP_VERB_DIALOG_ITEM), title);
+        Inkscape::Preferences *prefs = Inkscape::Preferences::get();
 
         dlg = sp_window_new (title, TRUE);
         if (x == -1000 || y == -1000) {
-            x = prefs_get_int_attribute (prefs_path, "x", -1000);
-            y = prefs_get_int_attribute (prefs_path, "y", -1000);
+            x = prefs->getInt(prefs_path + "x", -1000);
+            y = prefs->getInt(prefs_path + "y", -1000);
         }
-
         if (w ==0 || h == 0) {
-            w = prefs_get_int_attribute (prefs_path, "w", 0);
-            h = prefs_get_int_attribute (prefs_path, "h", 0);
+            w = prefs->getInt(prefs_path + "w", 0);
+            h = prefs->getInt(prefs_path + "h", 0);
         }
 
 //        if (x<0) x=0;
