@@ -1,20 +1,17 @@
-#ifndef __SP_ITEM_PROPERTIES_H__
-#define __SP_ITEM_PROPERTIES_H__
-
-/**
+/** @file
  * \brief  Display settings dialog
- *
- * Author:
+ */
+/* Author:
  *   Lauris Kaplinski <lauris@ximian.com>
  *
  * Copyright (C) 2001 Ximian, Inc.
- *
+ * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifndef SEEN_DIALOGS_ITEM_PROPERTIES_H
+#define SEEN_DIALOGS_ITEM_PROPERTIES_H
+
 #include <glib.h>
-
-
-
 #include <gtk/gtkwidget.h>
 #include "../forward.h"
 
@@ -22,8 +19,6 @@ GtkWidget *sp_item_widget_new (void);
 
 void sp_item_dialog (void);
 
-
-
 #endif
 
 /*
