@@ -17,10 +17,8 @@
 # include <config.h>
 #endif
 #include <libnr/nr-blit.h>
-#include <libnr/nr-path.h>
-#include <libnr/n-art-bpath.h>
-#include <libnr/nr-matrix-ops.h>
-#include <libnr/nr-matrix-fns.h>
+#include <libnr/nr-convert2geom.h>
+#include <2geom/matrix.h>
 #include "../style.h"
 #include "nr-arena.h"
 #include "nr-arena-glyphs.h"
@@ -37,13 +35,17 @@
 void nr_pixblock_render_shape_mask_or(NRPixBlock &m, Shape *theS);
 #endif
 
+#ifdef ENABLE_SVG_FONTS
+#include "nr-svgfonts.h"
+#endif //#ifdef ENABLE_SVG_FONTS
+
 static void nr_arena_glyphs_class_init(NRArenaGlyphsClass *klass);
 static void nr_arena_glyphs_init(NRArenaGlyphs *glyphs);
 static void nr_arena_glyphs_finalize(NRObject *object);
 
 static guint nr_arena_glyphs_update(NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
 static guint nr_arena_glyphs_clip(NRArenaItem *item, NRRectL *area, NRPixBlock *pb);
-static NRArenaItem *nr_arena_glyphs_pick(NRArenaItem *item, NR::Point p, double delta, unsigned int sticky);
+static NRArenaItem *nr_arena_glyphs_pick(NRArenaItem *item, Geom::Point p, double delta, unsigned int sticky);
 
 static NRArenaItemClass *glyphs_parent_class;
 
@@ -85,7 +87,7 @@ static void
 nr_arena_glyphs_init(NRArenaGlyphs *glyphs)
 {
     glyphs->style = NULL;
-    nr_matrix_set_identity(&glyphs->g_transform);
+    glyphs->g_transform.setIdentity();
     glyphs->font = NULL;
     glyphs->glyph = 0;
 
@@ -138,15 +140,15 @@ nr_arena_glyphs_update(NRArenaItem *item, NRRectL */*area*/, NRGC *gc, guint /*s
     bbox.x0 = bbox.y0 = NR_HUGE;
     bbox.x1 = bbox.y1 = -NR_HUGE;
 
-    float const scale = NR_MATRIX_DF_EXPANSION(&gc->transform);
+    float const scale = gc->transform.descrim();
 
     if (!glyphs->style->fill.isNone()) {
-        NRMatrix t;
-        nr_matrix_multiply(&t, &glyphs->g_transform, &gc->transform);
-        glyphs->x = t.c[4];
-        glyphs->y = t.c[5];
-        t.c[4]=0;
-        t.c[5]=0;
+        Geom::Matrix t;
+        t = glyphs->g_transform * gc->transform;
+        glyphs->x = t[4];
+        glyphs->y = t[5];
+        t[4]=0;
+        t[5]=0;
         rfont = glyphs->font->RasterFont(t, 0);
         if (glyphs->rfont) glyphs->rfont->Unref();
         glyphs->rfont = rfont;
@@ -163,12 +165,12 @@ nr_arena_glyphs_update(NRArenaItem *item, NRRectL */*area*/, NRGC *gc, guint /*s
 
     if (!glyphs->style->stroke.isNone()) {
         /* Build state data */
-        NRMatrix t;
-        nr_matrix_multiply(&t, &glyphs->g_transform, &gc->transform);
-        glyphs->x = t.c[4];
-        glyphs->y = t.c[5];
-        t.c[4]=0;
-        t.c[5]=0;
+        Geom::Matrix t;
+        t = glyphs->g_transform * gc->transform;
+        glyphs->x = t[4];
+        glyphs->y = t[5];
+        t[4]=0;
+        t[5]=0;
 
         if ( fabs(glyphs->style->stroke_width.computed * scale) > 0.01 ) { // sinon c'est 0=oon veut pas de bord
             font_style nstyl;
@@ -182,9 +184,10 @@ nr_arena_glyphs_update(NRArenaItem *item, NRRectL */*area*/, NRGC *gc, guint /*s
             if ( glyphs->style->stroke_linejoin.computed == SP_STROKE_LINEJOIN_BEVEL ) nstyl.stroke_join=join_straight;
             nstyl.stroke_miter_limit = glyphs->style->stroke_miterlimit.value;
             nstyl.nbDash=0;
-            nstyl.dash_offset=0;
+            nstyl.dash_offset = 0;
             nstyl.dashes=NULL;
             if ( glyphs->style->stroke_dash.n_dash > 0 ) {
+                nstyl.dash_offset = glyphs->style->stroke_dash.offset * scale;
                 nstyl.nbDash=glyphs->style->stroke_dash.n_dash;
                 nstyl.dashes=(double*)malloc(nstyl.nbDash*sizeof(double));
                 for (int i = 0; i < nstyl.nbDash; i++) nstyl.dashes[i]= glyphs->style->stroke_dash.dash[i] * scale;
@@ -206,13 +209,12 @@ nr_arena_glyphs_update(NRArenaItem *item, NRRectL */*area*/, NRGC *gc, guint /*s
             bbox.y1 = narea.y1 + glyphs->y;
         }
     }
-    if (nr_rect_d_test_empty(&bbox)) return NR_ARENA_ITEM_STATE_ALL;
+    if (nr_rect_d_test_empty(bbox)) return NR_ARENA_ITEM_STATE_ALL;
 
     item->bbox.x0 = (gint32)(bbox.x0 - 1.0);
     item->bbox.y0 = (gint32)(bbox.y0 - 1.0);
     item->bbox.x1 = (gint32)(bbox.x1 + 1.0);
     item->bbox.y1 = (gint32)(bbox.y1 + 1.0);
-    nr_arena_request_render_rect(item->arena, &item->bbox);
 
     return NR_ARENA_ITEM_STATE_ALL;
 }
@@ -232,7 +234,7 @@ nr_arena_glyphs_clip(NRArenaItem *item, NRRectL */*area*/, NRPixBlock */*pb*/)
 }
 
 static NRArenaItem *
-nr_arena_glyphs_pick(NRArenaItem *item, NR::Point p, gdouble /*delta*/, unsigned int /*sticky*/)
+nr_arena_glyphs_pick(NRArenaItem *item, Geom::Point p, gdouble /*delta*/, unsigned int /*sticky*/)
 {
     NRArenaGlyphs *glyphs;
 
@@ -241,8 +243,8 @@ nr_arena_glyphs_pick(NRArenaItem *item, NR::Point p, gdouble /*delta*/, unsigned
     if (!glyphs->font ) return NULL;
     if (!glyphs->style) return NULL;
 
-    double const x = p[NR::X];
-    double const y = p[NR::Y];
+    double const x = p[Geom::X];
+    double const y = p[Geom::Y];
     /* With text we take a simple approach: pick if the point is in a characher bbox */
     if ((x >= item->bbox.x0) && (y >= item->bbox.y0) && (x <= item->bbox.x1) && (y <= item->bbox.y1)) return item;
 
@@ -250,7 +252,7 @@ nr_arena_glyphs_pick(NRArenaItem *item, NR::Point p, gdouble /*delta*/, unsigned
 }
 
 void
-nr_arena_glyphs_set_path(NRArenaGlyphs *glyphs, SPCurve */*curve*/, unsigned int /*lieutenant*/, font_instance *font, gint glyph, NRMatrix const *transform)
+nr_arena_glyphs_set_path(NRArenaGlyphs *glyphs, SPCurve */*curve*/, unsigned int /*lieutenant*/, font_instance *font, gint glyph, Geom::Matrix const *transform)
 {
     nr_return_if_fail(glyphs != NULL);
     nr_return_if_fail(NR_IS_ARENA_GLYPHS(glyphs));
@@ -260,7 +262,7 @@ nr_arena_glyphs_set_path(NRArenaGlyphs *glyphs, SPCurve */*curve*/, unsigned int
     if (transform) {
         glyphs->g_transform = *transform;
     } else {
-        nr_matrix_set_identity(&glyphs->g_transform);
+        glyphs->g_transform.setIdentity();
     }
 
     if (font) font->Ref();
@@ -291,9 +293,9 @@ nr_arena_glyphs_fill_mask(NRArenaGlyphs *glyphs, NRRectL *area, NRPixBlock *m)
 
     NRArenaItem *item = NR_ARENA_ITEM(glyphs);
 
-    if (glyphs->rfont && nr_rect_l_test_intersect(area, &item->bbox)) {
+    if (glyphs->rfont && nr_rect_l_test_intersect_ptr(area, &item->bbox)) {
         raster_glyph *g = glyphs->rfont->GetGlyph(glyphs->glyph);
-        if ( g ) g->Blit(NR::Point(glyphs->x, glyphs->y), *m);
+        if ( g ) g->Blit(Geom::Point(glyphs->x, glyphs->y), *m);
     }
 
     return item->state;
@@ -303,9 +305,9 @@ static guint
 nr_arena_glyphs_stroke_mask(NRArenaGlyphs *glyphs, NRRectL *area, NRPixBlock *m)
 {
     NRArenaItem *item = NR_ARENA_ITEM(glyphs);
-    if (glyphs->sfont && nr_rect_l_test_intersect(area, &item->bbox)) {
+    if (glyphs->sfont && nr_rect_l_test_intersect_ptr(area, &item->bbox)) {
         raster_glyph *g=glyphs->sfont->GetGlyph(glyphs->glyph);
-        if ( g ) g->Blit(NR::Point(glyphs->x, glyphs->y),*m);
+        if ( g ) g->Blit(Geom::Point(glyphs->x, glyphs->y),*m);
     }
 
     return item->state;
@@ -318,7 +320,7 @@ static void nr_arena_glyphs_group_finalize(NRObject *object);
 static guint nr_arena_glyphs_group_update(NRArenaItem *item, NRRectL *area, NRGC *gc, guint state, guint reset);
 static unsigned int nr_arena_glyphs_group_render(cairo_t *ct, NRArenaItem *item, NRRectL *area, NRPixBlock *pb, unsigned int flags);
 static unsigned int nr_arena_glyphs_group_clip(NRArenaItem *item, NRRectL *area, NRPixBlock *pb);
-static NRArenaItem *nr_arena_glyphs_group_pick(NRArenaItem *item, NR::Point p, gdouble delta, unsigned int sticky);
+static NRArenaItem *nr_arena_glyphs_group_pick(NRArenaItem *item, Geom::Point p, gdouble delta, unsigned int sticky);
 
 static NRArenaGroupClass *group_parent_class;
 
@@ -409,14 +411,14 @@ nr_arena_glyphs_group_update(NRArenaItem *item, NRRectL *area, NRGC *gc, guint s
     item->render_opacity = TRUE;
     if (group->style->fill.isPaintserver()) {
         group->fill_painter = sp_paint_server_painter_new(SP_STYLE_FILL_SERVER(group->style),
-                                                          NR::Matrix(&gc->transform), NR::Matrix(&gc->parent->transform),
+                                                          gc->transform, gc->parent->transform,
                                                           &group->paintbox);
         item->render_opacity = FALSE;
     }
 
     if (group->style->stroke.isPaintserver()) {
         group->stroke_painter = sp_paint_server_painter_new(SP_STYLE_STROKE_SERVER(group->style),
-                                                            NR::Matrix(&gc->transform), NR::Matrix(&gc->parent->transform),
+                                                            gc->transform, gc->parent->transform,
                                                             &group->paintbox);
         item->render_opacity = FALSE;
     }
@@ -443,7 +445,7 @@ nr_arena_glyphs_group_render(cairo_t *ct, NRArenaItem *item, NRRectL *area, NRPi
 
     guint ret = item->state;
 
-    if (item->arena->rendermode == RENDERMODE_OUTLINE) {
+    if (item->arena->rendermode == Inkscape::RENDERMODE_OUTLINE) {
 
         if (!ct)
             return item->state;
@@ -457,11 +459,11 @@ nr_arena_glyphs_group_render(cairo_t *ct, NRArenaItem *item, NRRectL *area, NRPi
         for (child = group->children; child != NULL; child = child->next) {
             NRArenaGlyphs *g = NR_ARENA_GLYPHS(child);
 
-            NArtBpath *bpath = (NArtBpath *) g->font->ArtBPath(g->glyph);
+            Geom::PathVector const * pathv = g->font->PathVector(g->glyph);
 
             cairo_new_path(ct);
-            NR::Matrix g_t(g->g_transform);
-            feed_curve_to_cairo (ct, bpath, g_t * group->ctm, (pb->area).upgrade(), false, 0);
+            Geom::Matrix transform = g->g_transform * group->ctm;
+            feed_pathvector_to_cairo (ct, *pathv, transform, to_2geom((pb->area).upgrade()), false, 0);
             cairo_fill(ct);
             pb->empty = FALSE;
         }
@@ -472,7 +474,7 @@ nr_arena_glyphs_group_render(cairo_t *ct, NRArenaItem *item, NRRectL *area, NRPi
 
 
     /* Fill */
-    if (!style->fill.isNone() || item->arena->rendermode == RENDERMODE_OUTLINE) {
+    if (!style->fill.isNone() || item->arena->rendermode == Inkscape::RENDERMODE_OUTLINE) {
         NRPixBlock m;
         nr_pixblock_setup_fast(&m, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
 
@@ -498,9 +500,9 @@ nr_arena_glyphs_group_render(cairo_t *ct, NRArenaItem *item, NRRectL *area, NRPi
             if (ggroup->fill_painter) {
                 nr_arena_render_paintserver_fill(pb, area, ggroup->fill_painter, SP_SCALE24_TO_FLOAT(style->fill_opacity.value), &m);
             }
-        } else if (style->fill.isColor() || item->arena->rendermode == RENDERMODE_OUTLINE) {
+        } else if (style->fill.isColor() || item->arena->rendermode == Inkscape::RENDERMODE_OUTLINE) {
             guint32 rgba;
-            if (item->arena->rendermode == RENDERMODE_OUTLINE) {
+            if (item->arena->rendermode == Inkscape::RENDERMODE_OUTLINE) {
                 // In outline mode, render fill only, using outlinecolor
                 rgba = item->arena->outlinecolor;
             } else if ( item->render_opacity ) {
@@ -517,7 +519,7 @@ nr_arena_glyphs_group_render(cairo_t *ct, NRArenaItem *item, NRRectL *area, NRPi
     }
 
     /* Stroke */
-    if (!style->stroke.isNone() && !(item->arena->rendermode == RENDERMODE_OUTLINE)) {
+    if (!style->stroke.isNone() && !(item->arena->rendermode == Inkscape::RENDERMODE_OUTLINE)) {
         NRPixBlock m;
         guint32 rgba;
         nr_pixblock_setup_fast(&m, NR_PIXBLOCK_MODE_A8, area->x0, area->y0, area->x1, area->y1, TRUE);
@@ -577,7 +579,7 @@ nr_arena_glyphs_group_clip(NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 }
 
 static NRArenaItem *
-nr_arena_glyphs_group_pick(NRArenaItem *item, NR::Point p, gdouble delta, unsigned int sticky)
+nr_arena_glyphs_group_pick(NRArenaItem *item, Geom::Point p, gdouble delta, unsigned int sticky)
 {
     NRArenaItem *picked = NULL;
 
@@ -602,24 +604,22 @@ nr_arena_glyphs_group_clear(NRArenaGlyphsGroup *sg)
 }
 
 void
-nr_arena_glyphs_group_add_component(NRArenaGlyphsGroup *sg, font_instance *font, int glyph, NRMatrix const *transform)
+nr_arena_glyphs_group_add_component(NRArenaGlyphsGroup *sg, font_instance *font, int glyph, Geom::Matrix const &transform)
 {
     NRArenaGroup *group;
-    NRBPath bpath;
 
     group = NR_ARENA_GROUP(sg);
 
-    bpath.path = ( font
-                   ? (NArtBpath *) font->ArtBPath(glyph)
-                   : NULL );
-    if ( bpath.path ) {
-
+    Geom::PathVector const * pathv = ( font
+                                       ? font->PathVector(glyph)
+                                       : NULL );
+    if ( pathv ) {
         nr_arena_item_request_render(NR_ARENA_ITEM(group));
 
         NRArenaItem *new_arena = NRArenaGlyphs::create(group->arena);
         nr_arena_item_append_child(NR_ARENA_ITEM(group), new_arena);
         nr_arena_item_unref(new_arena);
-        nr_arena_glyphs_set_path(NR_ARENA_GLYPHS(new_arena), NULL, FALSE, font, glyph, transform);
+        nr_arena_glyphs_set_path(NR_ARENA_GLYPHS(new_arena), NULL, FALSE, font, glyph, &transform);
         nr_arena_glyphs_set_style(NR_ARENA_GLYPHS(new_arena), sg->style);
     }
 }
