@@ -13,7 +13,7 @@
  * Authors:
  *   Bob Jamison
  *
- * Copyright (C) 2005 Bob Jamison
+ * Copyright (C) 2005-2007 Bob Jamison
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -33,7 +33,6 @@
 
 #include <stdio.h>
 #include <stdarg.h>
-#include <glib.h>
 
 #include <string>
 #include <vector>
@@ -64,7 +63,7 @@ typedef dom::NodeList  NodeList;
 typedef struct
 {
    int   ival;
-   char *sval;
+   char const *sval;
 } LookupEntry;
 
 
@@ -261,8 +260,8 @@ public:
         {
         if (type == OPERATOR)
             {
-            char *tokenStr = "unknown";
-            for (LookupEntry *entry = operatorTable; entry->sval ; entry++)
+            char const *tokenStr = "unknown";
+            for (LookupEntry const *entry = operatorTable; entry->sval ; entry++)
                 {
                 if (entry->ival == ival)
                     {
@@ -274,7 +273,7 @@ public:
             }
         else if (type == NODE_TYPE)
             {
-            char *tokenStr = "unknown";
+            char const *tokenStr = "unknown";
             for (LookupEntry *entry = nodeTypeTable; entry->sval ; entry++)
                 {
                 if (entry->ival == ival)
@@ -287,7 +286,7 @@ public:
             }
         else if (type == AXIS_NAME)
             {
-            char *tokenStr = "unknown";
+            char const *tokenStr = "unknown";
             for (LookupEntry *entry = axisNameTable; entry->sval ; entry++)
                 {
                 if (entry->ival == ival)
@@ -310,7 +309,7 @@ public:
             printf("#%f\n", dval);
         else
             {
-            char *tokenStr = "unknown";
+            char const *tokenStr = "unknown";
             for (LookupEntry *entry = exprTokenTable; entry->sval ; entry++)
                 {
                 if (entry->ival == type)
@@ -372,13 +371,13 @@ public:
      */
     XPathParser()
         {
-        debug = false;
+            debug = false;
         }
 
     /**
      *
      */
-    ~XPathParser() {}
+    virtual ~XPathParser() {}
 
     /**
      *
@@ -416,7 +415,11 @@ private:
     /**
      *
      */
-    void trace(const char *fmt, ...) G_GNUC_PRINTF(2,3);
+    void trace(const char *fmt, ...)
+    #ifdef G_GNUC_PRINTF
+    G_GNUC_PRINTF(2, 3)
+    #endif
+    ;
 
     /**
      *
@@ -426,7 +429,11 @@ private:
     /**
      *
      */
-    void error(const char *fmt, ...) G_GNUC_PRINTF(2,3);
+    void error(const char *fmt, ...)
+    #ifdef G_GNUC_PRINTF
+    G_GNUC_PRINTF(2, 3)
+    #endif
+    ;
 
     //#################################
     //# LEXICAL  SCANNING
