@@ -22,7 +22,6 @@
 #include <gtkmm/table.h>
 #include <glibmm/ustring.h>
 #include "xml/repr.h"
-#include "document.h"
 #include "extension/extension-forward.h"
 
 /** The key that is used to identify that the I/O should be autodetected */
@@ -44,6 +43,7 @@
 /** Defines the key for Postscript printing */
 #define SP_MODULE_KEY_PRINT_PS    "org.inkscape.print.ps"
 #define SP_MODULE_KEY_PRINT_CAIRO_PS    "org.inkscape.print.ps.cairo"
+#define SP_MODULE_KEY_PRINT_CAIRO_EPS    "org.inkscape.print.eps.cairo"
 /** Defines the key for PDF printing */
 #define SP_MODULE_KEY_PRINT_PDF    "org.inkscape.print.pdf"
 #define SP_MODULE_KEY_PRINT_CAIRO_PDF    "org.inkscape.print.pdf.cairo"
@@ -67,6 +67,13 @@
 /** Name of the extension error file */
 #define EXTENSION_ERROR_LOG_FILENAME  "extension-errors.log"
 
+
+#define INKSCAPE_EXTENSION_URI   "http://www.inkscape.org/namespace/inkscape/extension"
+#define INKSCAPE_EXTENSION_NS_NC "extension"
+#define INKSCAPE_EXTENSION_NS    "extension:"
+
+struct SPDocument;
+
 namespace Inkscape {
 namespace Extension {
 
@@ -138,7 +145,7 @@ public:
               the appropriate function in code so that it'll work in the
               future.
     */
-    unsigned int param_visible_count ( ) { return param_count(); };
+    unsigned int param_visible_count ( );
 
 public:
     /** An error class for when a parameter is called on a type it is not */
