@@ -30,7 +30,7 @@ void
 Equalize::init(void)
 {
 	Inkscape::Extension::build_from_mem(
-		"<inkscape-extension>\n"
+		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
 			"<name>" N_("Equalize") "</name>\n"
 			"<id>org.inkscape.effect.bitmap.equalize</id>\n"
 			"<effect>\n"
