@@ -32,7 +32,7 @@ void
 Solarize::init(void)
 {
 	Inkscape::Extension::build_from_mem(
-		"<inkscape-extension>\n"
+		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
 			"<name>" N_("Solarize") "</name>\n"
 			"<id>org.inkscape.effect.bitmap.solarize</id>\n"
 			"<param name=\"factor\" gui-text=\"" N_("Factor") "\" type=\"float\" min=\"0\" max=\"100\">50</param>\n"
