@@ -21,7 +21,6 @@
 #include <set>
 #include <string>
 
-#include "libnr/nr-path.h"
 //#include "libnrtype/font-instance.h"
 #include "style.h"
 
@@ -53,7 +52,7 @@ public:
     /** Initializes the CairoRenderContext according to the specified
     SPDocument. A set*Target function can only be called on the context
     before setupDocument. */
-    bool setupDocument(CairoRenderContext *ctx, SPDocument *doc);
+    bool setupDocument(CairoRenderContext *ctx, SPDocument *doc, bool pageBoundingBox, SPItem *base);
 
     /** Traverses the object tree and invokes the render methods. */
     void renderItem(CairoRenderContext *ctx, SPItem *item);
