@@ -31,9 +31,10 @@ class PrintLatex : public Inkscape::Extension::Implementation::Implementation {
     float  _height;
     FILE * _stream;
     
-    std::stack<NR::Matrix> m_tr_stack;
+    std::stack<Geom::Matrix> m_tr_stack;
 
-    void print_bpath (SVGOStringStream &os, const NArtBpath *bp, const NRMatrix *transform);
+    void print_pathvector(SVGOStringStream &os, Geom::PathVector const &pathv_in, const Geom::Matrix * /*transform*/);
+    void print_2geomcurve(SVGOStringStream &os, Geom::Curve const & c );
 
 public:
         PrintLatex (void);
@@ -46,12 +47,12 @@ public:
         virtual unsigned int finish (Inkscape::Extension::Print * module);
 
         /* Rendering methods */
-	virtual unsigned int bind(Inkscape::Extension::Print *module, NRMatrix const *transform, float opacity);
-	virtual unsigned int release(Inkscape::Extension::Print *module);
+        virtual unsigned int bind(Inkscape::Extension::Print *module, Geom::Matrix const *transform, float opacity);
+        virtual unsigned int release(Inkscape::Extension::Print *module);
 
-	virtual unsigned int fill (Inkscape::Extension::Print * module, const NRBPath *bpath, const NRMatrix *ctm, const SPStyle *style,
+        virtual unsigned int fill (Inkscape::Extension::Print * module, Geom::PathVector const &pathv, const Geom::Matrix *ctm, const SPStyle *style,
                 const NRRect *pbox, const NRRect *dbox, const NRRect *bbox);
-        virtual unsigned int stroke (Inkscape::Extension::Print * module, const NRBPath *bpath, const NRMatrix *transform, const SPStyle *style,
+        virtual unsigned int stroke (Inkscape::Extension::Print * module, Geom::PathVector const &pathv, const Geom::Matrix *transform, const SPStyle *style,
                 const NRRect *pbox, const NRRect *dbox, const NRRect *bbox);
         virtual unsigned int comment(Inkscape::Extension::Print *module, const char * comment);
         bool textToPath (Inkscape::Extension::Print * ext);
