@@ -98,7 +98,7 @@ WpgInput::init(void) {
     Inkscape::Extension::Extension * ext;
 
     ext = Inkscape::Extension::build_from_mem(
-        "<inkscape-extension>\n"
+        "<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
             "<name>" N_("WPG Input") "</name>\n"
             "<id>org.inkscape.input.wpg</id>\n"
             "<input>\n"
