@@ -12,7 +12,6 @@
 
 #include "extension.h"
 
-#include "libnr/nr-path.h"
 #include "display/nr-arena-forward.h"
 #include "forward.h"
 
@@ -41,18 +40,18 @@ public:
     unsigned int  finish      (void);
 
     /* Rendering methods */
-    unsigned int  bind        (NRMatrix const *transform,
+    unsigned int  bind        (Geom::Matrix const *transform,
                                float opacity);
     unsigned int  release     (void);
     unsigned int  comment     (const char * comment);
-    unsigned int  fill        (NRBPath const *bpath,
-                               NRMatrix const *ctm,
+    unsigned int  fill        (Geom::PathVector const &pathv,
+                               Geom::Matrix const *ctm,
                                SPStyle const *style,
                                NRRect const *pbox,
                                NRRect const *dbox,
                                NRRect const *bbox);
-    unsigned int  stroke      (NRBPath const *bpath,
-                               NRMatrix const *transform,
+    unsigned int  stroke      (Geom::PathVector const &pathv,
+                               Geom::Matrix const *transform,
                                SPStyle const *style,
                                NRRect const *pbox,
                                NRRect const *dbox,
@@ -61,10 +60,10 @@ public:
                                unsigned int w,
                                unsigned int h,
                                unsigned int rs,
-                               NRMatrix const *transform,
+                               Geom::Matrix const *transform,
                                SPStyle const *style);
     unsigned int  text        (char const *text,
-                               NR::Point p,
+                               Geom::Point p,
                                SPStyle const *style);
     bool          textToPath  (void);
     bool          fontEmbedded  (void);
