@@ -8,7 +8,7 @@
  *   Ted Gould <ted@gould.cx>
  *   Johan Engelen <johan@shouraizou.nl>
  *
- * Copyright (C) 2006-2007 Johan Engelen 
+ * Copyright (C) 2006-2007 Johan Engelen
  * Copyright (C) 2002-2004 Ted Gould
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -20,6 +20,9 @@
 
 #include <interface.h>
 
+#include "system.h"
+#include "preferences.h"
+#include "extension.h"
 #include "db.h"
 #include "input.h"
 #include "output.h"
@@ -28,6 +31,8 @@
 #include "print.h"
 #include "implementation/script.h"
 #include "implementation/xslt.h"
+#include "xml/rebase-hrefs.h"
+#include "io/sys.h"
 /* #include "implementation/plugin.h" */
 
 namespace Inkscape {
@@ -91,11 +96,11 @@ open(Extension *key, gchar const *filename)
 
     SPDocument *doc = imod->open(filename);
     if (!doc) {
-        return NULL;
+        throw Input::open_failed();
     }
 
     if (last_chance_svg) {
-        /* We can't call sp_ui_error_dialog because we may be 
+        /* We can't call sp_ui_error_dialog because we may be
            running from the console, in which case calling sp_ui
            routines will cause a segfault.  See bug 1000350 - bryce */
         // sp_ui_error_dialog(_("Format autodetect failed. The file is being opened as SVG."));
@@ -182,7 +187,8 @@ open_internal(Extension *in_plug, gpointer in_data)
  * Lastly, the save function is called in the module itself.
  */
 void
-save(Extension *key, SPDocument *doc, gchar const *filename, bool setextension, bool check_overwrite, bool official)
+save(Extension *key, SPDocument *doc, gchar const *filename, bool setextension, bool check_overwrite, bool official,
+    Inkscape::Extension::FileSaveMethod save_method)
 {
     Output *omod;
     if (key == NULL) {
@@ -200,7 +206,7 @@ save(Extension *key, SPDocument *doc, gchar const *filename, bool setextension,
         }
         /* If autodetect fails, save as Inkscape SVG */
         if (omod == NULL) {
-            omod = dynamic_cast<Output *>(db.get(SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE));
+            // omod = dynamic_cast<Output *>(db.get(SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE)); use exception and let user choose
         }
     } else {
         omod = dynamic_cast<Output *>(key);
@@ -217,8 +223,9 @@ save(Extension *key, SPDocument *doc, gchar const *filename, bool setextension,
         throw Output::save_failed();
     }
 
-    if (!omod->prefs())
-        return;
+    if (!omod->prefs()) {
+        throw Output::save_cancelled();
+    }
 
     gchar *fileName = NULL;
     if (setextension) {
@@ -242,58 +249,87 @@ save(Extension *key, SPDocument *doc, gchar const *filename, bool setextension,
         throw Output::no_overwrite();
     }
 
+    // test if the file exists and is writable
+    // the test only checks the file attributes and might pass where ACL does not allow to write
+    if (Inkscape::IO::file_test(filename, G_FILE_TEST_EXISTS) && !Inkscape::IO::file_is_writable(filename)) {
+        g_free(fileName);
+        throw Output::file_read_only();
+    }
+
     Inkscape::XML::Node *repr = sp_document_repr_root(doc);
 
-    // remember attributes in case this is an unofficial save
+
+    // remember attributes in case this is an unofficial save and/or overwrite fails
+    gchar *saved_uri = g_strdup(doc->uri);
     bool saved_modified = false;
     gchar *saved_output_extension = NULL;
     gchar *saved_dataloss = NULL;
-    gchar *saved_uri = NULL;
-    if (!official) {
-        saved_modified = doc->isModifiedSinceSave();
-        if (repr->attribute("inkscape:output_extension")) {
-            saved_output_extension = g_strdup(repr->attribute("inkscape:output_extension"));
-        }
-        if (repr->attribute("inkscape:dataloss")) {
-            saved_dataloss = g_strdup(repr->attribute("inkscape:dataloss"));
-        }
-        if (doc->uri) {
-            saved_uri = g_strdup(doc->uri);    
-        }
-    }    
-
-    // update attributes:
-    bool saved = sp_document_get_undo_sensitive(doc);
-    sp_document_set_undo_sensitive (doc, false); 
-        // save the filename for next use
-        sp_document_set_uri(doc, fileName);
-        // also save the extension for next use
-        repr->setAttribute("inkscape:output_extension", omod->get_id());
-        // set the "dataloss" attribute if the chosen extension is lossy
-        repr->setAttribute("inkscape:dataloss", NULL);
-        if ( omod->causes_dataloss() ) {
-            repr->setAttribute("inkscape:dataloss", "true");
-        }
-    sp_document_set_undo_sensitive (doc, saved);
-    doc->setModifiedSinceSave(false);
-
-    omod->save(doc, fileName);
-    
-    // if it is an unofficial save, set the modified attributes back to what they were    
-    if ( !official) {
-        saved = sp_document_get_undo_sensitive(doc);
-        sp_document_set_undo_sensitive (doc, false);
-            repr->setAttribute("inkscape:output_extension", saved_output_extension);
-            repr->setAttribute("inkscape:dataloss", saved_dataloss);
-            sp_document_set_uri(doc, saved_uri);
-        sp_document_set_undo_sensitive (doc, saved);
-        doc->setModifiedSinceSave(saved_modified);
+    saved_modified = doc->isModifiedSinceSave();
+    saved_output_extension = g_strdup(get_file_save_extension(save_method).c_str());
+    saved_dataloss = g_strdup(repr->attribute("inkscape:dataloss"));
+    if (official) {
+        /* The document is changing name/uri. */
+        sp_document_change_uri_and_hrefs(doc, fileName);
     }
-    
-    if (saved_output_extension)  g_free(saved_output_extension);
-    if (saved_dataloss)          g_free(saved_dataloss);
-    if (saved_uri)               g_free(saved_uri);    
-    
+
+    // Update attributes:
+    {
+        bool const saved = sp_document_get_undo_sensitive(doc);
+        sp_document_set_undo_sensitive(doc, false);
+        {
+            // also save the extension for next use
+            store_file_extension_in_prefs (omod->get_id(), save_method);
+            // set the "dataloss" attribute if the chosen extension is lossy
+            repr->setAttribute("inkscape:dataloss", NULL);
+            if (omod->causes_dataloss()) {
+                repr->setAttribute("inkscape:dataloss", "true");
+            }
+        }
+        sp_document_set_undo_sensitive(doc, saved);
+        doc->setModifiedSinceSave(false);
+    }
+
+    try {
+        omod->save(doc, fileName);
+    }
+    catch(...) {
+        // revert attributes in case of official and overwrite
+        if(check_overwrite && official) {
+            bool const saved = sp_document_get_undo_sensitive(doc);
+            sp_document_set_undo_sensitive(doc, false);
+            {
+                store_file_extension_in_prefs (saved_output_extension, save_method);
+                repr->setAttribute("inkscape:dataloss", saved_dataloss);
+            }
+            sp_document_set_undo_sensitive(doc, saved);
+            sp_document_change_uri_and_hrefs(doc, saved_uri);
+        }
+        doc->setModifiedSinceSave(saved_modified);
+        // free used ressources
+        g_free(saved_output_extension);
+        g_free(saved_dataloss);
+        g_free(saved_uri);
+
+        g_free(fileName);
+
+        throw Inkscape::Extension::Output::save_failed();
+    }
+
+    // If it is an unofficial save, set the modified attributes back to what they were.
+    if ( !official) {
+        bool const saved = sp_document_get_undo_sensitive(doc);
+        sp_document_set_undo_sensitive(doc, false);
+        {
+            store_file_extension_in_prefs (saved_output_extension, save_method);
+            repr->setAttribute("inkscape:dataloss", saved_dataloss);
+        }
+        sp_document_set_undo_sensitive(doc, saved);
+        doc->setModifiedSinceSave(saved_modified);
+
+        g_free(saved_output_extension);
+        g_free(saved_dataloss);
+    }
+
     g_free(fileName);
     return;
 }
@@ -388,10 +424,8 @@ build_from_reprdoc(Inkscape::XML::Document *doc, Implementation::Implementation
 
     Inkscape::XML::Node *repr = doc->root();
 
-    /* sp_repr_print(repr); */
-
-    if (strcmp(repr->name(), "inkscape-extension")) {
-        g_warning("Extension definition started with <%s> instead of <inkscape-extension>.  Extension will not be created.\n", repr->name());
+    if (strcmp(repr->name(), INKSCAPE_EXTENSION_NS "inkscape-extension")) {
+        g_warning("Extension definition started with <%s> instead of <" INKSCAPE_EXTENSION_NS "inkscape-extension>.  Extension will not be created. See http://wiki.inkscape.org/wiki/index.php/Extensions for reference.\n", repr->name());
         return NULL;
     }
 
@@ -399,19 +433,19 @@ build_from_reprdoc(Inkscape::XML::Document *doc, Implementation::Implementation
     while (child_repr != NULL) {
         char const *element_name = child_repr->name();
         /* printf("Child: %s\n", child_repr->name()); */
-        if (!strcmp(element_name, "input")) {
+        if (!strcmp(element_name, INKSCAPE_EXTENSION_NS "input")) {
             module_functional_type = MODULE_INPUT;
-        } else if (!strcmp(element_name, "output")) {
+        } else if (!strcmp(element_name, INKSCAPE_EXTENSION_NS "output")) {
             module_functional_type = MODULE_OUTPUT;
-        } else if (!strcmp(element_name, "effect")) {
+        } else if (!strcmp(element_name, INKSCAPE_EXTENSION_NS "effect")) {
             module_functional_type = MODULE_FILTER;
-        } else if (!strcmp(element_name, "print")) {
+        } else if (!strcmp(element_name, INKSCAPE_EXTENSION_NS "print")) {
             module_functional_type = MODULE_PRINT;
-        } else if (!strcmp(element_name, "path-effect")) {
+        } else if (!strcmp(element_name, INKSCAPE_EXTENSION_NS "path-effect")) {
             module_functional_type = MODULE_PATH_EFFECT;
-        } else if (!strcmp(element_name, "script")) {
+        } else if (!strcmp(element_name, INKSCAPE_EXTENSION_NS "script")) {
             module_implementation_type = MODULE_EXTENSION;
-        } else if (!strcmp(element_name, "xslt")) {
+        } else if (!strcmp(element_name, INKSCAPE_EXTENSION_NS "xslt")) {
             module_implementation_type = MODULE_XSLT;
 #if 0
         } else if (!strcmp(element_name, "plugin")) {
@@ -494,9 +528,7 @@ build_from_reprdoc(Inkscape::XML::Document *doc, Implementation::Implementation
 Extension *
 build_from_file(gchar const *filename)
 {
-    /* TODO: Need to define namespace here, need to write the
-       DTD in general for this stuff */
-    Inkscape::XML::Document *doc = sp_repr_read_file(filename, NULL);
+    Inkscape::XML::Document *doc = sp_repr_read_file(filename, INKSCAPE_EXTENSION_URI);
     Extension *ext = build_from_reprdoc(doc, NULL);
     if (ext != NULL)
         Inkscape::GC::release(doc);
@@ -517,12 +549,112 @@ build_from_file(gchar const *filename)
 Extension *
 build_from_mem(gchar const *buffer, Implementation::Implementation *in_imp)
 {
-    Inkscape::XML::Document *doc = sp_repr_read_mem(buffer, strlen(buffer), NULL);
+    Inkscape::XML::Document *doc = sp_repr_read_mem(buffer, strlen(buffer), INKSCAPE_EXTENSION_URI);
     Extension *ext = build_from_reprdoc(doc, in_imp);
     Inkscape::GC::release(doc);
     return ext;
 }
 
+/*
+ * TODO: Is it guaranteed that the returned extension is valid? If so, we can remove the check for
+ * filename_extension in sp_file_save_dialog().
+ */
+Glib::ustring
+get_file_save_extension (Inkscape::Extension::FileSaveMethod method) {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+    Glib::ustring extension;
+    switch (method) {
+        case FILE_SAVE_METHOD_SAVE_AS:
+        case FILE_SAVE_METHOD_TEMPORARY:
+            extension = prefs->getString("/dialogs/save_as/default");
+            break;
+        case FILE_SAVE_METHOD_SAVE_COPY:
+            extension = prefs->getString("/dialogs/save_copy/default");
+            break;
+        case FILE_SAVE_METHOD_INKSCAPE_SVG:
+            extension = SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE;
+            break;
+    }
+
+    if(extension.empty())
+        extension = SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE;
+
+    return extension;
+}
+
+Glib::ustring
+get_file_save_path (SPDocument *doc, FileSaveMethod method) {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+    Glib::ustring path;
+    switch (method) {
+        case FILE_SAVE_METHOD_SAVE_AS:
+        {
+            bool use_current_dir = prefs->getBool("/dialogs/save_as/use_current_dir");
+            if (doc->uri && use_current_dir) {
+                path = Glib::path_get_dirname(doc->uri);
+            } else {
+                path = prefs->getString("/dialogs/save_as/path");
+            }
+            break;
+        }
+        case FILE_SAVE_METHOD_TEMPORARY:
+            path = prefs->getString("/dialogs/save_as/path");
+            break;
+        case FILE_SAVE_METHOD_SAVE_COPY:
+            path = prefs->getString("/dialogs/save_copy/path");
+            break;
+        case FILE_SAVE_METHOD_INKSCAPE_SVG:
+            if (doc->uri) {
+                path = Glib::path_get_dirname(doc->uri);
+            } else {
+                // FIXME: should we use the save_as path here or something else? Maybe we should
+                // leave this as a choice to the user.
+                path = prefs->getString("/dialogs/save_as/path");
+            }
+    }
+
+    if(path.empty())
+        path = g_get_home_dir(); // Is this the most sensible solution? Note that we should avoid
+                                 // g_get_current_dir because this leads to problems on OS X where
+                                 // Inkscape opens the dialog inside application bundle when it is
+                                 // invoked for the first teim.
+
+    return path;
+}
+
+void
+store_file_extension_in_prefs (Glib::ustring extension, FileSaveMethod method) {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+    switch (method) {
+        case FILE_SAVE_METHOD_SAVE_AS:
+        case FILE_SAVE_METHOD_TEMPORARY:
+            prefs->setString("/dialogs/save_as/default", extension);
+            break;
+        case FILE_SAVE_METHOD_SAVE_COPY:
+            prefs->setString("/dialogs/save_copy/default", extension);
+            break;
+        case FILE_SAVE_METHOD_INKSCAPE_SVG:
+            // do nothing
+            break;
+    }
+}
+
+void
+store_save_path_in_prefs (Glib::ustring path, FileSaveMethod method) {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+    switch (method) {
+        case FILE_SAVE_METHOD_SAVE_AS:
+        case FILE_SAVE_METHOD_TEMPORARY:
+            prefs->setString("/dialogs/save_as/path", path);
+            break;
+        case FILE_SAVE_METHOD_SAVE_COPY:
+            prefs->setString("/dialogs/save_copy/path", path);
+            break;
+        case FILE_SAVE_METHOD_INKSCAPE_SVG:
+            // do nothing
+            break;
+    }
+}
 
 } } /* namespace Inkscape::Extension */
 
