@@ -31,7 +31,10 @@
 #include "libavoid/connector.h"
 #include "libavoid/graph.h"
 #include "libavoid/router.h"
+#include <algorithm>
 #include <vector>
+#include <climits>
+#include <limits.h>
 #include <math.h>
 
 namespace Avoid {
@@ -270,7 +273,7 @@ static void dijkstraPath(ConnRef *lineRef, VertInf *src, VertInf *tar)
 {
     Router *router = src->_router;
 
-    double unseen = (double) INT_MAX;
+    double unseen = (double) __INT_MAX__;
 
     // initialize arrays
     VertInf *finish = router->vertices.end();
