@@ -2,7 +2,7 @@
 #include "conjugate_gradient.h"
 #include "straightener.h"
 #include "shortest_paths.h"
-#include "isnan.h"
+#include "2geom/isnan.h"
 
 namespace cola {
 
@@ -132,7 +132,7 @@ void ConstrainedMajorizationLayout::majlayout(
             }
             b[i] += degree * coords[i];
         }
-        assert(!isNaN(b[i]));
+        assert(!IS_NAN(b[i]));
     }
     if(constrainedLayout) {
         setupDummyVars();
