@@ -4,7 +4,7 @@
 #include <cmath>
 
 #include "libnr/in-svg-plane.h"
-#include "isnan.h"
+#include "2geom/isnan.h"
 
 class InSvgPlaneTest : public CxxTest::TestSuite
 {
@@ -21,8 +21,8 @@ public:
         small_n3_4( -3.0 * small, 4.0 * small ),
         part_nan( 3., nan )
     {
-        setupValid &= isNaN(nan);
-        setupValid &= !isNaN(small);
+        setupValid &= IS_NAN(nan);
+        setupValid &= !IS_NAN(small);
     }
     virtual ~InSvgPlaneTest() {}
 
