@@ -8,13 +8,15 @@
  *
  */
 
+#define PANGO_ENABLE_ENGINE
+
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 #include <libnr/nr-rect.h>
 #include <libnrtype/font-glyph.h>
 #include <libnrtype/font-instance.h>
-
+#include <2geom/pathvector.h>
 #include <livarot/Path.h>
 
 #include "RasterFont.h"
@@ -87,7 +89,7 @@ bool  font_style_equal::operator()(const font_style &a,const font_style &b) {
 typedef struct ft2_to_liv {
 	Path*        theP;
 	double       scale;
-	NR::Point    last;
+	Geom::Point    last;
 } ft2_to_liv;
 
 // Note: Freetype 2.2.1 redefined function signatures for functions to be placed in an
@@ -105,7 +107,7 @@ typedef FT_Vector FREETYPE_VECTOR;
 // see nr-type-ft2.cpp for the freetype -> artBPath on which this code is based
 static int ft2_move_to(FREETYPE_VECTOR *to, void * i_user) {
 	ft2_to_liv* user=(ft2_to_liv*)i_user;
-	NR::Point   p(user->scale*to->x,user->scale*to->y);
+	Geom::Point   p(user->scale*to->x,user->scale*to->y);
 	//	printf("m  t=%f %f\n",p[0],p[1]);
 	user->theP->MoveTo(p);
 	user->last=p;
@@ -115,7 +117,7 @@ static int ft2_move_to(FREETYPE_VECTOR *to, void * i_user) {
 static int ft2_line_to(FREETYPE_VECTOR *to, void *i_user)
 {
 	ft2_to_liv* user=(ft2_to_liv*)i_user;
-	NR::Point   p(user->scale*to->x,user->scale*to->y);
+	Geom::Point   p(user->scale*to->x,user->scale*to->y);
 	//	printf("l  t=%f %f\n",p[0],p[1]);
 	user->theP->LineTo(p);
 	user->last=p;
@@ -125,7 +127,7 @@ static int ft2_line_to(FREETYPE_VECTOR *to, void *i_user)
 static int ft2_conic_to(FREETYPE_VECTOR *control, FREETYPE_VECTOR *to, void *i_user)
 {
 	ft2_to_liv* user=(ft2_to_liv*)i_user;
-	NR::Point   p(user->scale*to->x,user->scale*to->y),c(user->scale*control->x,user->scale*control->y);
+	Geom::Point   p(user->scale*to->x,user->scale*to->y),c(user->scale*control->x,user->scale*control->y);
 	//	printf("b c=%f %f  t=%f %f\n",c[0],c[1],p[0],p[1]);
 	user->theP->BezierTo(p);
 	user->theP->IntermBezierTo(c);
@@ -137,7 +139,7 @@ static int ft2_conic_to(FREETYPE_VECTOR *control, FREETYPE_VECTOR *to, void *i_u
 static int ft2_cubic_to(FREETYPE_VECTOR *control1, FREETYPE_VECTOR *control2, FREETYPE_VECTOR *to, void *i_user)
 {
 	ft2_to_liv* user=(ft2_to_liv*)i_user;
-	NR::Point   p(user->scale*to->x,user->scale*to->y),
+	Geom::Point   p(user->scale*to->x,user->scale*to->y),
 	c1(user->scale*control1->x,user->scale*control1->y),
 	c2(user->scale*control2->x,user->scale*control2->y);
 	//	printf("c c1=%f %f  c2=%f %f   t=%f %f\n",c1[0],c1[1],c2[0],c2[1],p[0],p[1]);
@@ -178,7 +180,7 @@ font_instance::~font_instance(void)
 
 	for (int i=0;i<nbGlyph;i++) {
 		if ( glyphs[i].outline ) delete glyphs[i].outline;
-		if ( glyphs[i].artbpath ) free(glyphs[i].artbpath);
+        if ( glyphs[i].pathvector ) delete glyphs[i].pathvector;
 	}
 	if ( glyphs ) free(glyphs);
 	nbGlyph=maxGlyph=0;
@@ -261,53 +263,53 @@ unsigned int font_instance::Attribute(const gchar *key, gchar *str, unsigned int
 	} else if ( strcmp(key,"style") == 0 ) {
 		PangoStyle v=pango_font_description_get_style(descr);
 		if ( v == PANGO_STYLE_ITALIC ) {
-			res="italic";
+			res=(char*)"italic";
 		} else if ( v == PANGO_STYLE_OBLIQUE ) {
-			res="oblique";
+			res=(char*)"oblique";
 		} else {
-			res="normal";
+			res=(char*)"normal";
 		}
 		free_res=false;
 	} else if ( strcmp(key,"weight") == 0 ) {
 		PangoWeight v=pango_font_description_get_weight(descr);
 		if ( v <= PANGO_WEIGHT_ULTRALIGHT ) {
-			res="200";
+			res=(char*)"200";
 		} else if ( v <= PANGO_WEIGHT_LIGHT ) {
-			res="300";
+			res=(char*)"300";
 		} else if ( v <= PANGO_WEIGHT_NORMAL ) {
-			res="normal";
+			res=(char*)"normal";
 		} else if ( v <= PANGO_WEIGHT_BOLD ) {
-			res="bold";
+			res=(char*)"bold";
 		} else if ( v <= PANGO_WEIGHT_ULTRABOLD ) {
-		    res="800";
+		    res=(char*)"800";
 		} else { // HEAVY
-			res="900";
+			res=(char*)"900";
 		}
 		free_res=false;
 	} else if ( strcmp(key,"stretch") == 0 ) {
 		PangoStretch v=pango_font_description_get_stretch(descr);
 		if ( v <= PANGO_STRETCH_EXTRA_CONDENSED ) {
-			res="extra-condensed";
+			res=(char*)"extra-condensed";
 		} else if ( v <= PANGO_STRETCH_CONDENSED ) {
-			res="condensed";
+			res=(char*)"condensed";
 		} else if ( v <= PANGO_STRETCH_SEMI_CONDENSED ) {
-			res="semi-condensed";
+			res=(char*)"semi-condensed";
 		} else if ( v <= PANGO_STRETCH_NORMAL ) {
-			res="normal";
+			res=(char*)"normal";
 		} else if ( v <= PANGO_STRETCH_SEMI_EXPANDED ) {
-			res="semi-expanded";
+			res=(char*)"semi-expanded";
 		} else if ( v <= PANGO_STRETCH_EXPANDED ) {
-			res="expanded";
+			res=(char*)"expanded";
 		} else {
-			res="extra-expanded";
+			res=(char*)"extra-expanded";
 		}
 		free_res=false;
 	} else if ( strcmp(key,"variant") == 0 ) {
 		PangoVariant v=pango_font_description_get_variant(descr);
 		if ( v == PANGO_VARIANT_SMALL_CAPS ) {
-			res="small-caps";
+			res=(char*)"small-caps";
 		} else {
-			res="normal";
+			res=(char*)"normal";
 		}
 		free_res=false;
 	} else {
@@ -407,9 +409,9 @@ int font_instance::MapUnicodeChar(gunichar c)
 
 
 #ifdef USE_PANGO_WIN32
-static inline NR::Point pointfx_to_nrpoint(const POINTFX &p, double scale)
+static inline Geom::Point pointfx_to_nrpoint(const POINTFX &p, double scale)
 {
-    return NR::Point(*(long*)&p.x / 65536.0 * scale,
+    return Geom::Point(*(long*)&p.x / 65536.0 * scale,
                      *(long*)&p.y / 65536.0 * scale);
 }
 #endif
@@ -429,7 +431,7 @@ void font_instance::LoadGlyph(int glyph_id)
 		}
 		font_glyph  n_g;
 		n_g.outline=NULL;
-		n_g.artbpath=NULL;
+        n_g.pathvector=NULL;
 		n_g.bbox[0]=n_g.bbox[1]=n_g.bbox[2]=n_g.bbox[3]=0;
 		bool   doAdd=false;
 
@@ -486,9 +488,9 @@ void font_instance::LoadGlyph(int glyph_id)
                                 {
                                     g_assert(polyCurve->cpfx >= 2);
                                     endp -= 2;
-                                    NR::Point this_mid=pointfx_to_nrpoint(p[0], scale);
+                                    Geom::Point this_mid=pointfx_to_nrpoint(p[0], scale);
                                     while ( p != endp ) {
-                                        NR::Point next_mid=pointfx_to_nrpoint(p[1], scale);
+                                        Geom::Point next_mid=pointfx_to_nrpoint(p[1], scale);
 	                                    n_g.outline->BezierTo((next_mid+this_mid)/2);
 	                                    n_g.outline->IntermBezierTo(this_mid);
 	                                    n_g.outline->EndBezierTo();
@@ -546,7 +548,7 @@ void font_instance::LoadGlyph(int glyph_id)
 				ft2_to_liv   tData;
 				tData.theP=n_g.outline;
 				tData.scale=1.0/((double)theFace->units_per_EM);
-				tData.last=NR::Point(0,0);
+				tData.last=Geom::Point(0,0);
 				FT_Outline_Decompose (&theFace->glyph->outline, &ft2_outline_funcs, &tData);
 			}
 			doAdd=true;
@@ -556,7 +558,7 @@ void font_instance::LoadGlyph(int glyph_id)
 		if ( doAdd ) {
 			if ( n_g.outline ) {
 				n_g.outline->FastBBox(n_g.bbox[0],n_g.bbox[1],n_g.bbox[2],n_g.bbox[3]);
-				n_g.artbpath=n_g.outline->MakeArtBPath();
+                n_g.pathvector=n_g.outline->MakePathVector();
 			}
 			glyphs[nbGlyph]=n_g;
 			id_to_no[glyph_id]=nbGlyph;
@@ -613,7 +615,7 @@ bool font_instance::FontSlope(double &run, double &rise)
 	return true;
 }
 
-NR::Maybe<NR::Rect> font_instance::BBox(int glyph_id)
+Geom::OptRect font_instance::BBox(int glyph_id)
 {
 	int no=-1;
 	if ( id_to_no.find(glyph_id) == id_to_no.end() ) {
@@ -627,11 +629,11 @@ NR::Maybe<NR::Rect> font_instance::BBox(int glyph_id)
 		no=id_to_no[glyph_id];
 	}
 	if ( no < 0 ) {
-            return NR::Nothing();
+            return Geom::OptRect();
         } else {
-	    NR::Point rmin(glyphs[no].bbox[0],glyphs[no].bbox[1]);
-	    NR::Point rmax(glyphs[no].bbox[2],glyphs[no].bbox[3]);
-	    return NR::Rect(rmin, rmax);
+	    Geom::Point rmin(glyphs[no].bbox[0],glyphs[no].bbox[1]);
+	    Geom::Point rmax(glyphs[no].bbox[2],glyphs[no].bbox[3]);
+	    return Geom::Rect(rmin, rmax);
         }
 }
 
@@ -658,21 +660,21 @@ Path* font_instance::Outline(int glyph_id,Path* copyInto)
 	return src_o;
 }
 
-void* font_instance::ArtBPath(int glyph_id)
+Geom::PathVector* font_instance::PathVector(int glyph_id)
 {
-	int no=-1;
-	if ( id_to_no.find(glyph_id) == id_to_no.end() ) {
-		LoadGlyph(glyph_id);
-		if ( id_to_no.find(glyph_id) == id_to_no.end() ) {
-			// didn't load
-		} else {
-			no=id_to_no[glyph_id];
-		}
-	} else {
-		no=id_to_no[glyph_id];
-	}
-	if ( no < 0 ) return NULL;
-	return glyphs[no].artbpath;
+    int no = -1;
+    if ( id_to_no.find(glyph_id) == id_to_no.end() ) {
+        LoadGlyph(glyph_id);
+        if ( id_to_no.find(glyph_id) == id_to_no.end() ) {
+            // didn't load
+        } else {
+            no = id_to_no[glyph_id];
+        }
+    } else {
+        no = id_to_no[glyph_id];
+    }
+    if ( no < 0 ) return NULL;
+    return glyphs[no].pathvector;
 }
 
 double font_instance::Advance(int glyph_id,bool vertical)
@@ -699,7 +701,7 @@ double font_instance::Advance(int glyph_id,bool vertical)
 }
 
 
-raster_font* font_instance::RasterFont(const NR::Matrix &trs, double stroke_width, bool vertical, JoinType stroke_join, ButtType stroke_cap, float /*miter_limit*/)
+raster_font* font_instance::RasterFont(const Geom::Matrix &trs, double stroke_width, bool vertical, JoinType stroke_join, ButtType stroke_cap, float /*miter_limit*/)
 {
 	font_style  nStyle;
 	nStyle.transform=trs;
