@@ -11,21 +11,35 @@
  * This code is in public domain
  */
 
-#include <glib.h>
+#include <glib/gtypes.h>
 
-typedef struct _NRNameList NRNameList;
-typedef struct _NRTypeDict NRTypeDict;
+struct NRNameList;
+struct NRStyleList;
+struct NRTypeDict;
 
 typedef void (* NRNameListDestructor) (NRNameList *list);
+typedef void (* NRStyleListDestructor) (NRStyleList *list);
 
-struct _NRNameList {
-	unsigned long length;
+struct NRNameList {
+	guint length;
       guchar **names;
       guchar **families;
 	NRNameListDestructor destructor;
 };
 
+struct NRStyleRecord {
+	const char *name;
+      const char *descr;
+};
+
+struct NRStyleList {
+	guint length;
+      NRStyleRecord *records;
+	NRStyleListDestructor destructor;
+};
+
 void nr_name_list_release (NRNameList *list);
+void nr_style_list_release (NRStyleList *list);
 
 NRTypeDict *nr_type_dict_new (void);
 
