@@ -1,7 +1,7 @@
 #ifndef SEEN_MENUS_SKELETON_H
 #define SEEN_MENUS_SKELETON_H
 
-#include <inkscape_version.h>
+#include "config.h"
 
 #ifdef __cplusplus
 #undef N_
@@ -9,7 +9,7 @@
 #endif
 
 static char const menus_skeleton[] =
-"<inkscape version=\"" INKSCAPE_VERSION "\"\n"
+"<inkscape version=\"" VERSION "\"\n"
 "  xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"\n"
 "  xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\">\n"
 "\n"
@@ -19,9 +19,7 @@ static char const menus_skeleton[] =
 "           <template-list/>\n"
 "       </submenu>\n"
 "       <verb verb-id=\"FileOpen\" />\n"
-"       <submenu name=\"" N_("Open _Recent") "\">\n"
-"           <recent-file-list/>\n"
-"       </submenu>\n"
+"       <recent-file-list/>\n"
 "       <verb verb-id=\"FileRevert\" />\n"
 "       <verb verb-id=\"FileSave\" />\n"
 "       <verb verb-id=\"FileSaveAs\" />\n"
@@ -31,6 +29,7 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"FileExport\" />\n"
 #ifdef WITH_GNOME_VFS
 "       <verb verb-id=\"FileImportFromOCAL\" />\n"
+//"       <verb verb-id=\"FileExportToOCAL\" />\n"
 #endif
 "       <separator/>\n"
 "       <verb verb-id=\"FilePrint\" />\n"
@@ -41,6 +40,8 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"DialogMetadata\" />\n"
 "       <verb verb-id=\"DialogPreferences\" />\n"
 "       <verb verb-id=\"DialogInput\" />\n"
+// TODO look at some dynamic option for changing the menu tree:
+//"       <verb verb-id=\"DialogInput2\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"FileClose\" />\n"
 "       <verb verb-id=\"FileQuit\" />\n"
@@ -55,7 +56,6 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"EditPaste\" />\n"
 "       <verb verb-id=\"EditPasteInPlace\" />\n"
 "       <verb verb-id=\"EditPasteStyle\" />\n"
-"       <verb verb-id=\"EditPasteLivePathEffect\" />\n"
 "       <submenu name=\"" N_("Paste Si_ze") "\">\n"
 "           <verb verb-id=\"EditPasteSize\" />\n"
 "           <verb verb-id=\"EditPasteWidth\" />\n"
@@ -72,6 +72,7 @@ static char const menus_skeleton[] =
 "           <verb verb-id=\"EditClone\" />\n"
 "           <verb verb-id=\"DialogClonetiler\" />\n"
 "           <verb verb-id=\"EditUnlinkClone\" />\n"
+"           <verb verb-id=\"EditRelinkClone\" />\n"
 "           <verb verb-id=\"EditCloneSelectOriginal\" />\n"
 "       </submenu>\n"
 "       <verb verb-id=\"SelectionCreateBitmap\" />\n"
@@ -83,6 +84,8 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"EditInvert\" />\n"
 "       <verb verb-id=\"EditDeselect\" />\n"
 "       <separator/>\n"
+"       <verb verb-id=\"EditGuidesAroundPage\" />\n"
+"       <separator/>\n"
 "       <verb verb-id=\"DialogXMLEditor\" />\n"
 "   </submenu>\n"
 "   <submenu name=\"" N_("_View") "\">\n"
@@ -105,12 +108,13 @@ static char const menus_skeleton[] =
 "       <submenu name=\"" N_("_Display mode") "\">\n"
 "         <verb verb-id=\"ViewModeToggle\"/>\n"
 "         <verb verb-id=\"ViewModeNormal\" radio=\"yes\" default=\"yes\"/>\n"
+"         <verb verb-id=\"ViewModeNoFilters\" radio=\"yes\"/>\n"
 "         <verb verb-id=\"ViewModeOutline\" radio=\"yes\"/>\n"
 "       </submenu>\n"
 "       <separator/>\n"
 "       <verb verb-id=\"ToggleGrid\" />\n"
 "       <verb verb-id=\"ToggleGuides\" />\n"
-"       <verb verb-id=\"ToggleSnapping\" />\n"
+"       <verb verb-id=\"ToggleSnapGlobal\" />\n"
 "       <verb verb-id=\"ViewCmsToggle\" />\n"
 "       <separator/>\n"
 "       <submenu name=\"" N_("Show/Hide") "\">\n"
@@ -120,7 +124,7 @@ static char const menus_skeleton[] =
 "       <separator/>\n"
 "       <verb verb-id=\"DialogSwatches\" />\n"
 "       <verb verb-id=\"DialogDebug\" />\n"
-"       <verb verb-id=\"DialogScript\" />\n"
+//"       <verb verb-id=\"DialogScript\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"PrevWindow\" />\n"
 "       <verb verb-id=\"NextWindow\" />\n"
@@ -129,9 +133,12 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"ViewNew\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"FullScreen\" />\n"
+// Not quite ready to be in the menus.
+// "       <verb verb-id=\"FocusToggle\" />\n"
 "   </submenu>\n"
 "   <submenu name=\"" N_("_Layer") "\">\n"
 "       <verb verb-id=\"LayerNew\" />\n"
+"       <verb verb-id=\"LayerDuplicate\" />\n"
 "       <verb verb-id=\"LayerRename\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"LayerNext\" />\n"
@@ -151,7 +158,6 @@ static char const menus_skeleton[] =
 "   </submenu>\n"
 "   <submenu name=\"" N_("_Object") "\">\n"
 "       <verb verb-id=\"DialogFillStroke\" />\n"
-"       <verb verb-id=\"DialogFilterEffects\" />\n"
 "       <verb verb-id=\"DialogObjectProperties\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"SelectionGroup\" />\n"
@@ -213,9 +219,14 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"SelectionReverse\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"DialogLivePathEffect\" />\n"
+"       <verb verb-id=\"PasteLivePathEffect\" />\n"
+"       <verb verb-id=\"RemoveLivePathEffect\" />\n"
 "   </submenu>\n"
 "   <submenu name=\"" N_("_Text") "\">\n"
 "       <verb verb-id=\"DialogText\" />\n"
+#ifdef ENABLE_SVG_FONTS
+"       <verb verb-id=\"DialogSVGFonts\" />\n"
+#endif // ENABLE_SVG_FONTS
 "       <separator/>\n"
 "       <verb verb-id=\"SelectionTextToPath\" />\n"
 "       <verb verb-id=\"SelectionTextFromPath\" />\n"
@@ -225,8 +236,18 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"ObjectFlowtextToText\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"SelectionTextRemoveKerns\" />\n"
+#ifdef HAVE_ASPELL
+"       <separator/>\n"
+"       <verb verb-id=\"DialogSpellcheck\" />\n"
+#endif
 "   </submenu>\n"
-"   <submenu name=\"" N_("Effe_cts") "\">\n"
+"   <submenu name=\"" N_("Filter_s") "\">\n"
+"       <filters-list/>\n"
+"       <separator/>\n"
+"       <verb verb-id=\"DialogFilterEffects\" />\n"
+"       <verb verb-id=\"RemoveFilter\" />\n"
+"   </submenu>\n"
+"   <submenu name=\"" N_("Exte_nsions") "\">\n"
 "       <verb verb-id=\"EffectLast\" />\n"
 "       <verb verb-id=\"EffectLastPref\" />\n"
 "       <separator/>\n"
