@@ -22,7 +22,10 @@ extern "C" {
 
 #ifdef ENABLE_BINRELOC
 #  define INKSCAPE_APPICONDIR     BR_DATADIR( "/pixmaps" )
+#  define INKSCAPE_BINDDIR        BR_DATADIR( "/inkscape/bind" )
+#  define INKSCAPE_EXAMPLESDIR    BR_DATADIR( "/inkscape/examples" )
 #  define INKSCAPE_EXTENSIONDIR   BR_DATADIR( "/inkscape/extensions" )
+#  define INKSCAPE_FILTERDIR      BR_DATADIR( "/inkscape/filters" )
 #  define INKSCAPE_GRADIENTSDIR   BR_DATADIR( "/inkscape/gradients" )
 #  define INKSCAPE_KEYSDIR        BR_DATADIR( "/inkscape/keys" )
 #  define INKSCAPE_PIXMAPDIR      BR_DATADIR( "/inkscape/icons" )
@@ -31,35 +34,39 @@ extern "C" {
 #  define INKSCAPE_PATTERNSDIR    BR_DATADIR( "/inkscape/patterns" )
 #  define INKSCAPE_SCREENSDIR     BR_DATADIR( "/inkscape/screens" )
 #  define INKSCAPE_TUTORIALSDIR   BR_DATADIR( "/inkscape/tutorials" )
-#  define INKSCAPE_PLUGINDIR      BR_LIBDIR(  "/inkscape/plugins" )
 #  define INKSCAPE_TEMPLATESDIR   BR_DATADIR( "/inkscape/templates" )
 #  define INKSCAPE_UIDIR          BR_DATADIR( "/inkscape/ui" )
 //CREATE V0.1 support
-#    define CREATE_GRADIENTSDIR   BR_DATADIR( "/create/gradients/gimp" )  
+#    define CREATE_GRADIENTSDIR   BR_DATADIR( "/create/gradients/gimp" )
 #    define CREATE_PALETTESDIR    BR_DATADIR( "/create/swatches" )
 #    define CREATE_PATTERNSDIR    BR_DATADIR( "/create/patterns/vector" )
 #else
 #  ifdef WIN32
-#    define INKSCAPE_APPICONDIR   "pixmaps"
-#    define INKSCAPE_EXTENSIONDIR "share\\extensions"
-#    define INKSCAPE_GRADIENTSDIR "share\\gradients"
-#    define INKSCAPE_KEYSDIR      "share\\keys"
-#    define INKSCAPE_PIXMAPDIR    "share\\icons"
-#    define INKSCAPE_MARKERSDIR   "share\\markers"
-#    define INKSCAPE_PALETTESDIR  "share\\palettes"
-#    define INKSCAPE_PATTERNSDIR  "share\\patterns"
-#    define INKSCAPE_SCREENSDIR   "share\\screens"
-#    define INKSCAPE_TUTORIALSDIR "share\\tutorials"
-#    define INKSCAPE_PLUGINDIR    "plugins"
-#    define INKSCAPE_TEMPLATESDIR "share\\templates"
-#    define INKSCAPE_UIDIR        INKSCAPE_DATADIR "\\share\\ui"
+#    define INKSCAPE_APPICONDIR   WIN32_DATADIR("pixmaps")
+#    define INKSCAPE_BINDDIR      WIN32_DATADIR("share\\bind")
+#    define INKSCAPE_EXAMPLESDIR  WIN32_DATADIR("share\\examples")
+#    define INKSCAPE_EXTENSIONDIR WIN32_DATADIR("share\\extensions")
+#    define INKSCAPE_FILTERDIR    WIN32_DATADIR("share\\filters")
+#    define INKSCAPE_GRADIENTSDIR WIN32_DATADIR("share\\gradients")
+#    define INKSCAPE_KEYSDIR      WIN32_DATADIR("share\\keys")
+#    define INKSCAPE_PIXMAPDIR    WIN32_DATADIR("share\\icons")
+#    define INKSCAPE_MARKERSDIR   WIN32_DATADIR("share\\markers")
+#    define INKSCAPE_PALETTESDIR  WIN32_DATADIR("share\\palettes")
+#    define INKSCAPE_PATTERNSDIR  WIN32_DATADIR("share\\patterns")
+#    define INKSCAPE_SCREENSDIR   WIN32_DATADIR("share\\screens")
+#    define INKSCAPE_TUTORIALSDIR WIN32_DATADIR("share\\tutorials")
+#    define INKSCAPE_TEMPLATESDIR WIN32_DATADIR("share\\templates")
+#    define INKSCAPE_UIDIR        WIN32_DATADIR("share\\ui")
 //CREATE V0.1  WIN32 support
-#    define CREATE_GRADIENTSDIR INKSCAPE_DATADIR "create\\gradients\\gimp"
-#    define CREATE_PALETTESDIR  INKSCAPE_DATADIR "create\\swatches"
-#    define CREATE_PATTERNSDIR  INKSCAPE_DATADIR "create\\patterns\\vector"
+#    define CREATE_GRADIENTSDIR   WIN32_DATADIR("create\\gradients\\gimp")
+#    define CREATE_PALETTESDIR    WIN32_DATADIR("create\\swatches")
+#    define CREATE_PATTERNSDIR    WIN32_DATADIR("create\\patterns\\vector")
 #  elif defined ENABLE_OSX_APP_LOCATIONS
 #    define INKSCAPE_APPICONDIR   "Contents/Resources/pixmaps"
+#    define INKSCAPE_BINDDIR      "Contents/Resources/bind"
+#    define INKSCAPE_EXAMPLESDIR  "Contents/Resources/examples"
 #    define INKSCAPE_EXTENSIONDIR "Contents/Resources/extensions"
+#    define INKSCAPE_FILTERDIR    "Contents/Resources/filters"
 #    define INKSCAPE_GRADIENTSDIR "Contents/Resources/gradients"
 #    define INKSCAPE_KEYSDIR      "Contents/Resources/keys"
 #    define INKSCAPE_PIXMAPDIR    "Contents/Resources/icons"
@@ -68,7 +75,6 @@ extern "C" {
 #    define INKSCAPE_PATTERNSDIR  "Contents/Resources/patterns"
 #    define INKSCAPE_SCREENSDIR   "Contents/Resources/screens"
 #    define INKSCAPE_TUTORIALSDIR "Contents/Resources/tutorials"
-#    define INKSCAPE_PLUGINDIR    "Contents/Resources/plugins"
 #    define INKSCAPE_TEMPLATESDIR "Contents/Resources/templates"
 #    define INKSCAPE_UIDIR        "Contents/Resources/ui"
 //CREATE V0.1 support
@@ -77,7 +83,10 @@ extern "C" {
 #    define CREATE_PATTERNSDIR   "/Library/Application Support/create/patterns/vector"
 #  else
 #    define INKSCAPE_APPICONDIR   INKSCAPE_DATADIR "/pixmaps"
+#    define INKSCAPE_BINDDIR      INKSCAPE_DATADIR "/inkscape/bind"
+#    define INKSCAPE_EXAMPLESDIR  INKSCAPE_DATADIR "/inkscape/examples"
 #    define INKSCAPE_EXTENSIONDIR INKSCAPE_DATADIR "/inkscape/extensions"
+#    define INKSCAPE_FILTERDIR    INKSCAPE_DATADIR "/inkscape/filters"
 #    define INKSCAPE_GRADIENTSDIR INKSCAPE_DATADIR "/inkscape/gradients"
 #    define INKSCAPE_KEYSDIR      INKSCAPE_DATADIR "/inkscape/keys"
 #    define INKSCAPE_PIXMAPDIR    INKSCAPE_DATADIR "/inkscape/icons"
@@ -86,7 +95,6 @@ extern "C" {
 #    define INKSCAPE_PATTERNSDIR  INKSCAPE_DATADIR "/inkscape/patterns"
 #    define INKSCAPE_SCREENSDIR   INKSCAPE_DATADIR "/inkscape/screens"
 #    define INKSCAPE_TUTORIALSDIR INKSCAPE_DATADIR "/inkscape/tutorials"
-#    define INKSCAPE_PLUGINDIR    INKSCAPE_LIBDIR  "/inkscape/plugins"
 #    define INKSCAPE_TEMPLATESDIR INKSCAPE_DATADIR "/inkscape/templates"
 #    define INKSCAPE_UIDIR        INKSCAPE_DATADIR "/inkscape/ui"
 //CREATE V0.1 support
