@@ -2,12 +2,14 @@
 #define __SATISFIED_GUIDE_CNS_H__
 
 #include <forward.h>
-namespace NR { class Point; }
+#include <2geom/forward.h>
 #include <vector>
+#include <sp-item.h>
+
 class SPGuideConstraint;
 
 void satisfied_guide_cns(SPDesktop const &desktop,
-                         std::vector<NR::Point> const &snappoints,
+                         SnapPointsWithType const &snappoints,
                          std::vector<SPGuideConstraint> &cns);
 
 
