@@ -24,9 +24,14 @@ class MessageStack;
 class SelectionDescriber : public sigc::trackable {
 public:
     SelectionDescriber(Inkscape::Selection *selection, MessageStack *stack);
+    ~SelectionDescriber();
 
 private:
     void _updateMessageFromSelection(Inkscape::Selection *selection);
+    void _selectionModified(Inkscape::Selection *selection, guint /*flags*/);
+
+    sigc::connection *_selection_changed_connection;
+    sigc::connection *_selection_modified_connection;
 
     MessageContext _context;
 };
