@@ -18,9 +18,10 @@
 
 #include <vector>
 #include <map>
+#include <list>
 #include <sigc++/sigc++.h>
 
-#include "libnr/nr-rect.h"
+//#include "libnr/nr-rect.h"
 #include "libnr/nr-convex-hull.h"
 #include "forward.h"
 #include "gc-managed.h"
@@ -29,8 +30,11 @@
 #include "gc-soft-ptr.h"
 #include "util/list.h"
 #include "sp-item.h"
+#include "snapped-point.h"
 
 class SPItem;
+class SPBox3D;
+class Persp3D;
 
 namespace Inkscape {
 namespace XML {
@@ -54,7 +58,7 @@ namespace Inkscape {
  * at the given desktop. Both SPItem and SPRepr lists can be retrieved
  * from the selection. Many actions operate on the selection, so it is
  * widely used throughout the code.
- * It also implements its own asynchronous notification signals that 
+ * It also implements its own asynchronous notification signals that
  * UI elements can listen to.
  */
 class Selection : public Inkscape::GC::Managed<>,
@@ -225,6 +229,12 @@ public:
     ///      method for that
     GSList const *reprList();
 
+    /* list of all perspectives which have a 3D box in the current selection
+       (these may also be nested in groups) */
+    std::list<Persp3D *> const perspList();
+
+    std::list<SPBox3D *> const box3DList();
+
     /** @brief Returns the number of layers in which there are selected objects */
     guint numberOfLayers();
 
@@ -234,13 +244,13 @@ public:
     /** @brief Returns the bounding rectangle of the selection */
     NRRect *bounds(NRRect *dest, SPItem::BBoxType type = SPItem::APPROXIMATE_BBOX) const;
     /** @brief Returns the bounding rectangle of the selection */
-    NR::Maybe<NR::Rect> bounds(SPItem::BBoxType type = SPItem::APPROXIMATE_BBOX) const;
+    Geom::OptRect bounds(SPItem::BBoxType type = SPItem::APPROXIMATE_BBOX) const;
 
     /**
      * @brief Returns the bounding rectangle of the selection
      *
      * \todo how is this different from bounds()?
-     */ 
+     */
     NRRect *boundsInDocument(NRRect *dest, SPItem::BBoxType type = SPItem::APPROXIMATE_BBOX) const;
 
     /**
@@ -248,24 +258,24 @@ public:
      *
      * \todo how is this different from bounds()?
      */
-    NR::Maybe<NR::Rect> boundsInDocument(SPItem::BBoxType type = SPItem::APPROXIMATE_BBOX) const;
+    Geom::OptRect boundsInDocument(SPItem::BBoxType type = SPItem::APPROXIMATE_BBOX) const;
 
     /**
      * @brief Returns the rotation/skew center of the selection
      */
-    NR::Maybe<NR::Point> center() const;
+    boost::optional<Geom::Point> center() const;
 
     /**
      * @brief Gets the selection's snap points.
      * @return Selection's snap points
      */
-    std::vector<NR::Point> getSnapPoints(bool includeItemCenter) const;
+    std::vector<std::pair<Geom::Point, int> > getSnapPoints(SnapPreferences const *snapprefs) const;
 
     /**
      * @brief Gets the snap points of a selection that form a convex hull.
      * @return Selection's convex hull points
      */
-    std::vector<NR::Point> getSnapPointsConvexHull() const;
+    std::vector<std::pair<Geom::Point, int> > getSnapPointsConvexHull(SnapPreferences const *snapprefs) const;
 
     /**
      * @brief Connects a slot to be notified of selection changes
@@ -282,8 +292,8 @@ public:
     }
 
     /**
-     * @brief Connects a slot to be notified of selected 
-     *        object modifications 
+     * @brief Connects a slot to be notified of selected
+     *        object modifications
      *
      * This method connects the given slot such that it will
      * receive notifications whenever any selected item is
@@ -336,6 +346,14 @@ private:
     mutable GSList *_reprs;
     mutable GSList *_items;
 
+    void add_box_perspective(SPBox3D *box);
+    void add_3D_boxes_recursively(SPObject *obj);
+    void remove_box_perspective(SPBox3D *box);
+    void remove_3D_boxes_recursively(SPObject *obj);
+
+    std::map<Persp3D *, unsigned int> _persps;
+    std::list<SPBox3D *> _3dboxes;
+
     GC::soft_ptr<SPDesktop> _desktop;
     SPObject* _selection_context;
     guint _flags;
