@@ -57,8 +57,8 @@ NRArenaItem *sp_clippath_show(SPClipPath *cp, NRArena *arena, unsigned int key);
 void sp_clippath_hide(SPClipPath *cp, unsigned int key);
 
 void sp_clippath_set_bbox(SPClipPath *cp, unsigned int key, NRRect *bbox);
-void sp_clippath_get_bbox(SPClipPath *cp, NRRect *bbox, NR::Matrix const &transform, unsigned const flags);
+void sp_clippath_get_bbox(SPClipPath *cp, NRRect *bbox, Geom::Matrix const &transform, unsigned const flags);
 
-const gchar *sp_clippath_create (GSList *reprs, SPDocument *document, NR::Matrix const* applyTransform);
+const gchar *sp_clippath_create (GSList *reprs, SPDocument *document, Geom::Matrix const* applyTransform);
 
 #endif
