@@ -36,7 +36,7 @@ static void sp_filter_primitive_build(SPObject *object, SPDocument *document, In
 static void sp_filter_primitive_release(SPObject *object);
 static void sp_filter_primitive_set(SPObject *object, unsigned int key, gchar const *value);
 static void sp_filter_primitive_update(SPObject *object, SPCtx *ctx, guint flags);
-static Inkscape::XML::Node *sp_filter_primitive_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
+static Inkscape::XML::Node *sp_filter_primitive_write(SPObject *object, Inkscape::XML::Document *doc, Inkscape::XML::Node *repr, guint flags);
 
 static SPObjectClass *filter_primitive_parent_class;
 
@@ -83,8 +83,8 @@ sp_filter_primitive_class_init(SPFilterPrimitiveClass *klass)
 static void
 sp_filter_primitive_init(SPFilterPrimitive *filter_primitive)
 {
-    filter_primitive->image_in = NR::NR_FILTER_SLOT_NOT_SET;
-    filter_primitive->image_out = NR::NR_FILTER_SLOT_NOT_SET;
+    filter_primitive->image_in = Inkscape::Filters::NR_FILTER_SLOT_NOT_SET;
+    filter_primitive->image_out = Inkscape::Filters::NR_FILTER_SLOT_NOT_SET;
 }
 
 /**
@@ -129,7 +129,7 @@ sp_filter_primitive_set(SPObject *object, unsigned int key, gchar const *value)
             if (value) {
                 image_nr = sp_filter_primitive_read_in(filter_primitive, value);
             } else {
-                image_nr = NR::NR_FILTER_SLOT_NOT_SET;
+                image_nr = Inkscape::Filters::NR_FILTER_SLOT_NOT_SET;
             }
             if (image_nr != filter_primitive->image_in) {
                 filter_primitive->image_in = image_nr;
@@ -140,7 +140,7 @@ sp_filter_primitive_set(SPObject *object, unsigned int key, gchar const *value)
             if (value) {
                 image_nr = sp_filter_primitive_read_result(filter_primitive, value);
             } else {
-                image_nr = NR::NR_FILTER_SLOT_NOT_SET;
+                image_nr = Inkscape::Filters::NR_FILTER_SLOT_NOT_SET;
             }
             if (image_nr != filter_primitive->image_out) {
                 filter_primitive->image_out = image_nr;
@@ -177,23 +177,23 @@ sp_filter_primitive_update(SPObject *object, SPCtx *ctx, guint flags)
  * Writes its settings to an incoming repr object, if any.
  */
 static Inkscape::XML::Node *
-sp_filter_primitive_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
+sp_filter_primitive_write(SPObject *object, Inkscape::XML::Document *doc, Inkscape::XML::Node *repr, guint flags)
 {
-    //SPFilterPrimitive *filterPrimitive = SP_FILTER_PRIMITIVE(object);
+    SPFilterPrimitive *prim = SP_FILTER_PRIMITIVE(object);
+    SPFilter *parent = SP_FILTER(object->parent);
 
-    // Inkscape-only object, not copied during an "plain SVG" dump:
-    if (flags & SP_OBJECT_WRITE_EXT) {
-        if (repr) {
-            // is this sane?
-            //repr->mergeFrom(SP_OBJECT_REPR(object), "id");
-        } else {
-             /// \todo FIXME:  Plumb an appropriate XML::Document into this
-             repr = SP_OBJECT_REPR(object)->duplicate(NULL);
-        }
+    if (!repr) {
+        repr = SP_OBJECT_REPR(object)->duplicate(doc);
     }
 
+    gchar const *in_name = sp_filter_name_for_image(parent, prim->image_in);
+    repr->setAttribute("in", in_name);
+
+    gchar const *out_name = sp_filter_name_for_image(parent, prim->image_out);
+    repr->setAttribute("result", out_name);
+
     if (((SPObjectClass *) filter_primitive_parent_class)->write) {
-        ((SPObjectClass *) filter_primitive_parent_class)->write(object, repr, flags);
+        ((SPObjectClass *) filter_primitive_parent_class)->write(object, doc, repr, flags);
     }
 
     return repr;
@@ -201,26 +201,26 @@ sp_filter_primitive_write(SPObject *object, Inkscape::XML::Node *repr, guint fla
 
 int sp_filter_primitive_read_in(SPFilterPrimitive *prim, gchar const *name)
 {
-    if (!name) return NR::NR_FILTER_SLOT_NOT_SET;
+    if (!name) return Inkscape::Filters::NR_FILTER_SLOT_NOT_SET;
     // TODO: are these case sensitive or not? (assumed yes)
     switch (name[0]) {
         case 'S':
             if (strcmp(name, "SourceGraphic") == 0)
-                return NR::NR_FILTER_SOURCEGRAPHIC;
+                return Inkscape::Filters::NR_FILTER_SOURCEGRAPHIC;
             if (strcmp(name, "SourceAlpha") == 0)
-                return NR::NR_FILTER_SOURCEALPHA;
+                return Inkscape::Filters::NR_FILTER_SOURCEALPHA;
             if (strcmp(name, "StrokePaint") == 0)
-                return NR::NR_FILTER_STROKEPAINT;
+                return Inkscape::Filters::NR_FILTER_STROKEPAINT;
             break;
         case 'B':
             if (strcmp(name, "BackgroundImage") == 0)
-                return NR::NR_FILTER_BACKGROUNDIMAGE;
+                return Inkscape::Filters::NR_FILTER_BACKGROUNDIMAGE;
             if (strcmp(name, "BackgroundAlpha") == 0)
-                return NR::NR_FILTER_BACKGROUNDALPHA;
+                return Inkscape::Filters::NR_FILTER_BACKGROUNDALPHA;
             break;
         case 'F':
             if (strcmp(name, "FillPaint") == 0)
-                return NR::NR_FILTER_FILLPAINT;
+                return Inkscape::Filters::NR_FILTER_FILLPAINT;
             break;
     }
 
@@ -228,7 +228,7 @@ int sp_filter_primitive_read_in(SPFilterPrimitive *prim, gchar const *name)
     int ret = sp_filter_get_image_name(parent, name);
     if (ret >= 0) return ret;
 
-    return NR::NR_FILTER_SLOT_NOT_SET;
+    return Inkscape::Filters::NR_FILTER_SLOT_NOT_SET;
 }
 
 int sp_filter_primitive_read_result(SPFilterPrimitive *prim, gchar const *name)
@@ -240,11 +240,36 @@ int sp_filter_primitive_read_result(SPFilterPrimitive *prim, gchar const *name)
     ret = sp_filter_set_image_name(parent, name);
     if (ret >= 0) return ret;
 
-    return NR::NR_FILTER_SLOT_NOT_SET;
+    return Inkscape::Filters::NR_FILTER_SLOT_NOT_SET;
+}
+
+/**
+ * Gives name for output of previous filter. Makes things clearer when prim
+ * is a filter with two or more inputs. Returns the slot number of result
+ * of previous primitive, or NR_FILTER_SOURCEGRAPHIC if this is the first
+ * primitive.
+ */
+int sp_filter_primitive_name_previous_out(SPFilterPrimitive *prim) {
+    SPFilter *parent = SP_FILTER(prim->parent);
+    SPObject *i = parent->children;
+    while (i && i->next != prim) i = i->next;
+    if (i) {
+        SPFilterPrimitive *i_prim = SP_FILTER_PRIMITIVE(i);
+        if (i_prim->image_out < 0) {
+            Glib::ustring name = sp_filter_get_new_result_name(parent);
+            int slot = sp_filter_set_image_name(parent, name.c_str());
+            i_prim->image_out = slot;
+            i_prim->repr->setAttribute("result", name.c_str());
+            return slot;
+        } else {
+            return i_prim->image_out;
+        }
+    }
+    return Inkscape::Filters::NR_FILTER_SOURCEGRAPHIC;
 }
 
 /* Common initialization for filter primitives */
-void sp_filter_primitive_renderer_common(SPFilterPrimitive *sp_prim, NR::FilterPrimitive *nr_prim)
+void sp_filter_primitive_renderer_common(SPFilterPrimitive *sp_prim, Inkscape::Filters::FilterPrimitive *nr_prim)
 {
     g_assert(sp_prim != NULL);
     g_assert(nr_prim != NULL);
