@@ -20,7 +20,7 @@
 #include <string>
 #include <libnr/nr-matrix-ops.h>
 #include "libnr/nr-matrix-fns.h"
-#include <libnr/nr-translate-matrix-ops.h>
+#include <2geom/transforms.h>
 #include "macros.h"
 #include "svg/svg.h"
 #include "display/nr-arena.h"
@@ -44,19 +44,21 @@ struct SPPatPainter {
 	SPPainter painter;
 	SPPattern *pat;
 
-	NRMatrix ps2px;
-	NRMatrix px2ps;
-	NRMatrix pcs2px;
+	Geom::Matrix ps2px;
+	Geom::Matrix px2ps;
+	Geom::Matrix pcs2px;
 
 	NRArena *arena;
 	unsigned int dkey;
 	NRArenaItem *root;
 	
 	bool         use_cached_tile;
-	NRMatrix     ca2pa;
-	NRMatrix     pa2ca;
+	Geom::Matrix     ca2pa;
+	Geom::Matrix     pa2ca;
 	NRRectL      cached_bbox;
 	NRPixBlock   cached_tile;
+
+  std::map<SPObject *, sigc::connection> *_release_connections;
 };
 
 static void sp_pattern_class_init (SPPatternClass *klass);
@@ -72,7 +74,7 @@ static void sp_pattern_modified (SPObject *object, unsigned int flags);
 static void pattern_ref_changed(SPObject *old_ref, SPObject *ref, SPPattern *pat);
 static void pattern_ref_modified (SPObject *ref, guint flags, SPPattern *pattern);
 
-static SPPainter *sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR::Matrix const &parent_transform, const NRRect *bbox);
+static SPPainter *sp_pattern_painter_new (SPPaintServer *ps, Geom::Matrix const &full_transform, Geom::Matrix const &parent_transform, const NRRect *bbox);
 static void sp_pattern_painter_free (SPPaintServer *ps, SPPainter *painter);
 
 static SPPaintServerClass * pattern_parent_class;
@@ -226,7 +228,7 @@ sp_pattern_set (SPObject *object, unsigned int key, const gchar *value)
 		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_PATTERNTRANSFORM: {
-		NR::Matrix t;
+		Geom::Matrix t;
 		if (value && sp_svg_transform_read (value, &t)) {
 			pat->patternTransform = t;
 			pat->patternTransform_set = TRUE;
@@ -430,6 +432,7 @@ pattern_ref_modified (SPObject */*ref*/, guint /*flags*/, SPPattern *pattern)
 {
 	if (SP_IS_OBJECT (pattern))
 		SP_OBJECT (pattern)->requestModified(SP_OBJECT_MODIFIED_FLAG);
+        /* Conditional to avoid causing infinite loop if there's a cycle in the href chain. */
 }
 
 guint
@@ -474,7 +477,7 @@ sp_pattern_clone_if_necessary (SPItem *item, SPPattern *pattern, const gchar *pr
 }
 
 void
-sp_pattern_transform_multiply (SPPattern *pattern, NR::Matrix postmul, bool set)
+sp_pattern_transform_multiply (SPPattern *pattern, Geom::Matrix postmul, bool set)
 {
 	// this formula is for a different interpretation of pattern transforms as described in (*) in sp-pattern.cpp
 	// for it to work, we also need    sp_object_read_attr (SP_OBJECT (item), "transform");
@@ -494,15 +497,15 @@ sp_pattern_transform_multiply (SPPattern *pattern, NR::Matrix postmul, bool set)
 }
 
 const gchar *
-pattern_tile (GSList *reprs, NR::Rect bounds, SPDocument *document, NR::Matrix transform, NR::Matrix move)
+pattern_tile (GSList *reprs, Geom::Rect bounds, SPDocument *document, Geom::Matrix transform, Geom::Matrix move)
 {
 	Inkscape::XML::Document *xml_doc = sp_document_repr_doc(document);
 	Inkscape::XML::Node *defsrepr = SP_OBJECT_REPR (SP_DOCUMENT_DEFS (document));
 
 	Inkscape::XML::Node *repr = xml_doc->createElement("svg:pattern");
 	repr->setAttribute("patternUnits", "userSpaceOnUse");
-	sp_repr_set_svg_double(repr, "width", bounds.extent(NR::X));
-	sp_repr_set_svg_double(repr, "height", bounds.extent(NR::Y));
+	sp_repr_set_svg_double(repr, "width", bounds.dimensions()[Geom::X]);
+	sp_repr_set_svg_double(repr, "height", bounds.dimensions()[Geom::Y]);
 
 	gchar *t=sp_svg_transform_write(transform);
 	repr->setAttribute("patternTransform", t);
@@ -516,12 +519,12 @@ pattern_tile (GSList *reprs, NR::Rect bounds, SPDocument *document, NR::Matrix t
 	        Inkscape::XML::Node *node = (Inkscape::XML::Node *)(i->data);
 		SPItem *copy = SP_ITEM(pat_object->appendChildRepr(node));
 
-		NR::Matrix dup_transform;
+		Geom::Matrix dup_transform;
 		if (!sp_svg_transform_read (node->attribute("transform"), &dup_transform))
-			dup_transform = NR::identity();
+			dup_transform = Geom::identity();
 		dup_transform *= move;
 
-		sp_item_write_transform(copy, SP_OBJECT_REPR(copy), dup_transform);
+		sp_item_write_transform(copy, SP_OBJECT_REPR(copy), dup_transform, NULL, false);
 	}
 
 	Inkscape::GC::release(repr);
@@ -562,7 +565,7 @@ guint pattern_patternContentUnits (SPPattern *pat)
 	return pat->patternContentUnits;
 }
 
-NR::Matrix const &pattern_patternTransform(SPPattern const *pat)
+Geom::Matrix const &pattern_patternTransform(SPPattern const *pat)
 {
 	for (SPPattern const *pat_i = pat; pat_i != NULL; pat_i = pat_i->ref ? pat_i->ref->getObject() : NULL) {
 		if (pat_i->patternTransform_set)
@@ -632,12 +635,25 @@ bool pattern_hasItemChildren (SPPattern *pat)
 
 static void sp_pat_fill (SPPainter *painter, NRPixBlock *pb);
 
+// item in this pattern is about to be deleted, hide it on our arena and disconnect
+void
+sp_pattern_painter_release (SPObject *obj, SPPatPainter *painter)
+{
+	std::map<SPObject *, sigc::connection>::iterator iter = painter->_release_connections->find(obj);
+	if (iter != painter->_release_connections->end()) {
+		iter->second.disconnect();
+    painter->_release_connections->erase(obj);
+	}
+
+	sp_item_invoke_hide(SP_ITEM(obj), painter->dkey);
+}
+
 /**
 Creates a painter (i.e. the thing that does actual filling at the given zoom).
 See (*) below for why the parent_transform may be necessary.
 */
 static SPPainter *
-sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR::Matrix const &/*parent_transform*/, const NRRect *bbox)
+sp_pattern_painter_new (SPPaintServer *ps, Geom::Matrix const &full_transform, Geom::Matrix const &/*parent_transform*/, const NRRect *bbox)
 {
 	SPPattern *pat = SP_PATTERN (ps);
 	SPPatPainter *pp = g_new (SPPatPainter, 1);
@@ -649,16 +665,13 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 
 	if (pattern_patternUnits (pat) == SP_PATTERN_UNITS_OBJECTBOUNDINGBOX) {
 		/* BBox to user coordinate system */
-		NR::Matrix bbox2user (bbox->x1 - bbox->x0, 0.0, 0.0, bbox->y1 - bbox->y0, bbox->x0, bbox->y0);
+		Geom::Matrix bbox2user (bbox->x1 - bbox->x0, 0.0, 0.0, bbox->y1 - bbox->y0, bbox->x0, bbox->y0);
 
 		// the final patternTransform, taking into account bbox
-		NR::Matrix const ps2user(pattern_patternTransform(pat) * bbox2user);
+		Geom::Matrix const ps2user(pattern_patternTransform(pat) * bbox2user);
 
 		// see (*) comment below
-		NR::Matrix ps2px = ps2user * full_transform;
-
-		ps2px.copyto (&pp->ps2px);
-
+		pp->ps2px = ps2user * full_transform;
 	} else {
 		/* Problem: What to do, if we have mixed lengths and percentages? */
 		/* Currently we do ignore percentages at all, but that is not good (lauris) */
@@ -675,48 +688,40 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 		// So here I comply with the majority opinion, but leave my interpretation commented out below.
 		// (To get item_transform, I subtract parent from full.)
 
-		//NR::Matrix ps2px = (full_transform / parent_transform) * pattern_patternTransform(pat) * parent_transform;
-		NR::Matrix ps2px = pattern_patternTransform(pat) * full_transform;
-
-		ps2px.copyto (&pp->ps2px);
+		//pp->ps2px = (full_transform / parent_transform) * pattern_patternTransform(pat) * parent_transform;
+		pp->ps2px = pattern_patternTransform(pat) * full_transform;
 	}
 
-	nr_matrix_invert (&pp->px2ps, &pp->ps2px);
+	pp->px2ps = pp->ps2px.inverse();
 
 	if (pat->viewBox_set) {
-		gdouble tmp_x = (pattern_viewBox(pat)->x1 - pattern_viewBox(pat)->x0) / pattern_width (pat);
-		gdouble tmp_y = (pattern_viewBox(pat)->y1 - pattern_viewBox(pat)->y0) / pattern_height (pat);
+		gdouble tmp_x = pattern_width (pat) / (pattern_viewBox(pat)->x1 - pattern_viewBox(pat)->x0);
+		gdouble tmp_y = pattern_height (pat) / (pattern_viewBox(pat)->y1 - pattern_viewBox(pat)->y0);
 
 		// FIXME: preserveAspectRatio must be taken into account here too!
-		NR::Matrix vb2ps (tmp_x, 0.0, 0.0, tmp_y, pattern_x(pat) - pattern_viewBox(pat)->x0, pattern_y(pat) - pattern_viewBox(pat)->y0);
+		Geom::Matrix vb2ps (tmp_x, 0.0, 0.0, tmp_y, pattern_x(pat) - pattern_viewBox(pat)->x0 * tmp_x, pattern_y(pat) - pattern_viewBox(pat)->y0 * tmp_y);
 
-		NR::Matrix vb2us = vb2ps * pattern_patternTransform(pat);
+		Geom::Matrix vb2us = vb2ps * pattern_patternTransform(pat);
 
 		// see (*)
-		NR::Matrix pcs2px = vb2us * full_transform;
-
-		pcs2px.copyto (&pp->pcs2px);
+		pp->pcs2px = vb2us * full_transform;
 	} else {
-		NR::Matrix pcs2px;
-
 		/* No viewbox, have to parse units */
 		if (pattern_patternContentUnits (pat) == SP_PATTERN_UNITS_OBJECTBOUNDINGBOX) {
 			/* BBox to user coordinate system */
-			NR::Matrix bbox2user (bbox->x1 - bbox->x0, 0.0, 0.0, bbox->y1 - bbox->y0, bbox->x0, bbox->y0);
+			Geom::Matrix bbox2user (bbox->x1 - bbox->x0, 0.0, 0.0, bbox->y1 - bbox->y0, bbox->x0, bbox->y0);
 
-			NR::Matrix pcs2user = pattern_patternTransform(pat) * bbox2user;
+			Geom::Matrix pcs2user = pattern_patternTransform(pat) * bbox2user;
 
 			// see (*)
-			pcs2px = pcs2user * full_transform;
+			pp->pcs2px = pcs2user * full_transform;
 		} else {
 			// see (*)
 			//pcs2px = (full_transform / parent_transform) * pattern_patternTransform(pat) * parent_transform;
-			pcs2px = pattern_patternTransform(pat) * full_transform;
+			pp->pcs2px = pattern_patternTransform(pat) * full_transform;
 		}
 
-		pcs2px = NR::translate (pattern_x (pat), pattern_y (pat)) * pcs2px;
-
-		pcs2px.copyto (&pp->pcs2px);
+		pp->pcs2px = Geom::Translate (pattern_x (pat), pattern_y (pat)) * pp->pcs2px;
 	}
 
 	/* Create arena */
@@ -728,14 +733,19 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 	pp->root = NRArenaGroup::create(pp->arena);
 
 	/* Show items */
+	pp->_release_connections = new std::map<SPObject *, sigc::connection>;
 	for (SPPattern *pat_i = pat; pat_i != NULL; pat_i = pat_i->ref ? pat_i->ref->getObject() : NULL) {
 		if (pat_i && SP_IS_OBJECT (pat_i) && pattern_hasItemChildren(pat_i)) { // find the first one with item children
 			for (SPObject *child = sp_object_first_child(SP_OBJECT(pat_i)) ; child != NULL; child = SP_OBJECT_NEXT(child) ) {
 				if (SP_IS_ITEM (child)) {
+					// for each item in pattern,
 					NRArenaItem *cai;
+					// show it on our arena,
 					cai = sp_item_invoke_show (SP_ITEM (child), pp->arena, pp->dkey, SP_ITEM_REFERENCE_FLAGS);
+					// add to the group,
 					nr_arena_item_append_child (pp->root, cai);
-					nr_arena_item_unref (cai);
+					// and connect to the release signal in case the item gets deleted
+					pp->_release_connections->insert(std::make_pair(child, child->connectRelease(sigc::bind<1>(sigc::ptr_fun(&sp_pattern_painter_release), pp))));
 				}
 			}
 			break; // do not go further up the chain if children are found
@@ -748,7 +758,9 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 		one_tile.y0=pattern_y(pp->pat);
 		one_tile.x1=one_tile.x0+pattern_width (pp->pat);
 		one_tile.y1=one_tile.y0+pattern_height (pp->pat);
-		nr_rect_d_matrix_transform (&tr_tile, &one_tile, &pp->ps2px);
+		// TODO: remove ps2px_nr after converting to 2geom
+		NR::Matrix ps2px_nr = from_2geom(pp->ps2px);
+		nr_rect_d_matrix_transform (&tr_tile, &one_tile, &ps2px_nr);
 		int       tr_width=(int)ceil(1.3*(tr_tile.x1-tr_tile.x0));
 		int       tr_height=(int)ceil(1.3*(tr_tile.y1-tr_tile.y0));
 //		if ( tr_width < 10000 && tr_height < 10000 && tr_width*tr_height < 1000000 ) {
@@ -764,18 +776,18 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 				nr_pixblock_setup (&pp->cached_tile,NR_PIXBLOCK_MODE_R8G8B8A8N, pp->cached_bbox.x0, pp->cached_bbox.y0, pp->cached_bbox.x1, pp->cached_bbox.y1,TRUE);
 			}
 
-			pp->pa2ca.c[0]=((double)tr_width)/(one_tile.x1-one_tile.x0);
-			pp->pa2ca.c[1]=0;
-			pp->pa2ca.c[2]=0;
-			pp->pa2ca.c[3]=((double)tr_height)/(one_tile.y1-one_tile.y0);
-			pp->pa2ca.c[4]=-one_tile.x0*pp->pa2ca.c[0];
-			pp->pa2ca.c[5]=-one_tile.y0*pp->pa2ca.c[1];
-			pp->ca2pa.c[0]=(one_tile.x1-one_tile.x0)/((double)tr_width);
-			pp->ca2pa.c[1]=0;
-			pp->ca2pa.c[2]=0;
-			pp->ca2pa.c[3]=(one_tile.y1-one_tile.y0)/((double)tr_height);
-			pp->ca2pa.c[4]=one_tile.x0;
-			pp->ca2pa.c[5]=one_tile.y0;
+			pp->pa2ca[0]=((double)tr_width)/(one_tile.x1-one_tile.x0);
+			pp->pa2ca[1]=0;
+			pp->pa2ca[2]=0;
+			pp->pa2ca[3]=((double)tr_height)/(one_tile.y1-one_tile.y0);
+			pp->pa2ca[4]=-one_tile.x0*pp->pa2ca[0];
+			pp->pa2ca[5]=-one_tile.y0*pp->pa2ca[1];
+			pp->ca2pa[0]=(one_tile.x1-one_tile.x0)/((double)tr_width);
+			pp->ca2pa[1]=0;
+			pp->ca2pa[2]=0;
+			pp->ca2pa[3]=(one_tile.y1-one_tile.y0)/((double)tr_height);
+			pp->ca2pa[4]=one_tile.x0;
+			pp->ca2pa[5]=one_tile.y0;
 //		} else {
 //			pp->use_cached_tile=false;
 //		}
@@ -797,22 +809,25 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 	return (SPPainter *) pp;
 }
 
+
 static void
 sp_pattern_painter_free (SPPaintServer */*ps*/, SPPainter *painter)
 {
 	SPPatPainter *pp = (SPPatPainter *) painter;
-	SPPattern *pat = pp->pat;
+	// free our arena
+  if (pp->arena) {
+      ((NRObject *) pp->arena)->unreference();
+      pp->arena = NULL;
+  }
 
-	for (SPPattern *pat_i = pat; pat_i != NULL; pat_i = pat_i->ref ? pat_i->ref->getObject() : NULL) {
-		if (pat_i && SP_IS_OBJECT (pat_i) && pattern_hasItemChildren(pat_i)) { // find the first one with item children
-			for (SPObject *child = sp_object_first_child(SP_OBJECT(pat_i)) ; child != NULL; child = SP_OBJECT_NEXT(child) ) {
-				if (SP_IS_ITEM (child)) {
-						sp_item_invoke_hide (SP_ITEM (child), pp->dkey);
-				}
-			}
-			break; // do not go further up the chain if children are found
-		}
+	// disconnect all connections
+  std::map<SPObject *, sigc::connection>::iterator iter;
+  for (iter = pp->_release_connections->begin() ; iter!=pp->_release_connections->end() ; iter++) {
+	  iter->second.disconnect();
 	}
+	pp->_release_connections->clear();
+  delete pp->_release_connections;
+
 	if ( pp->use_cached_tile ) nr_pixblock_release(&pp->cached_tile);
 	g_free (pp);
 }
@@ -890,15 +905,15 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
 				unsigned char* cpx=lpx;
 				double         px_x = pb->area.x0;
 				
-				double ps_x=pp->px2ps.c[0]*px_x+pp->px2ps.c[2]*px_y+pp->px2ps.c[4];
-				double ps_y=pp->px2ps.c[1]*px_x+pp->px2ps.c[3]*px_y+pp->px2ps.c[5];
+				double ps_x=pp->px2ps[0]*px_x+pp->px2ps[2]*px_y+pp->px2ps[4];
+				double ps_y=pp->px2ps[1]*px_x+pp->px2ps[3]*px_y+pp->px2ps[5];
 				for (int i=pb->area.x0;i<pb->area.x1;i++) {
 					while ( ps_x > pat_w ) ps_x-=pat_w;
 					while ( ps_x < 0 ) ps_x+=pat_w;
 					while ( ps_y > pat_h ) ps_y-=pat_h;
 					while ( ps_y < 0 ) ps_y+=pat_h;
-					double ca_x=pp->pa2ca.c[0]*ps_x+pp->pa2ca.c[2]*ps_y+pp->pa2ca.c[4];
-					double ca_y=pp->pa2ca.c[1]*ps_x+pp->pa2ca.c[3]*ps_y+pp->pa2ca.c[5];
+					double ca_x=pp->pa2ca[0]*ps_x+pp->pa2ca[2]*ps_y+pp->pa2ca[4];
+					double ca_y=pp->pa2ca[1]*ps_x+pp->pa2ca[3]*ps_y+pp->pa2ca[5];
 					unsigned char n_a,n_r,n_g,n_b;
 					get_cached_tile_pixel(pp,ca_x,ca_y,n_r,n_g,n_b,n_a);
 					cpx[0]=n_r;
@@ -907,8 +922,8 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
 					cpx[3]=n_a;
 					
 					px_x+=1.0;
-					ps_x+=pp->px2ps.c[0];
-					ps_y+=pp->px2ps.c[1];
+					ps_x+=pp->px2ps[0];
+					ps_y+=pp->px2ps[1];
 					cpx+=4;
 				}
 				px_y+=1.0;
@@ -921,15 +936,15 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
 				unsigned char* cpx=lpx;
 				double         px_x = pb->area.x0;
 				
-				double ps_x=pp->px2ps.c[0]*px_x+pp->px2ps.c[2]*px_y+pp->px2ps.c[4];
-				double ps_y=pp->px2ps.c[1]*px_x+pp->px2ps.c[3]*px_y+pp->px2ps.c[5];
+				double ps_x=pp->px2ps[0]*px_x+pp->px2ps[2]*px_y+pp->px2ps[4];
+				double ps_y=pp->px2ps[1]*px_x+pp->px2ps[3]*px_y+pp->px2ps[5];
 				for (int i=pb->area.x0;i<pb->area.x1;i++) {
 					while ( ps_x > pat_w ) ps_x-=pat_w;
 					while ( ps_x < 0 ) ps_x+=pat_w;
 					while ( ps_y > pat_h ) ps_y-=pat_h;
 					while ( ps_y < 0 ) ps_y+=pat_h;
-					double ca_x=pp->pa2ca.c[0]*ps_x+pp->pa2ca.c[2]*ps_y+pp->pa2ca.c[4];
-					double ca_y=pp->pa2ca.c[1]*ps_x+pp->pa2ca.c[3]*ps_y+pp->pa2ca.c[5];
+					double ca_x=pp->pa2ca[0]*ps_x+pp->pa2ca[2]*ps_y+pp->pa2ca[4];
+					double ca_y=pp->pa2ca[1]*ps_x+pp->pa2ca[3]*ps_y+pp->pa2ca[5];
 					unsigned char n_a,n_r,n_g,n_b;
 					get_cached_tile_pixel(pp,ca_x,ca_y,n_r,n_g,n_b,n_a);
 					cpx[0]=n_r;
@@ -937,8 +952,8 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
 					cpx[2]=n_b;
 					
 					px_x+=1.0;
-					ps_x+=pp->px2ps.c[0];
-					ps_y+=pp->px2ps.c[1];
+					ps_x+=pp->px2ps[0];
+					ps_y+=pp->px2ps[1];
 					cpx+=4;
 				}
 				px_y+=1.0;
@@ -955,12 +970,14 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
         // Bail out if the transformation matrix has extreme values. If we bail out
         // however, then something (which was meaningless anyway) won't be rendered, 
         // which is better than getting stuck in a virtually infinite loop
-        if (fabs(pp->px2ps.c[0]) < 1e6 && 
-            fabs(pp->px2ps.c[3]) < 1e6 &&
-            fabs(pp->px2ps.c[4]) < 1e6 &&
-            fabs(pp->px2ps.c[5]) < 1e6) 
+        if (fabs(pp->px2ps[0]) < 1e6 && 
+            fabs(pp->px2ps[3]) < 1e6 &&
+            fabs(pp->px2ps[4]) < 1e6 &&
+            fabs(pp->px2ps[5]) < 1e6) 
         {
-            nr_rect_d_matrix_transform (&psa, &ba, &pp->px2ps);
+	    // TODO: remove px2ps_nr after converting to 2geom
+	    NR::Matrix px2ps_nr = from_2geom(pp->px2ps);
+            nr_rect_d_matrix_transform (&psa, &ba, &px2ps_nr);
     		
     		psa.x0 = floor ((psa.x0 - pattern_x (pp->pat)) / pattern_width (pp->pat)) -1;
     		psa.y0 = floor ((psa.y0 - pattern_y (pp->pat)) / pattern_height (pp->pat)) -1;
@@ -980,8 +997,8 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
     				psx = x * pattern_width (pp->pat);
     				psy = y * pattern_height (pp->pat);
     				
-    				area.x0 = (gint32)(pb->area.x0 - (pp->ps2px.c[0] * psx + pp->ps2px.c[2] * psy));
-    				area.y0 = (gint32)(pb->area.y0 - (pp->ps2px.c[1] * psx + pp->ps2px.c[3] * psy));
+    				area.x0 = (gint32)(pb->area.x0 - (pp->ps2px[0] * psx + pp->ps2px[2] * psy));
+    				area.y0 = (gint32)(pb->area.y0 - (pp->ps2px[1] * psx + pp->ps2px[3] * psy));
     				area.x1 = area.x0 + pb->area.x1 - pb->area.x0;
     				area.y1 = area.y0 + pb->area.y1 - pb->area.y0;
     				
@@ -996,6 +1013,6 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
     				nr_pixblock_release (&ppb);
     			}
     		}
-        } 
+     } 
 	}
 }
