@@ -15,8 +15,9 @@
  */
 
 #include "display/display-forward.h"
-#include "sp-item.h"
+#include "sp-lpe-item.h"
 #include "sp-marker-loc.h"
+#include <2geom/forward.h>
 
 #include <sigc++/connection.h>
 
@@ -29,34 +30,20 @@
 #define SP_SHAPE_WRITE_PATH (1 << 2)
 
 struct SPDesktop;
-struct LivePathEffectObject;
-namespace Inkscape{ 
-namespace LivePathEffect{
-    class LPEObjectReference;
-    class Effect;
-};
-};
 
-
-struct SPShape : public SPItem {
+struct SPShape : public SPLPEItem {
     SPCurve *curve;
 
       SPObject *marker[SP_MARKER_LOC_QTY];
       sigc::connection release_connect [SP_MARKER_LOC_QTY];
       sigc::connection modified_connect [SP_MARKER_LOC_QTY];
-
-    gchar *path_effect_href;
-    Inkscape::LivePathEffect::LPEObjectReference *path_effect_ref;
-    sigc::connection lpe_modified_connection;
 };
 
 struct SPShapeClass {
-	SPItemClass item_class;
+	SPLPEItemClass item_class;
 
 	/* Build bpath from extra shape attributes */
 	void (* set_shape) (SPShape *shape);
-
-    void (* update_patheffect) (SPShape *shape, bool write);
 };
 
 GType sp_shape_get_type (void);
@@ -76,19 +63,9 @@ void sp_shape_set_curve_insync (SPShape *shape, SPCurve *curve, unsigned int own
 void sp_shape_set_marker (SPObject *object, unsigned int key, const gchar *value);
 int sp_shape_has_markers (SPShape const *shape);
 int sp_shape_number_of_markers (SPShape* Shape, int type);
-NR::Matrix sp_shape_marker_get_transform(SPShape const *shape, NArtBpath const *bp);
-bool sp_shape_marker_required(SPShape const *shape, int const m, NArtBpath *bp);
 
-LivePathEffectObject * sp_shape_get_livepatheffectobject(SPShape *shape);
-Inkscape::LivePathEffect::Effect * sp_shape_get_livepatheffect(SPShape *shape);
-void sp_shape_update_patheffect (SPShape *shape, bool write);
-void sp_shape_perform_path_effect(SPCurve *curve, SPShape *shape);
-
-void sp_shape_set_path_effect(SPShape *shape, gchar *value);
-void sp_shape_set_path_effect(SPShape *shape, LivePathEffectObject * new_lpeobj);
-void sp_shape_remove_path_effect(SPShape *shape);
-bool sp_shape_has_path_effect(SPShape *shape);
-
-void sp_shape_edit_next_param_oncanvas(SPShape *shape, SPDesktop *dt);
+Geom::Matrix sp_shape_marker_get_transform(Geom::Curve const & c1, Geom::Curve const & c2);
+Geom::Matrix sp_shape_marker_get_transform_at_start(Geom::Curve const & c);
+Geom::Matrix sp_shape_marker_get_transform_at_end(Geom::Curve const & c);
 
 #endif
