@@ -1,8 +1,8 @@
 #ifndef __SP_SPIRAL_H__
 #define __SP_SPIRAL_H__
 
-/*
- * <sodipodi:spiral> implementation
+/** \file
+ * SPSpiral: <sodipodi:spiral> implementation
  *
  * Authors:
  *   Mitsuru Oka <oka326@parkcity.ne.jp>
@@ -23,8 +23,8 @@
 #define SP_HUGE          1e5
 
 #define SPIRAL_TOLERANCE 3.0
-#define SAMPLE_STEP      (1.0/4.0) /* step per 2PI */
-#define SAMPLE_SIZE      8      /* sample size per one bezier */
+#define SAMPLE_STEP      (1.0/4.0) ///< step per 2PI 
+#define SAMPLE_SIZE      8         ///< sample size per one bezier 
 
 #define SP_TYPE_SPIRAL            (sp_spiral_get_type ())
 #define SP_SPIRAL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_SPIRAL, SPSpiral))
@@ -35,23 +35,28 @@
 class SPSpiral;
 class SPSpiralClass;
 
+/**
+ * A spiral Shape.
+ *
+ * The Spiral shape is defined as:
+ * \verbatim
+   x(t) = rad * t^exp cos(2 * Pi * revo*t + arg) + cx
+   y(t) = rad * t^exp sin(2 * Pi * revo*t + arg) + cy    \endverbatim
+ * where spiral curve is drawn for {t | t0 <= t <= 1}. The  rad and arg 
+ * parameters can also be represented by transformation. 
+ *
+ * \todo Should I remove these attributes?
+ */
 struct SPSpiral : public SPShape {
-	/*
-	 * Spiral shape is defined as:
-	 * x(t) = rad * t^exp cos(2 * Pi * revo*t + arg) + cx
-	 * y(t) = rad * t^exp sin(2 * Pi * revo*t + arg) + cy
-	 * where spiral curve is drawn for {t | t0 <= t <= 1}.
-	 * rad and arg parameters can also be represented by
-	 * transformation. shoud I remove these attributes?
-	 */
 	float cx, cy;
-	float exp; /* Spiral expansion factor */
-	float revo; /* Spiral revolution factor */
-	float rad; /* Spiral radius */
-	float arg; /* Spiral argument */
+	float exp;  ///< Spiral expansion factor
+	float revo; ///< Spiral revolution factor
+	float rad;  ///< Spiral radius
+	float arg;  ///< Spiral argument
 	float t0;
 };
 
+/// The SPSpiral vtable.
 struct SPSpiralClass {
 	SPShapeClass parent_class;
 };
@@ -70,7 +75,7 @@ void    sp_spiral_position_set		(SPSpiral      *spiral,
 				 gdouble	arg,
 				 gdouble	t0);
 
-NR::Point    sp_spiral_get_xy	(SPSpiral const *spiral,
+Geom::Point    sp_spiral_get_xy	(SPSpiral const *spiral,
 				 gdouble	t);
 
 void    sp_spiral_get_polar	(SPSpiral const *spiral,
@@ -78,7 +83,7 @@ void    sp_spiral_get_polar	(SPSpiral const *spiral,
 				 gdouble       *rad,
 				 gdouble       *arg);
 
-gboolean sp_spiral_is_invalid   (SPSpiral const *spiral);
+bool sp_spiral_is_invalid   (SPSpiral const *spiral);
 
 
 
