@@ -1,9 +1,10 @@
 #ifndef __SP_SPIRAL_CONTEXT_H__
 #define __SP_SPIRAL_CONTEXT_H__
 
-/*
+/** \file
  * Spiral drawing context
- *
+ */
+/*
  * Authors:
  *   Mitsuru Oka
  *   Lauris Kaplinski <lauris@kaplinski.com>
@@ -14,10 +15,10 @@
  * Released under GNU GPL
  */
 
+#include <gtk/gtktypeutils.h>
 #include <sigc++/sigc++.h>
 #include "event-context.h"
 #include "libnr/nr-point.h"
-struct SPKnotHolder;
 
 #define SP_TYPE_SPIRAL_CONTEXT            (sp_spiral_context_get_type ())
 #define SP_SPIRAL_CONTEXT(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_SPIRAL_CONTEXT, SPSpiralContext))
@@ -30,7 +31,7 @@ class SPSpiralContextClass;
 
 struct SPSpiralContext : public SPEventContext {
 	SPItem * item;
-	NR::Point center;
+	Geom::Point center;
 	gdouble revo;
 	gdouble exp;
 	gdouble t0;
