@@ -17,7 +17,6 @@
 #include <sigc++/sigc++.h>
 #include "event-context.h"
 #include "libnr/nr-point.h"
-struct SPKnotHolder;
 
 #define SP_TYPE_STAR_CONTEXT (sp_star_context_get_type ())
 #define SP_STAR_CONTEXT(obj) (GTK_CHECK_CAST ((obj), SP_TYPE_STAR_CONTEXT, SPStarContext))
@@ -30,7 +29,7 @@ class SPStarContextClass;
 
 struct SPStarContext : public SPEventContext {
 	SPItem *item;
-	NR::Point center;
+	Geom::Point center;
 
 	/* Number of corners */
 	gint magnitude;
