@@ -32,14 +32,15 @@ enum {
   TOOLS_ZOOM,
   TOOLS_DROPPER,
   TOOLS_CONNECTOR,
-  TOOLS_PAINTBUCKET 
+  TOOLS_PAINTBUCKET,
+  TOOLS_ERASER,
+  TOOLS_LPETOOL
 };
 
 int tools_isactive(SPDesktop *dt, unsigned num);
 int tools_active(SPDesktop *dt);
 void tools_switch(SPDesktop *dt, int num);
-void tools_switch_current(int num);
-void tools_switch_by_item (SPDesktop *dt, SPItem *item);
+void tools_switch_by_item (SPDesktop *dt, SPItem *item, Geom::Point const p);
 
 #endif /* !SEEN_TOOLS_SWITCH_H */
 
