@@ -2,7 +2,7 @@
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: /local/inkscape.rel.46/src/trace/potrace/greymap.cpp 16879 2007-10-26T21:01:30.934311Z ishmal  $ */
+/* $Id: greymap.cpp 18522 2008-05-04 12:26:23Z pjrm $ */
 
 /* Routines for manipulating greymaps, including reading pgm files. We
    only deal with greymaps of depth 8 bits. */
@@ -161,16 +161,16 @@ static int readbit(FILE *f) {
 
 /* ---------------------------------------------------------------------- */
 
-/* read a PNM stream: P1-P6 format (see pnm(5)), or a BMP stream, and
+char const *gm_read_error = NULL;
+
+/** Read a PNM stream: P1-P6 format (see pnm(5)), or a BMP stream, and
    convert the output to a greymap. Return greymap in *gmp. Return 0
    on success, -1 on error with errno set, -2 on bad file format (with
    error message in gm_read_error), and 1 on premature end of file, -3
    on empty file (including files with only whitespace and comments),
    -4 if wrong magic number. If the return value is >=0, *gmp is
-   valid. */
-
-char *gm_read_error = NULL;
-
+   valid.
+   */
 int gm_read(FILE *f, greymap_t **gmp) {
   int magic[2];
 
