@@ -1,7 +1,7 @@
-/**
- * \brief AboutBox - the Inkscape about dialog
- *
- * Authors:
+/** @file
+ * @brief Inkscape About box - implementation
+ */
+/* Authors:
  *   Derek P. Moore <derekm@hackunix.org>
  *   MenTaLguY <mental@rydia.net>
  *   Kees Cook <kees@outflux.net>
@@ -33,7 +33,7 @@
 #include "text-editing.h"
 #include "ui/dialog/aboutbox.h"
 
-#include "inkscape_version.h"
+#include "inkscape-version.h"
 
 
 
@@ -104,7 +104,7 @@ AboutBox::AboutBox() : Gtk::Dialog(_("About Inkscape")) {
     Gtk::Label *label=new Gtk::Label();
     gchar *label_text = 
         g_strdup_printf("<small><i>Inkscape %s, built %s</i></small>",
-              INKSCAPE_VERSION, __DATE__);
+              Inkscape::version_string, __DATE__);
     label->set_markup(label_text);
     label->set_alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
     g_free(label_text);
@@ -153,7 +153,7 @@ Gtk::Widget *build_splash_widget() {
 
     SPObject *version = doc->getObjectById("version");
     if ( version && SP_IS_TEXT(version) ) {
-        sp_te_set_repr_text_multiline (SP_TEXT (version), INKSCAPE_VERSION);
+        sp_te_set_repr_text_multiline (SP_TEXT (version), Inkscape::version_string);
     }
     sp_document_ensure_up_to_date(doc);
 
@@ -212,26 +212,39 @@ void AboutBox::initStrings() {
     //# A U T H O R S
     //##############################
     
-    /**
-     * This text is copied from the AUTHORS file.  It should be
-     * periodically updated.     
+    /* This text is copied from the AUTHORS file.
+     * To update it, execute this snippet of sed magic in the toplevel
+     * source directory:
+     *
+     * sed -e 's/^\(.*\) \([^ ]*\)*$/\2_ \1/' ../../../AUTHORS
+           | sort
+           | sed -e 's/^\([^_]*\)_ \(.*\)$/\2 \1/;s/^.*$/"\0\\n"/;$s/\\n//'
+           | zenity --text-info
+     *
+     * and paste the result from the combo box here.
      */ 
     authors_text =
         "Maximilian Albert\n"
         "Josh Andler\n"
-        "Gail Carmichael\n"
+        "Jean-François Barraud\n"
+        "Bill Baxter\n"
         "John Bintz\n"
         "Arpad Biro\n"
         "Nicholas Bishop\n"
+        "Joshua L. Blocher\n"
+        "Henrik Bohre\n"
         "Boldewyn\n"
         "Daniel Borgmann\n"
-        "Gustav Broberg\n"
+        "Bastien Bouclet\n"
         "Hans Breuer\n"
+        "Gustav Broberg\n"
+        "Christopher Brown\n"
         "Marcus Brubaker\n"
         "Luca Bruno\n"
         "Nicu Buculei\n"
         "Bulia Byak\n"
         "Ian Caldwell\n"
+        "Gail Carmichael\n"
         "Ed Catmur\n"
         "Chema Celorio\n"
         "Johan Ceuppens\n"
@@ -242,40 +255,58 @@ void AboutBox::initStrings() {
         "Ben Cromwell\n"
         "Robert Crosbie\n"
         "Jon Cruz\n"
-        "Miłosz Dereżyński\n"
+        "Aurélie De-Cooman\n"
+        "Milosz Derezynski\n"
         "Daniel Díaz\n"
+        "Bruno Dilly\n"
         "Larry Doolittle\n"
         "Tim Dwyer\n"
         "Maxim V. Dziumanenko\n"
         "Johan Engelen\n"
+        "Miklos Erdelyi\n"
+        "Ulf Erikson\n"
+        "Noé Falzon\n"
         "Frank Felfe\n"
         "Andrew Fitzsimon\n"
         "Edward Flick\n"
         "Marcin Floryan\n"
-        "Fred\n"
         "Ben Fowler\n"
+        "Fred\n"
         "Cedric Gemy\n"
+        "Steren Giannini\n"
+        "Olivier Gondouin\n"
         "Ted Gould\n"
+        "Toine de Greef\n"
+        "Michael Grosberg\n"
         "Bryce Harrington\n"
         "Dale Harvey\n"
+        "Aurélio Heckert\n"
         "Carl Hetherington\n"
         "Jos Hirth\n"
+        "Thomas Holder\n"
         "Joel Holdsworth\n"
         "Alan Horkan\n"
         "Karl Ove Hufthammer\n"
         "Richard Hughes\n"
         "Nathan Hurst\n"
+        "inductiveload\n"
         "Thomas Ingham\n"
+        "Jean-Olivier Irisson\n"
         "Bob Jamison\n"
+        "jEsuSdA\n"
         "Lauris Kaplinski\n"
         "Lynn Kerby\n"
         "Niko Kiirala\n"
         "James Kilfiger\n"
+        "Jason Kivlighn\n"
+        "Adrian Knoth\n"
+        "Krzysztof Kosiński\n"
         "Petr Kovar\n"
         "Raph Levien\n"
         "Diederik van Lierop\n"
         "Nicklas Lindgren\n"
         "Vitaly Lipatov\n"
+        "Ivan Louette\n"
         "Colin Marquardt\n"
         "Dmitry G. Mastrukov\n"
         "Matiphas\n"
@@ -288,10 +319,12 @@ void AboutBox::initStrings() {
         "Peter Moulder\n"
         "Jörg Müller\n"
         "Yukihiro Nakai\n"
+        "Victor Navez\n"
         "Christian Neumair\n"
         "Andreas Nilsson\n"
         "Mitsuru Oka\n"
         "Marten Owens\n"
+        "Alvin Penner\n"
         "Jon Phillips\n"
         "Zdenko Podobny\n"
         "Alexandre Prokoudine\n"
@@ -305,9 +338,9 @@ void AboutBox::initStrings() {
         "Christian Schaller\n"
         "Marco Scholten\n"
         "Tom von Schwerdtner\n"
+        "Danilo Šegan\n"
         "Shivaken\n"
         "Michael Sloan\n"
-        "Danilo Šegan\n"
         "Boštjan Špetič\n"
         "Aaron Spike\n"
         "Kaushik Sridharan\n"
@@ -315,16 +348,16 @@ void AboutBox::initStrings() {
         "Dariusz Stojek\n"
         "Pat Suwalski\n"
         "Adib Taraben\n"
+        "Hugh Tebby\n"
+        "Jonas Termeau\n"
         "David Turner\n"
         "Andre Twupack\n"
         "Aleksandar Urošević\n"
         "Lucas Vieites\n"
         "Michael Wybrow\n"
         "Daniel Yacob\n"
-        "David Yip\n"
-        "Masatake Yamato";
-
-
+        "Masatake Yamato\n"
+        "David Yip";
 
     //##############################
     //# T R A N S L A T O R S
@@ -356,7 +389,7 @@ void AboutBox::initStrings() {
      * This list is copied from the TRANSLATORS file.  It should
      * be periodically updated.
      */              
-    gchar *allTranslators =
+    gchar const *allTranslators =
     "3ARRANO.com <3arrano@3arrano.com>, 2005.\n"
     "Adib Taraben <theadib@yahoo.com>, 2004.\n"
     "Alastair McKinstry <mckinstry@computer.org>, 2000.\n"
@@ -397,6 +430,7 @@ void AboutBox::initStrings() {
     "Hizkuntza Politikarako Sailburuordetza <hizkpol@ej-gv.es>, 2005.\n"
     "Ilia Penev <lichopicho@gmail.com>, 2006.\n"
     "Iñaki Larrañaga <dooteo@euskalgnu.org>, 2006.\n"
+    "Jin-Hwan Jeong <yongdoria@gmail.com>, 2009.\n"
     "Jörg Müller <jfm@ram-brand.de>, 2005.\n"
     "Jeroen van der Vegt <ajvdvegt (at) 123mail.org>, 2003, 2005.\n"
     "Jonathan Ernst <jernst@users.sourceforge.net>, 2006.\n"
@@ -423,6 +457,7 @@ void AboutBox::initStrings() {
     "Mitsuru Oka <oka326@parkcity.ne.jp>, 2002.\n"
     "Morphix User <pchitrakar@gmail.com>, 2006.\n"
     "Mufit Eribol <meribol@ere.com.tr>, 2000.\n"
+    "Nicolas Dufour <nicoduf@yahoo.fr>, 2008-2009.\n"
     "Pawan Chitrakar <pchitrakar@gmail.com>, 2006.\n"
     "Przemysław Loesch <p_loesch@poczta.onet.pl>, 2005.\n"
     "Quico Llach <quico@softcatala.org>, 2000. Traducció sodipodi.\n"
