@@ -1,9 +1,7 @@
-/*
- * A very simple dialog for displaying Inkscape messages. Messages
- * sent to g_log(), g_warning(), g_message(), ets, are routed here,
- * in order to avoid messing with the startup console.
- *
- * Authors:
+/** @file
+ * @brief Messages dialog - implementation
+ */
+/* Authors:
  *   Bob Jamison
  *   Other dudes from The Inkscape Organization
  *
@@ -46,7 +44,7 @@ void Messages::clear()
  * Constructor
  */
 Messages::Messages()
-    : UI::Widget::Panel("", "dialogs.messages", SP_VERB_DIALOG_DEBUG)
+    : UI::Widget::Panel("", "/dialogs/messages", SP_VERB_DIALOG_DEBUG)
 {
     Gtk::Box *contents = _getContents();
 
@@ -101,8 +99,8 @@ void Messages::message(char *msg)
     buffer->insert (buffer->end(), uMsg);
 }
 
-
-void dialogLoggingFunction(const gchar */*log_domain*/,
+// dialogLoggingCallback is already used in debug.cpp
+static void dialogLoggingCallback(const gchar */*log_domain*/,
                            GLogLevelFlags /*log_level*/,
                            const gchar *messageText,
                            gpointer user_data)
@@ -125,29 +123,29 @@ void Messages::captureLogMessages()
                              G_LOG_LEVEL_INFO    | G_LOG_LEVEL_DEBUG);
     if ( !handlerDefault ) {
         handlerDefault = g_log_set_handler(NULL, flags,
-              dialogLoggingFunction, (gpointer)this);
+              dialogLoggingCallback, (gpointer)this);
     }
     if ( !handlerGlibmm ) {
         handlerGlibmm = g_log_set_handler("glibmm", flags,
-              dialogLoggingFunction, (gpointer)this);
+              dialogLoggingCallback, (gpointer)this);
     }
     if ( !handlerAtkmm ) {
         handlerAtkmm = g_log_set_handler("atkmm", flags,
-              dialogLoggingFunction, (gpointer)this);
+              dialogLoggingCallback, (gpointer)this);
     }
     if ( !handlerPangomm ) {
         handlerPangomm = g_log_set_handler("pangomm", flags,
-              dialogLoggingFunction, (gpointer)this);
+              dialogLoggingCallback, (gpointer)this);
     }
     if ( !handlerGdkmm ) {
         handlerGdkmm = g_log_set_handler("gdkmm", flags,
-              dialogLoggingFunction, (gpointer)this);
+              dialogLoggingCallback, (gpointer)this);
     }
     if ( !handlerGtkmm ) {
         handlerGtkmm = g_log_set_handler("gtkmm", flags,
-              dialogLoggingFunction, (gpointer)this);
+              dialogLoggingCallback, (gpointer)this);
     }
-    message("log capture started");
+    message((char*)"log capture started");
 }
 
 void Messages::releaseLogMessages()
@@ -176,13 +174,20 @@ void Messages::releaseLogMessages()
         g_log_remove_handler("gtkmm", handlerGtkmm);
         handlerGtkmm = 0;
     }
-    message("log capture discontinued");
+    message((char*)"log capture discontinued");
 }
 
 } //namespace Dialog
 } //namespace UI
 } //namespace Inkscape
 
-//#########################################################################
-//## E N D    O F    F I L E
-//#########################################################################
+/*
+  Local Variables:
+  mode:c++
+  c-file-style:"stroustrup"
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
+  indent-tabs-mode:nil
+  fill-column:99
+  End:
+*/
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
