@@ -46,13 +46,10 @@
 
 #include "macros.h"
 #include "path-prefix.h"
-#include "prefs-utils.h"
+#include "preferences.h"
 #include "file.h"
 #include "application/editor.h"
 #include "edit-widget.h"
-#include "ui/stock.h"
-#include "ui/stock-items.h"
-#include "ui/icons.h"
 
 #include "display/sodipodi-ctrlrect.h"
 #include "helper/units.h"
@@ -86,8 +83,6 @@ EditWidget::EditWidget (SPDocument *doc)
     g_warning("Creating new EditWidget");
 
     _desktop = 0;
-    Icons::init();
-    Stock::init();
     initActions();
     initAccelMap();
     initUIManager();
@@ -317,16 +312,8 @@ EditWidget::onUriChanged()
 void
 EditWidget::initMenuActions()
 {
-//    _act_grp->add(Gtk::Action::create("MenuFile",   _("PLACEHOLDER, do not translate")));
-//    _act_grp->add(Gtk::Action::create("MenuEdit",   _("PLACEHOLDER, do not translate")));
-//    _act_grp->add(Gtk::Action::create("MenuView",   _("PLACEHOLDER, do not translate")));
-//    _act_grp->add(Gtk::Action::create("MenuLayer",  _("PLACEHOLDER, do not translate")));
-//    _act_grp->add(Gtk::Action::create("MenuObject", _("PLACEHOLDER, do not translate")));
-//    _act_grp->add(Gtk::Action::create("MenuPath",   _("PLACEHOLDER, do not translate")));
-//    _act_grp->add(Gtk::Action::create("MenuText",   _("PLACEHOLDER, do not translate")));
-//    _act_grp->add(Gtk::Action::create("MenuHelp",   _("PLACEHOLDER, do not translate")));
-// temporarily replaced with non-gettext version to have a well-sized menu
-// for testing:
+// This has no chance of working right now.
+// Has to be converted to Gtk::Action::create_with_icon_name.
 
     _act_grp->add(Gtk::Action::create("MenuFile",   "File"));
     _act_grp->add(Gtk::Action::create("MenuEdit",   "Edit"));
@@ -347,9 +334,9 @@ EditWidget::initMenuActions()
                                       Gtk::Stock::OPEN, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")),
                   sigc::mem_fun(*this, &EditWidget::onActionFileOpen));
-
+/*
     _act_grp->add(Gtk::Action::create("OpenRecent",
-                                      Stock::OPEN_RECENT));
+                                      Stock::OPEN_RECENT));*/
 
     _act_grp->add(Gtk::Action::create("Revert",
                                       Gtk::Stock::REVERT_TO_SAVED, Glib::ustring(),
@@ -365,7 +352,7 @@ EditWidget::initMenuActions()
                                       Gtk::Stock::SAVE_AS, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")),
                   sigc::mem_fun(*this, &EditWidget::onActionFileOpen));
-
+/*
     _act_grp->add(Gtk::Action::create("Import",
                                       Stock::IMPORT, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")),
@@ -374,7 +361,7 @@ EditWidget::initMenuActions()
     _act_grp->add(Gtk::Action::create("Export",
                                       Stock::EXPORT, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")),
-                  sigc::mem_fun(*this, &EditWidget::onDialogExport));
+                  sigc::mem_fun(*this, &EditWidget::onDialogExport));*/
 
     _act_grp->add(Gtk::Action::create("Print",
                                       Gtk::Stock::PRINT, Glib::ustring(),
@@ -384,10 +371,10 @@ EditWidget::initMenuActions()
     _act_grp->add(Gtk::Action::create("PrintPreview",
                                       Gtk::Stock::PRINT_PREVIEW),
                   sigc::mem_fun(*this, &EditWidget::onActionFileOpen));
-
+/*
     _act_grp->add(Gtk::Action::create("VacuumDefs",
                                       Stock::VACUUM_DEFS),
-                  sigc::mem_fun(*this, &EditWidget::onActionFileOpen));
+                  sigc::mem_fun(*this, &EditWidget::onActionFileOpen));*/
 
     _act_grp->add(Gtk::Action::create("DocumentProperties",
                                       Gtk::Stock::PROPERTIES, Glib::ustring(),
@@ -415,10 +402,10 @@ EditWidget::initMenuActions()
     _act_grp->add(Gtk::Action::create("Redo",
                                       Gtk::Stock::REDO, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")));
-
+/*
     _act_grp->add(Gtk::Action::create("UndoHistory",
                                       Stock::UNDO_HISTORY, Glib::ustring(),
-                                      _("PLACEHOLDER, do not translate")));
+                                      _("PLACEHOLDER, do not translate")));*/
 
     _act_grp->add(Gtk::Action::create("Cut",
                                       Gtk::Stock::CUT, Glib::ustring(),
@@ -431,17 +418,17 @@ EditWidget::initMenuActions()
     _act_grp->add(Gtk::Action::create("Paste",
                                       Gtk::Stock::PASTE, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")));
-
+/*
     _act_grp->add(Gtk::Action::create("PasteInPlace",
                                       Stock::PASTE_IN_PLACE));
 
     _act_grp->add(Gtk::Action::create("PasteStyle",
-                                      Stock::PASTE_STYLE));
+                                      Stock::PASTE_STYLE));*/
 
     _act_grp->add(Gtk::Action::create("Find",
                                       Gtk::Stock::FIND),
                   sigc::mem_fun(*this, &EditWidget::onDialogFind));
-
+/*
     _act_grp->add(Gtk::Action::create("Duplicate",
                                       Stock::DUPLICATE, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")));
@@ -762,7 +749,7 @@ EditWidget::initMenuActions()
                                       Stock::SIMPLIFY));
 
     _act_grp->add(Gtk::Action::create("Reverse",
-                                      Stock::REVERSE));
+                                      Stock::REVERSE));*/
 
     _act_grp->add(Gtk::Action::create("Cleanup",
                                       Gtk::Stock::CLEAR,
@@ -773,7 +760,7 @@ EditWidget::initMenuActions()
                                       Gtk::Stock::SELECT_FONT, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")),
                   sigc::mem_fun(*this, &EditWidget::onDialogTextProperties));
-
+/*
     _act_grp->add(Gtk::Action::create("PutOnPath",
                                       Stock::PUT_ON_PATH));
 
@@ -781,13 +768,14 @@ EditWidget::initMenuActions()
                                       Stock::REMOVE_FROM_PATH));
 
     _act_grp->add(Gtk::Action::create("RemoveManualKerns",
-                                      Stock::REMOVE_MANUAL_KERNS));
+                                      Stock::REMOVE_MANUAL_KERNS));*/
 
 	// Whiteboard menu
 #ifdef WITH_INKBOARD
 #endif
 
     // About menu
+/*
     _act_grp->add(Gtk::Action::create("KeysAndMouse",
                                       Stock::KEYS_MOUSE));
 
@@ -796,15 +784,16 @@ EditWidget::initMenuActions()
 
     _act_grp->add(Gtk::Action::create("About",
                                       Stock::ABOUT),
-                  sigc::mem_fun(*this, &EditWidget::onDialogAbout));
+                  sigc::mem_fun(*this, &EditWidget::onDialogAbout));*/
 }
 
 void
 EditWidget::initToolbarActions()
 {
     // Tools bar
+    // This has zero chance to work, and has to be converted to create_with_icon_name
     Gtk::RadioAction::Group tools;
-
+/*
     _act_grp->add(Gtk::RadioAction::create(tools, "ToolSelect",
                                            Stock::TOOL_SELECT, Glib::ustring(),
                                            _("PLACEHOLDER, do not translate")),
@@ -870,7 +859,7 @@ EditWidget::initToolbarActions()
 
     _act_grp->add(Gtk::ToggleAction::create("TransformPattern",
                                             Stock::TRANSFORM_PATTERN, Glib::ustring(),
-                                            _("PLACEHOLDER, do not translate")));
+                                            _("PLACEHOLDER, do not translate")));*/
 
     // Node Controls bar
     _act_grp->add(Gtk::Action::create("NodeInsert",
@@ -880,7 +869,7 @@ EditWidget::initToolbarActions()
     _act_grp->add(Gtk::Action::create("NodeDelete",
                                       Gtk::Stock::REMOVE, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")));
-
+/*
     _act_grp->add(Gtk::Action::create("NodeJoin",
                                       Stock::NODE_JOIN, Glib::ustring(),
                                       _("PLACEHOLDER, do not translate")));
@@ -915,7 +904,7 @@ EditWidget::initToolbarActions()
 
     _act_grp->add(Gtk::Action::create("NodeCurve",
                                       Stock::NODE_CURVE, Glib::ustring(),
-                                      _("PLACEHOLDER, do not translate")));
+                                      _("PLACEHOLDER, do not translate")));*/
 }
 
 void
@@ -1056,9 +1045,9 @@ EditWidget::initRightScrollbar()
 void
 EditWidget::initStickyZoom()
 {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
     _viewport_table.attach(_sticky_zoom, 2, 3, 0, 1, Gtk::SHRINK, Gtk::SHRINK);
-
-    _sticky_zoom.set_active (prefs_get_int_attribute ("options.stickyzoom", "value", 0) != 0);
+    _sticky_zoom.set_active (prefs->getBool("/options/stickyzoom/value"));
     _tooltips.set_tip (_sticky_zoom, _("Zoom drawing if window size changes"));
 
     /// \todo icon not implemented
@@ -1082,22 +1071,22 @@ EditWidget::initStatusbar()
     _coord_eventbox.add (_coord_status);
     _tooltips.set_tip (_coord_eventbox, _("Cursor coordinates"));
     _coord_status.attach (*new Gtk::VSeparator(), 0,1, 0,2, Gtk::FILL,Gtk::FILL, 0,0);
-    _coord_status.attach (*new Gtk::Label("X:", 0.0, 0.5), 1,2, 0,1, Gtk::FILL,Gtk::FILL, 0,0);
-    _coord_status.attach (*new Gtk::Label("Y:", 0.0, 0.5), 1,2, 1,2, Gtk::FILL,Gtk::FILL, 0,0);
+    _coord_status.attach (*new Gtk::Label(_("X:"), 0.0, 0.5), 1,2, 0,1, Gtk::FILL,Gtk::FILL, 0,0);
+    _coord_status.attach (*new Gtk::Label(_("Y:"), 0.0, 0.5), 1,2, 1,2, Gtk::FILL,Gtk::FILL, 0,0);
     _coord_status_x.set_text ("0.0");
     _coord_status_x.set_alignment (0.0, 0.5);
     _coord_status_y.set_text ("0.0");
     _coord_status_y.set_alignment (0.0, 0.5);
     _coord_status.attach (_coord_status_x, 2,3, 0,1, Gtk::FILL,Gtk::FILL, 0,0);
     _coord_status.attach (_coord_status_y, 2,3, 1,2, Gtk::FILL,Gtk::FILL, 0,0);
-    _coord_status.attach (*new Gtk::Label("Z:", 0.0, 0.5), 3,4, 0,2, Gtk::FILL,Gtk::FILL, 0,0);
+    _coord_status.attach (*new Gtk::Label(_("Z:"), 0.0, 0.5), 3,4, 0,2, Gtk::FILL,Gtk::FILL, 0,0);
     _coord_status.attach (_zoom_status, 4,5, 0,2, Gtk::FILL,Gtk::FILL, 0,0);
     sp_set_font_size_smaller (static_cast<GtkWidget*>((void*)_coord_status.gobj()));
     _statusbar.pack_end (_coord_eventbox, false, false, 1);
 
     _select_status.property_xalign() = 0.0;
     _select_status.property_yalign() = 0.5;
-    _select_status.set_markup (_("<b>Welcome to Inkscape!</b> Use shape or freehand tools to create objects; use selector (arrow) to move or transform them."));
+    _select_status.set_markup (_("<b>Welcome to Inkscape!</b> Use shape or drawing tools to create objects; use selector (arrow) to move or transform them."));
     // include this again with Gtk+-2.6
 #if GTK_VERSION_GE(2,6)
      gtk_label_set_ellipsize (GTK_LABEL(_select_status.gobj()), PANGO_ELLIPSIZE_END);
@@ -1149,9 +1138,9 @@ EditWidget::setSize (gint w, gint h)
 }
 
 void
-EditWidget::setPosition (NR::Point p)
+EditWidget::setPosition (Geom::Point p)
 {
-    move (int(p[NR::X]), int(p[NR::Y]));
+    move (int(p[Geom::X]), int(p[Geom::Y]));
 }
 
 /// \param p is already gobj()!
@@ -1163,12 +1152,12 @@ EditWidget::setTransient (void* p, int i)
         this->Gtk::Window::present();
 }
 
-NR::Point
+Geom::Point
 EditWidget::getPointer()
 {
     int x, y;
     get_pointer (x, y);
-    return NR::Point (x, y);
+    return Geom::Point (x, y);
 }
 
 void
@@ -1231,6 +1220,7 @@ EditWidget::shutdown()
         {
             case Gtk::RESPONSE_YES:
                 sp_document_ref(doc);
+                sp_namedview_document_from_window(_desktop);
                 if (sp_file_save_document(*this, doc)) {
                     sp_document_unref(doc);
                 } else { // save dialog cancelled or save failed
@@ -1257,7 +1247,7 @@ EditWidget::shutdown()
                 _("<span weight=\"bold\" size=\"larger\">The file \"%s\" was saved with a format (%s) that may cause data loss!</span>\n\n"
                   "Do you want to save this file as an Inkscape SVG?"),
                 SP_DOCUMENT_NAME(doc),
-                Inkscape::Extension::db.get(sp_document_repr_root(doc)->attribute("inkscape:output_extension"))->get_name());
+                SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE);
 
         Gtk::MessageDialog dlg (*this,
                        markup,
@@ -1318,7 +1308,7 @@ EditWidget::requestCanvasUpdateAndWait()
 {
     requestCanvasUpdate();
 
-    while (gtk_events_pending()) 
+    while (gtk_events_pending())
       gtk_main_iteration_do(FALSE);
 }
 
@@ -1326,9 +1316,9 @@ void
 EditWidget::enableInteraction()
 {
   g_return_if_fail(_interaction_disabled_counter > 0);
-  
+
   _interaction_disabled_counter--;
-  
+
   if (_interaction_disabled_counter == 0) {
     this->set_sensitive(true);
   }
@@ -1340,7 +1330,7 @@ EditWidget::disableInteraction()
   if (_interaction_disabled_counter == 0) {
     this->set_sensitive(false);
   }
-  
+
   _interaction_disabled_counter++;
 }
 
@@ -1357,34 +1347,34 @@ EditWidget::deactivateDesktop()
 }
 
 void
-EditWidget::viewSetPosition (NR::Point p)
+EditWidget::viewSetPosition (Geom::Point p)
 {
-    // p -= _namedview->gridorigin;    
+    // p -= _namedview->gridorigin;
     /// \todo Why was the origin corrected for the grid origin? (johan)
-    
+
     double lo, up, pos, max;
     _top_ruler.get_range (lo, up, pos, max);
-    _top_ruler.set_range (lo, up, p[NR::X], max);
+    _top_ruler.set_range (lo, up, p[Geom::X], max);
     _left_ruler.get_range (lo, up, pos, max);
-    _left_ruler.set_range (lo, up, p[NR::Y], max);
+    _left_ruler.set_range (lo, up, p[Geom::Y], max);
 }
 
 void
 EditWidget::updateRulers()
 {
-    //NR::Point gridorigin = _namedview->gridorigin;
+    //Geom::Point gridorigin = _namedview->gridorigin;
     /// \todo Why was the origin corrected for the grid origin? (johan)
-    
-    NR::Rect const viewbox = _svg_canvas.spobj()->getViewbox();
+
+    Geom::Rect const viewbox = _svg_canvas.spobj()->getViewbox();
     double lo, up, pos, max;
     double const scale = _desktop->current_zoom();
-    double s = viewbox.min()[NR::X] / scale; //- gridorigin[NR::X];
-    double e = viewbox.max()[NR::X] / scale; //- gridorigin[NR::X];
+    double s = viewbox.min()[Geom::X] / scale; //- gridorigin[Geom::X];
+    double e = viewbox.max()[Geom::X] / scale; //- gridorigin[Geom::X];
     _top_ruler.get_range(lo, up, pos, max);
     _top_ruler.set_range(s, e, pos, e);
-    s = viewbox.min()[NR::Y] / -scale; //- gridorigin[NR::Y];
-    e = viewbox.max()[NR::Y] / -scale; //- gridorigin[NR::Y];
-    _left_ruler.set_range(s, e, 0 /*gridorigin[NR::Y]*/, e);
+    s = viewbox.min()[Geom::Y] / -scale; //- gridorigin[Geom::Y];
+    e = viewbox.max()[Geom::Y] / -scale; //- gridorigin[Geom::Y];
+    _left_ruler.set_range(s, e, 0 /*gridorigin[Geom::Y]*/, e);
     /// \todo is that correct?
 }
 
@@ -1400,38 +1390,36 @@ EditWidget::updateScrollbars (double scale)
 
     /* The desktop region we always show unconditionally */
     SPDocument *doc = _desktop->doc();
-    NR::Rect darea = NR::Rect(NR::Point(-sp_document_width(doc),
-                                        -sp_document_height(doc)),
-                              NR::Point(2 * sp_document_width(doc),
-                                        2 * sp_document_height(doc)));
-    darea = NR::union_bounds(darea, sp_item_bbox_desktop(SP_ITEM(SP_DOCUMENT_ROOT(doc))));
+    Geom::Rect darea ( Geom::Point(-sp_document_width(doc), -sp_document_height(doc)),
+                     Geom::Point(2 * sp_document_width(doc), 2 * sp_document_height(doc))  );
+    SPObject* root = doc->root;
+    SPItem* item = SP_ITEM(root);
+    Geom::OptRect deskarea = Geom::unify(darea, sp_item_bbox_desktop(item));
 
     /* Canvas region we always show unconditionally */
-    NR::Rect carea(NR::Point(darea.min()[NR::X] * scale - 64,
-                             darea.max()[NR::Y] * -scale - 64),
-                   NR::Point(darea.max()[NR::X] * scale + 64,
-                             darea.min()[NR::Y] * -scale + 64));
+    Geom::Rect carea( Geom::Point(deskarea->min()[Geom::X] * scale - 64, deskarea->max()[Geom::Y] * -scale - 64),
+                    Geom::Point(deskarea->max()[Geom::X] * scale + 64, deskarea->min()[Geom::Y] * -scale + 64)  );
 
-    NR::Rect const viewbox = _svg_canvas.spobj()->getViewbox();
+    Geom::Rect const viewbox = _svg_canvas.spobj()->getViewbox();
 
     /* Viewbox is always included into scrollable region */
-    carea = NR::union_bounds(carea, viewbox);
+    carea = Geom::unify(carea, viewbox);
 
     Gtk::Adjustment *adj = _bottom_scrollbar.get_adjustment();
-    adj->set_value(viewbox.min()[NR::X]);
-    adj->set_lower(carea.min()[NR::X]);
-    adj->set_upper(carea.max()[NR::X]);
-    adj->set_page_increment(viewbox.dimensions()[NR::X]);
-    adj->set_step_increment(0.1 * (viewbox.dimensions()[NR::X]));
-    adj->set_page_size(viewbox.dimensions()[NR::X]);
+    adj->set_value(viewbox.min()[Geom::X]);
+    adj->set_lower(carea.min()[Geom::X]);
+    adj->set_upper(carea.max()[Geom::X]);
+    adj->set_page_increment(viewbox.dimensions()[Geom::X]);
+    adj->set_step_increment(0.1 * (viewbox.dimensions()[Geom::X]));
+    adj->set_page_size(viewbox.dimensions()[Geom::X]);
 
     adj = _right_scrollbar.get_adjustment();
-    adj->set_value(viewbox.min()[NR::Y]);
-    adj->set_lower(carea.min()[NR::Y]);
-    adj->set_upper(carea.max()[NR::Y]);
-    adj->set_page_increment(viewbox.dimensions()[NR::Y]);
-    adj->set_step_increment(0.1 * viewbox.dimensions()[NR::Y]);
-    adj->set_page_size(viewbox.dimensions()[NR::Y]);
+    adj->set_value(viewbox.min()[Geom::Y]);
+    adj->set_lower(carea.min()[Geom::Y]);
+    adj->set_upper(carea.max()[Geom::Y]);
+    adj->set_page_increment(viewbox.dimensions()[Geom::Y]);
+    adj->set_step_increment(0.1 * viewbox.dimensions()[Geom::Y]);
+    adj->set_page_size(viewbox.dimensions()[Geom::Y]);
 
     _update_s_f = false;
 }
@@ -1439,30 +1427,32 @@ EditWidget::updateScrollbars (double scale)
 void
 EditWidget::toggleRulers()
 {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
     if (_top_ruler.is_visible())
     {
         _top_ruler.hide_all();
         _left_ruler.hide_all();
-        prefs_set_int_attribute (_desktop->is_fullscreen() ? "fullscreen.rulers" : "window.rulers", "state", 0);
+        prefs->setBool(_desktop->is_fullscreen() ? "/fullscreen/rulers/state" : "/window/rulers/state", false);
     } else {
         _top_ruler.show_all();
         _left_ruler.show_all();
-        prefs_set_int_attribute (_desktop->is_fullscreen() ? "fullscreen.rulers" : "window.rulers", "state", 1);
+        prefs->setBool(_desktop->is_fullscreen() ? "/fullscreen/rulers/state" : "/window/rulers/state", true);
     }
 }
 
 void
 EditWidget::toggleScrollbars()
 {
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
     if (_bottom_scrollbar.is_visible())
     {
         _bottom_scrollbar.hide_all();
         _right_scrollbar.hide_all();
-        prefs_set_int_attribute (_desktop->is_fullscreen() ? "fullscreen.scrollbars" : "window.scrollbars", "state", 0);
+        prefs->setBool(_desktop->is_fullscreen() ? "/fullscreen/scrollbars/state" : "/window/scrollbars/state", false);
     } else {
         _bottom_scrollbar.show_all();
         _right_scrollbar.show_all();
-        prefs_set_int_attribute (_desktop->is_fullscreen() ? "fullscreen.scrollbars" : "window.scrollbars", "state", 1);
+        prefs->setBool(_desktop->is_fullscreen() ? "/fullscreen/scrollbars/state" : "/window/scrollbars/state", true);
     }
 }
 
@@ -1509,12 +1499,12 @@ EditWidget::isToolboxButtonActive (gchar const*)
 }
 
 void
-EditWidget::setCoordinateStatus (NR::Point p)
+EditWidget::setCoordinateStatus (Geom::Point p)
 {
-    gchar *cstr = g_strdup_printf ("%6.2f", _dt2r * p[NR::X]);
+    gchar *cstr = g_strdup_printf ("%6.2f", _dt2r * p[Geom::X]);
     _coord_status_x.property_label() = cstr;
     g_free (cstr);
-    cstr = g_strdup_printf ("%6.2f", _dt2r * p[NR::Y]);
+    cstr = g_strdup_printf ("%6.2f", _dt2r * p[Geom::Y]);
     _coord_status_y.property_label() = cstr;
     g_free (cstr);
 }
@@ -1561,11 +1551,10 @@ void
 EditWidget::initEdit (SPDocument *doc)
 {
     _desktop = new SPDesktop();
-    _desktop->registerEditWidget (this);
 
     _namedview = sp_document_namedview (doc, 0);
     _svg_canvas.init (_desktop);
-    _desktop->init (_namedview, _svg_canvas.spobj());
+    _desktop->init (_namedview, _svg_canvas.spobj(), this);
     sp_namedview_window_from_document (_desktop);
     sp_namedview_update_layers_from_document (_desktop);
     _dt2r = 1.0 / _namedview->doc_units->unittobase;
@@ -1646,32 +1635,33 @@ EditWidget::onWindowSizeAllocate (Gtk::Allocation &newall)
         return;
     }
 
-    NR::Rect const area = _desktop->get_display_area();
+    Geom::Rect const area = _desktop->get_display_area();
     double zoom = _desktop->current_zoom();
 
     if (_sticky_zoom.get_active()) {
         /* Calculate zoom per pixel */
-        double const zpsp = zoom / hypot(area.dimensions()[NR::X], area.dimensions()[NR::Y]);
+        double const zpsp = zoom / hypot(area.dimensions()[Geom::X], area.dimensions()[Geom::Y]);
         /* Find new visible area */
-        NR::Rect const newarea = _desktop->get_display_area();
+        Geom::Rect const newarea = _desktop->get_display_area();
         /* Calculate adjusted zoom */
-        zoom = zpsp * hypot(newarea.dimensions()[NR::X], newarea.dimensions()[NR::Y]);
+        zoom = zpsp * hypot(newarea.dimensions()[Geom::X], newarea.dimensions()[Geom::Y]);
     }
 
-    _desktop->zoom_absolute(area.midpoint()[NR::X], area.midpoint()[NR::Y], zoom);
+    _desktop->zoom_absolute(area.midpoint()[Geom::X], area.midpoint()[Geom::Y], zoom);
 }
 
 void
 EditWidget::onWindowRealize()
 {
-    NR::Rect d(NR::Point(0, 0),
-               NR::Point(sp_document_width(_desktop->doc()), sp_document_height(_desktop->doc())));
 
-    if (d.isEmpty(1.0)) {
+    if ( (sp_document_width(_desktop->doc()) < 1.0) || (sp_document_height(_desktop->doc()) < 1.0) ) {
         return;
     }
 
-    _desktop->set_display_area(d.min()[NR::X], d.min()[NR::Y], d.max()[NR::X], d.max()[NR::Y], 10);
+    Geom::Rect d( Geom::Point(0, 0),
+                  Geom::Point(sp_document_width(_desktop->doc()), sp_document_height(_desktop->doc())) );
+
+    _desktop->set_display_area(d.min()[Geom::X], d.min()[Geom::Y], d.max()[Geom::X], d.max()[Geom::Y], 10);
     _namedview_modified(_desktop->namedview, SP_OBJECT_MODIFIED_FLAG);
     setTitle (SP_DOCUMENT_NAME(_desktop->doc()));
 }
