@@ -26,10 +26,10 @@
 
 #include "panel.h"
 #include "icon-size.h"
-#include "prefs-utils.h"
+#include "preferences.h"
 #include "desktop-handles.h"
 #include "inkscape.h"
-#include "dialogs/eek-preview.h"
+#include "widgets/eek-preview.h"
 
 namespace Inkscape {
 namespace UI {
@@ -44,7 +44,7 @@ static const int PANEL_SETTING_NEXTFREE = 4;
 
 void Panel::prep() {
     GtkIconSize sizes[] = {
-        static_cast<GtkIconSize>(Inkscape::ICON_SIZE_DECORATION),
+        Inkscape::getRegisteredIconSize(Inkscape::ICON_SIZE_DECORATION),
         GTK_ICON_SIZE_MENU,
         GTK_ICON_SIZE_SMALL_TOOLBAR,
         GTK_ICON_SIZE_BUTTON,
@@ -94,71 +94,125 @@ void Panel::_init()
     Glib::ustring tmp("<");
     _anchor = Gtk::ANCHOR_CENTER;
 
-    guint panel_size = 0;
-    if (_prefs_path) {
-        panel_size = prefs_get_int_attribute_limited( _prefs_path, "panel_size", 1, 0, static_cast<long long>(PREVIEW_SIZE_HUGE) );
-    }
-
-    guint panel_mode = 0;
-    if (_prefs_path) {
-        panel_mode = prefs_get_int_attribute_limited( _prefs_path, "panel_mode", 1, 0, 10 );
-    }
-
-    guint panel_wrap = 0;
-    if (_prefs_path) {
-        panel_wrap = prefs_get_int_attribute_limited( _prefs_path, "panel_wrap", 0, 0, 1 );
-    }
-
-    guint panel_ratio = 100;
-    if (_prefs_path) {
-        panel_ratio = prefs_get_int_attribute_limited( _prefs_path, "panel_ratio", 100, 0, 500 );
+    guint panel_size = 0, panel_mode = 0, panel_ratio = 100;
+    bool panel_wrap = 0;
+    if (!_prefs_path.empty()) {
+        Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+        panel_wrap = prefs->getBool(_prefs_path + "/panel_wrap");
+        panel_size = prefs->getIntLimited(_prefs_path + "/panel_size", 1, 0, PREVIEW_SIZE_HUGE);
+        panel_mode = prefs->getIntLimited(_prefs_path + "/panel_mode", 1, 0, 10);
+        panel_ratio = prefs->getIntLimited(_prefs_path + "/panel_ratio", 100, 0, 500 );
     }
 
     _menu = new Gtk::Menu();
+
     {
-        const char *things[] = {
+        Gtk::RadioMenuItem::Group group;
+        Glib::ustring one_label(_("List"));
+        Glib::ustring two_label(_("Grid"));
+        Gtk::RadioMenuItem *one = manage(new Gtk::RadioMenuItem(group, one_label));
+        Gtk::RadioMenuItem *two = manage(new Gtk::RadioMenuItem(group, two_label));
+
+        if (panel_mode == 0) {
+            one->set_active(true);
+        } else if (panel_mode == 1) {
+            two->set_active(true);
+        }
+
+        _menu->append(*one);
+        _non_horizontal.push_back(one);
+        _menu->append(*two);
+        _non_horizontal.push_back(two);
+        Gtk::MenuItem* sep = manage(new Gtk::SeparatorMenuItem());
+        _menu->append(*sep);
+        _non_horizontal.push_back(sep);
+        one->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &Panel::_bounceCall), PANEL_SETTING_MODE, 0));
+        two->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &Panel::_bounceCall), PANEL_SETTING_MODE, 1));
+    }
+
+    {
+        //TRANSLATORS: only translate "string" in "context|string".
+        // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+    	Glib::ustring heightItemLabel(Q_("swatches|Size"));
+
+    	//TRANSLATORS: Indicates size of colour swatches
+        const gchar *heightLabels[] = {
             N_("tiny"),
             N_("small"),
-            N_("swatches|medium"),
+            //TRANSLATORS: only translate "string" in "context|string".
+            // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+            // "medium" indicates size of colour swatches
+            N_("swatchesHeight|medium"),
             N_("large"),
             N_("huge")
         };
-        Gtk::RadioMenuItem::Group groupOne;
-        for (unsigned int i = 0; i < G_N_ELEMENTS(things); i++) {
-            Glib::ustring foo(Q_(things[i]));
-            Gtk::RadioMenuItem* single = manage(new Gtk::RadioMenuItem(groupOne, foo));
-            _menu->append(*single);
+
+        Gtk::MenuItem *sizeItem = manage(new Gtk::MenuItem(heightItemLabel));
+        Gtk::Menu *sizeMenu = manage(new Gtk::Menu());
+        sizeItem->set_submenu(*sizeMenu);
+
+        Gtk::RadioMenuItem::Group heightGroup;
+        for (unsigned int i = 0; i < G_N_ELEMENTS(heightLabels); i++) {
+            Glib::ustring _label(Q_(heightLabels[i]));
+            Gtk::RadioMenuItem* _item = manage(new Gtk::RadioMenuItem(heightGroup, _label));
+            sizeMenu->append(*_item);
             if (i == panel_size) {
-                single->set_active(true);
+            	_item->set_active(true);
             }
-            single->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &Panel::_bounceCall), PANEL_SETTING_SIZE, i));
+            _item->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &Panel::_bounceCall), PANEL_SETTING_SIZE, i));
        }
-    }
-    _menu->append(*manage(new Gtk::SeparatorMenuItem()));
-    Gtk::RadioMenuItem::Group group;
-    Glib::ustring one_label(_("List"));
-    Glib::ustring two_label(_("Grid"));
-    Gtk::RadioMenuItem *one = manage(new Gtk::RadioMenuItem(group, one_label));
-    Gtk::RadioMenuItem *two = manage(new Gtk::RadioMenuItem(group, two_label));
 
-    if (panel_mode == 0) {
-        one->set_active(true);
-    } else if (panel_mode == 1) {
-        two->set_active(true);
+       _menu->append(*sizeItem);
     }
 
-    _menu->append(*one);
-    _non_horizontal.push_back(one);
-    _menu->append(*two);
-    _non_horizontal.push_back(two);
-    Gtk::MenuItem* sep = manage(new Gtk::SeparatorMenuItem());
-    _menu->append(*sep);
-    _non_horizontal.push_back(sep);
-    one->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &Panel::_bounceCall), PANEL_SETTING_MODE, 0));
-    two->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &Panel::_bounceCall), PANEL_SETTING_MODE, 1));
-
     {
-        Glib::ustring wrap_label(_("Wrap"));
+        //TRANSLATORS: only translate "string" in "context|string".
+        // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+        Glib::ustring widthItemLabel(Q_("swatches|Width"));
+
+        //TRANSLATORS: Indicates width of colour swatches
+        const gchar *widthLabels[] = {
+            N_("narrower"),
+            N_("narrow"),
+            //TRANSLATORS: only translate "string" in "context|string".
+            // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+            // "medium" indicates width of colour swatches
+            N_("swatchesWidth|medium"),
+            N_("wide"),
+            N_("wider")
+        };
+
+        Gtk::MenuItem *item = manage( new Gtk::MenuItem(widthItemLabel));
+        Gtk::Menu *type_menu = manage(new Gtk::Menu());
+        item->set_submenu(*type_menu);
+        _menu->append(*item);
+
+        Gtk::RadioMenuItem::Group widthGroup;
+
+        guint values[] = {0, 25, 50, 100, 200, 400};
+        guint hot_index = 3;
+        for ( guint i = 0; i < G_N_ELEMENTS(widthLabels); ++i ) {
+            // Assume all values are in increasing order
+            if ( values[i] <= panel_ratio ) {
+                hot_index = i;
+            }
+        }
+        for ( guint i = 0; i < G_N_ELEMENTS(widthLabels); ++i ) {
+        	Glib::ustring _label(Q_(widthLabels[i]));
+            Gtk::RadioMenuItem *_item = manage(new Gtk::RadioMenuItem(widthGroup, _label));
+            type_menu->append(*_item);
+            if ( i <= hot_index ) {
+            	_item->set_active(true);
+            }
+            _item->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &Panel::_bounceCall), PANEL_SETTING_SHAPE, values[i]));
+        }
+    }
+
+    {
+        //TRANSLATORS: only translate "string" in "context|string".
+        // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+        // "Wrap" indicates how colour swatches are displayed
+    	Glib::ustring wrap_label(Q_("swatches|Wrap"));
         Gtk::CheckMenuItem *check = manage(new Gtk::CheckMenuItem(wrap_label));
         check->set_active(panel_wrap);
         _menu->append(*check);
@@ -167,41 +221,7 @@ void Panel::_init()
         check->signal_toggled().connect(sigc::bind<Gtk::CheckMenuItem*>(sigc::mem_fun(*this, &Panel::_wrapToggled), check));
     }
 
-    {
-        Glib::ustring type_label(_("Shape"));
-
-        Glib::ustring ellipsis(_("..."));
-        Glib::ustring shape_1_label(_("Tall"));
-        Glib::ustring shape_2_label(_("Square"));
-        Glib::ustring shape_3_label(_("Wide"));
-
-        Gtk::MenuItem *item = manage( new Gtk::MenuItem(type_label));
-        Gtk::Menu *type_menu = manage(new Gtk::Menu());
-        item->set_submenu(*type_menu);
-        _menu->append(*item);
-
-        Gtk::RadioMenuItem::Group shapeGroup;
-
-
-        Glib::ustring* labels[] = {&ellipsis, &shape_1_label, &ellipsis, &shape_2_label, &ellipsis, &shape_3_label};
-        guint values[] = {0, 25, 50, 100, 200, 400};
-        guint hot_index = 3;
-        for ( guint i = 0; i < G_N_ELEMENTS(labels); ++i ) {
-            // Assume all values are in increasing order
-            if ( values[i] <= panel_ratio ) {
-                hot_index = i;
-            }
-        }
-        for ( guint i = 0; i < G_N_ELEMENTS(labels); ++i ) {
-            Gtk::RadioMenuItem *single = manage(new Gtk::RadioMenuItem(shapeGroup, *(labels[i])));
-            type_menu->append(*single);
-            if ( i <= hot_index ) {
-                single->set_active(true);
-            }
-            single->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &Panel::_bounceCall), PANEL_SETTING_SHAPE, values[i]));
-        }
-    }
-
+    Gtk::SeparatorMenuItem *sep;
     sep = manage(new Gtk::SeparatorMenuItem());
     _menu->append(*sep);
 
@@ -225,7 +245,7 @@ void Panel::_init()
         gint width = 0;
         gint height = 0;
 
-        if ( gtk_icon_size_lookup( static_cast<GtkIconSize>(Inkscape::ICON_SIZE_DECORATION), &width, &height ) ) {
+        if ( gtk_icon_size_lookup( Inkscape::getRegisteredIconSize(Inkscape::ICON_SIZE_DECORATION), &width, &height ) ) {
             _temp_arrow.set_size_request(width, height);
         }
 
@@ -316,21 +336,14 @@ void Panel::present()
 
 void Panel::restorePanelPrefs()
 {
-    guint panel_size = 0;
-    if (_prefs_path) {
-        panel_size = prefs_get_int_attribute_limited(_prefs_path, "panel_size", 1, 0, static_cast<long long>(PREVIEW_SIZE_HUGE));
-    }
-    guint panel_mode = 0;
-    if (_prefs_path) {
-        panel_mode = prefs_get_int_attribute_limited(_prefs_path, "panel_mode", 1, 0, 10);
-    }
-    guint panel_wrap = 0;
-    if (_prefs_path) {
-        panel_wrap = prefs_get_int_attribute_limited(_prefs_path, "panel_wrap", 0, 0, 1 );
-    }
-    guint panel_ratio = 100;
-    if (_prefs_path) {
-        panel_ratio = prefs_get_int_attribute_limited(_prefs_path, "panel_ratio", 000, 0, 500 );
+    guint panel_size = 0, panel_mode = 0, panel_ratio = 100;
+    bool panel_wrap = 0;
+    if (!_prefs_path.empty()) {
+        Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+        panel_wrap = prefs->getBool(_prefs_path + "/panel_wrap");
+        panel_size = prefs->getIntLimited(_prefs_path + "/panel_size", 1, 0, PREVIEW_SIZE_HUGE);
+        panel_mode = prefs->getIntLimited(_prefs_path + "/panel_mode", 1, 0, 10);
+        panel_ratio = prefs->getIntLimited(_prefs_path + "/panel_ratio", 000, 0, 500 );
     }
     _bounceCall(PANEL_SETTING_SIZE, panel_size);
     _bounceCall(PANEL_SETTING_MODE, panel_mode);
@@ -355,8 +368,9 @@ void Panel::_bounceCall(int i, int j)
     _menu->set_active(0);
     switch (i) {
     case PANEL_SETTING_SIZE:
-        if (_prefs_path) {
-            prefs_set_int_attribute(_prefs_path, "panel_size", j);
+        if (!_prefs_path.empty()) {
+            Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+            prefs->setInt(_prefs_path + "/panel_size", j);
         }
         if (_fillable) {
             ViewType curr_type = _fillable->getPreviewType();
@@ -393,8 +407,9 @@ void Panel::_bounceCall(int i, int j)
         }
         break;
     case PANEL_SETTING_MODE:
-        if (_prefs_path) {
-            prefs_set_int_attribute (_prefs_path, "panel_mode", j);
+        if (!_prefs_path.empty()) {
+            Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+            prefs->setInt(_prefs_path + "/panel_mode", j);
         }
         if (_fillable) {
             ::PreviewSize curr_size = _fillable->getPreviewSize();
@@ -416,8 +431,9 @@ void Panel::_bounceCall(int i, int j)
         }
         break;
     case PANEL_SETTING_SHAPE:
-        if (_prefs_path) {
-            prefs_set_int_attribute (_prefs_path, "panel_ratio", j);
+        if (!_prefs_path.empty()) {
+            Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+            prefs->setInt(_prefs_path + "/panel_ratio", j);
         }
         if ( _fillable ) {
             ViewType curr_type = _fillable->getPreviewType();
@@ -426,8 +442,9 @@ void Panel::_bounceCall(int i, int j)
         }
         break;
     case PANEL_SETTING_WRAP:
-        if (_prefs_path) {
-            prefs_set_int_attribute (_prefs_path, "panel_wrap", j ? 1 : 0);
+        if (!_prefs_path.empty()) {
+            Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+            prefs->setBool(_prefs_path + "/panel_wrap", j);
         }
         if ( _fillable ) {
             _fillable->setWrap(j);
@@ -448,7 +465,7 @@ void Panel::_wrapToggled(Gtk::CheckMenuItem* toggler)
 
 gchar const *Panel::getPrefsPath() const
 {
-    return _prefs_path;
+    return _prefs_path.data();
 }
 
 Glib::ustring const &Panel::getLabel() const
