@@ -40,6 +40,7 @@ Scalar::Scalar(Glib::ustring const &label, Glib::ustring const &tooltip,
     : Labelled(label, tooltip, new Gtk::SpinButton(), suffix, icon, mnemonic),
       setProgrammatically(false)
 {
+    static_cast<Gtk::SpinButton*>(_widget)->set_numeric();
 }
 
 /**
@@ -61,6 +62,7 @@ Scalar::Scalar(Glib::ustring const &label, Glib::ustring const &tooltip,
     : Labelled(label, tooltip, new Gtk::SpinButton(0.0, digits), suffix, icon, mnemonic),
       setProgrammatically(false)
 {
+    static_cast<Gtk::SpinButton*>(_widget)->set_numeric();
 }
 
 /**
@@ -84,6 +86,7 @@ Scalar::Scalar(Glib::ustring const &label, Glib::ustring const &tooltip,
     : Labelled(label, tooltip, new Gtk::SpinButton(adjust, 0.0, digits), suffix, icon, mnemonic),
       setProgrammatically(false)
 {
+    static_cast<Gtk::SpinButton*>(_widget)->set_numeric();
 }
 
 /** Fetches the precision of the spin buton */
@@ -159,12 +162,14 @@ Scalar::setDigits(unsigned digits)
     static_cast<Gtk::SpinButton*>(_widget)->set_digits(digits);
 }
 
-/** Sets the step and page increments for the spin button */
+/** Sets the step and page increments for the spin button
+ * @todo Remove the second parameter - deprecated
+ */
 void
-Scalar::setIncrements(double step, double page)
+Scalar::setIncrements(double step, double /*page*/)
 {
     g_assert(_widget != NULL);
-    static_cast<Gtk::SpinButton*>(_widget)->set_increments(step, page);
+    static_cast<Gtk::SpinButton*>(_widget)->set_increments(step, 0);
 }
 
 /** Sets the minimum and maximum range allowed for the spin button */
