@@ -19,6 +19,7 @@
 #include <cstring>
 #include <string>
 #include <string.h>
+#include "config.h"
 #include "require-config.h"   /* HAVE_GTK_WINDOW_FULLSCREEN */
 #include "helper/helper-forward.h"
 #include "forward.h"
@@ -37,14 +38,14 @@ enum {
     SP_VERB_FILE_REVERT,           /**< Revert this file to its original state. */
     SP_VERB_FILE_SAVE,             /**< Save the current file with its saved filename */
     SP_VERB_FILE_SAVE_AS,          /**< Save the current file with a new filename */
-    SP_VERB_FILE_SAVE_A_COPY,      /**< Save a copy of the current file */ 
+    SP_VERB_FILE_SAVE_A_COPY,      /**< Save a copy of the current file */
     SP_VERB_FILE_PRINT,
     SP_VERB_FILE_VACUUM,
     SP_VERB_FILE_PRINT_PREVIEW,
     SP_VERB_FILE_IMPORT,
     SP_VERB_FILE_EXPORT,
     SP_VERB_FILE_IMPORT_FROM_OCAL, /**< Import the file from Open Clip Art Library */
-    SP_VERB_FILE_EXPORT_TO_OCAL, /**< Export the file to Open Clip Art  Library */
+//    SP_VERB_FILE_EXPORT_TO_OCAL, /**< Export the file to Open Clip Art  Library */
     SP_VERB_FILE_NEXT_DESKTOP,
     SP_VERB_FILE_PREV_DESKTOP,
     SP_VERB_FILE_CLOSE_VIEW,
@@ -64,10 +65,13 @@ enum {
     SP_VERB_EDIT_PASTE_SIZE_SEPARATELY_Y,
     SP_VERB_EDIT_PASTE_IN_PLACE,
     SP_VERB_EDIT_PASTE_LIVEPATHEFFECT,
+    SP_VERB_EDIT_REMOVE_LIVEPATHEFFECT,
+    SP_VERB_EDIT_REMOVE_FILTER,
     SP_VERB_EDIT_DELETE,
     SP_VERB_EDIT_DUPLICATE,
     SP_VERB_EDIT_CLONE,
     SP_VERB_EDIT_UNLINK_CLONE,
+    SP_VERB_EDIT_RELINK_CLONE,
     SP_VERB_EDIT_CLONE_SELECT_ORIGINAL,
     SP_VERB_EDIT_SELECTION_2_MARKER,
     SP_VERB_EDIT_SELECTION_2_GUIDES,
@@ -81,6 +85,7 @@ enum {
     SP_VERB_EDIT_SELECT_NEXT,
     SP_VERB_EDIT_SELECT_PREV,
     SP_VERB_EDIT_DESELECT,
+    SP_VERB_EDIT_GUIDES_AROUND_PAGE,
     SP_VERB_EDIT_NEXT_PATHEFFECT_PARAMETER,
     /* Selection */
     SP_VERB_SELECTION_TO_FRONT,
@@ -125,7 +130,9 @@ enum {
     SP_VERB_LAYER_TO_BOTTOM,
     SP_VERB_LAYER_RAISE,
     SP_VERB_LAYER_LOWER,
+    SP_VERB_LAYER_DUPLICATE,
     SP_VERB_LAYER_DELETE,
+    SP_VERB_LAYER_SOLO,
     /* Object */
     SP_VERB_OBJECT_ROTATE_90_CW,
     SP_VERB_OBJECT_ROTATE_90_CCW,
@@ -137,8 +144,10 @@ enum {
     SP_VERB_OBJECT_FLIP_HORIZONTAL,
     SP_VERB_OBJECT_FLIP_VERTICAL,
     SP_VERB_OBJECT_SET_MASK,
+    SP_VERB_OBJECT_EDIT_MASK,
     SP_VERB_OBJECT_UNSET_MASK,
     SP_VERB_OBJECT_SET_CLIPPATH,
+    SP_VERB_OBJECT_EDIT_CLIPPATH,
     SP_VERB_OBJECT_UNSET_CLIPPATH,
     /* Tools */
     SP_VERB_CONTEXT_SELECT,
@@ -146,7 +155,6 @@ enum {
     SP_VERB_CONTEXT_TWEAK,
     SP_VERB_CONTEXT_RECT,
     SP_VERB_CONTEXT_3DBOX,
-    //SP_VERB_CONTEXT_BOX3D,
     SP_VERB_CONTEXT_ARC,
     SP_VERB_CONTEXT_STAR,
     SP_VERB_CONTEXT_SPIRAL,
@@ -159,13 +167,15 @@ enum {
     SP_VERB_CONTEXT_DROPPER,
     SP_VERB_CONTEXT_CONNECTOR,
     SP_VERB_CONTEXT_PAINTBUCKET,
+    SP_VERB_CONTEXT_LPE, /* not really a tool but used for editing LPE parameters on-canvas for example */
+    SP_VERB_CONTEXT_ERASER,
+    SP_VERB_CONTEXT_LPETOOL, /* note that this is very different from SP_VERB_CONTEXT_LPE above! */
     /* Tool preferences */
     SP_VERB_CONTEXT_SELECT_PREFS,
     SP_VERB_CONTEXT_NODE_PREFS,
     SP_VERB_CONTEXT_TWEAK_PREFS,
     SP_VERB_CONTEXT_RECT_PREFS,
     SP_VERB_CONTEXT_3DBOX_PREFS,
-    //SP_VERB_CONTEXT_BOX3D_PREFS,
     SP_VERB_CONTEXT_ARC_PREFS,
     SP_VERB_CONTEXT_STAR_PREFS,
     SP_VERB_CONTEXT_SPIRAL_PREFS,
@@ -178,6 +188,8 @@ enum {
     SP_VERB_CONTEXT_DROPPER_PREFS,
     SP_VERB_CONTEXT_CONNECTOR_PREFS,
     SP_VERB_CONTEXT_PAINTBUCKET_PREFS,
+    SP_VERB_CONTEXT_ERASER_PREFS,
+    SP_VERB_CONTEXT_LPETOOL_PREFS,
     /* Zooming and desktop settings */
     SP_VERB_ZOOM_IN,
     SP_VERB_ZOOM_OUT,
@@ -194,9 +206,11 @@ enum {
 #ifdef HAVE_GTK_WINDOW_FULLSCREEN
     SP_VERB_FULLSCREEN,
 #endif /* HAVE_GTK_WINDOW_FULLSCREEN */
+    SP_VERB_FOCUSTOGGLE,
     SP_VERB_VIEW_NEW,
     SP_VERB_VIEW_NEW_PREVIEW,
     SP_VERB_VIEW_MODE_NORMAL,
+    SP_VERB_VIEW_MODE_NO_FILTERS,
     SP_VERB_VIEW_MODE_OUTLINE,
     SP_VERB_VIEW_MODE_TOGGLE,
     SP_VERB_VIEW_CMS_TOGGLE,
@@ -217,19 +231,23 @@ enum {
     SP_VERB_DIALOG_TEXT,
     SP_VERB_DIALOG_XML_EDITOR,
     SP_VERB_DIALOG_FIND,
+    SP_VERB_DIALOG_FINDREPLACE,
+    SP_VERB_DIALOG_SPELLCHECK,
     SP_VERB_DIALOG_DEBUG,
     SP_VERB_DIALOG_SCRIPT,
     SP_VERB_DIALOG_TOGGLE,
     SP_VERB_DIALOG_CLONETILER,
     SP_VERB_DIALOG_ITEM,
-#ifdef WITH_INKBOARD
+/*#ifdef WITH_INKBOARD
     SP_VERB_XMPP_CLIENT,
-#endif
+#endif*/
     SP_VERB_DIALOG_INPUT,
+    SP_VERB_DIALOG_INPUT2,
     SP_VERB_DIALOG_EXTENSIONEDITOR,
     SP_VERB_DIALOG_LAYERS,
     SP_VERB_DIALOG_LIVE_PATH_EFFECT,
     SP_VERB_DIALOG_FILTER_EFFECTS,
+    SP_VERB_DIALOG_SVG_FONTS,
     /* Help */
     SP_VERB_HELP_ABOUT_EXTENSIONS,
     SP_VERB_HELP_MEMORY,
@@ -255,6 +273,12 @@ enum {
     SP_VERB_UNLOCK_ALL_IN_ALL_LAYERS,
     SP_VERB_UNHIDE_ALL,
     SP_VERB_UNHIDE_ALL_IN_ALL_LAYERS,
+    /* Color management */
+    SP_VERB_EDIT_LINK_COLOR_PROFILE,
+    SP_VERB_EDIT_REMOVE_COLOR_PROFILE,
+    /*Scripting*/
+    SP_VERB_EDIT_ADD_EXTERNAL_SCRIPT,
+    SP_VERB_EDIT_REMOVE_EXTERNAL_SCRIPT,
     /* Footer */
     SP_VERB_LAST
 };
