@@ -1,5 +1,5 @@
-#ifndef __SP_COLOR_WHEEL_SELECTOR_H__
-#define __SP_COLOR_WHEEL_SELECTOR_H__
+#ifndef SEEN_SP_COLOR_WHEEL_SELECTOR_H
+#define SEEN_SP_COLOR_WHEEL_SELECTOR_H
 
 #include <glib/gtypes.h>
 #include <gtk/gtktooltips.h>
@@ -25,7 +25,7 @@ public:
     virtual void init();
 
 protected:
-    virtual void _colorChanged( const SPColor& color, gfloat alpha );
+    virtual void _colorChanged();
 
     static void _adjustmentChanged ( GtkAdjustment *adjustment, SPColorWheelSelector *cs );
 
@@ -76,7 +76,7 @@ GtkWidget *sp_color_wheel_selector_new (void);
 
 
 
-#endif
+#endif // SEEN_SP_COLOR_WHEEL_SELECTOR_H
 
 /*
   Local Variables:
